#include <windows.h>
#include <math.h>
#include "trajpol.h"
#include "trajuseq.h"
#include "MenuSpline.h"

/*-------------------------------------------------*/
/* Definition des types POINT, TRAJECTOIRE  etc .. */
/*-------------------------------------------------*/
#define DEGRE1 1
#define DEGRE2 2
#define DEGRE3 3
#define DEGRE4 4
#define DEGRE5 5

#define DELTAU   10.	/* Pas d'integration de l'abscisse curviligne */
#define NPT     200		/* Nb points traces a l'ecran */
#define NBCORD    2		/* Nombre de coordonnees de l'espace */

/* VARIABLES IMPORTEES */
extern int flcont ; /* pour trac ventuel de l'hexagone de prcision */

/* VARIABLES STATIQUES */
static HDC hdc ;
static Seg_Traj *segsel ;
static Traj_List *Traj ;

static int debdef = NDFIC ; /* type debut segment par defaut */
static int findef = NDFIC ; /* type fin segment par defaut */
static int extdef = NDFIC ; /* type extremite segment par defaut */

static int typdeb = NDFIC ;
static int xtdeb, ytdeb ;
static int typfin = NDFIC ;

static int flagc3 = 0 ;
/* ATTENTION ICI FLAG CALCUL EN CURVILIGNE */
static int flabss = 0 ;     /* Flag calcul en curviligne */

static int mode2 = 0 ;
static int mode3 = 0 ;
static int moden = 0 ;
static double ck1N1 = 1., cmuc = 0.5, cliss = 0. ;

// typsplin	:
//	0 : C2
//	1 : C2 liss
//	2 : C3
//	3 : C3 liss

void set_cliss (double cl) 
{ cliss = cl ; }

void set_ck1N1 (double cl)
{ ck1N1 = cl ; }

void set_muc (double cl)
{ cmuc = cl ; }

void set_mode2 (int i)
{
	mode2 = i ;
}

void set_mode3 (int i)
{
	mode3 = i ;
}

void set_moden (int i)
{
	moden = i ;
}

/* Initialise la trajectoire et segment selectionn */
void ini_testspline(void) 
{
     Traj = new_trajlist (NBCORD) ;
	 segsel = NULL ;
}

/* Restitue la mmoire */
void fin_prog(void)
{
    del_trajlist (Traj) ;
}

/* Calcule la trajectoire */
void misajour(void)
{
Seg_Traj *seg, *segp ;

	calc_trajlistc3 (Traj, typdeb, typfin, flagc3, 
		mode2, mode3, moden, ck1N1, cmuc, cliss) ;
	/* Mise en place tangente finale pour tracs */
	if (Traj->Nbpt >= 2)
	{
		seg = Traj->last ;
		segp = seg->prev ;
        d1Mdu_segtraj (&seg->pCoefs[NBCORD], segp->uf, segp) ;
	}
	if (flabss)	long_trajlist (Traj, DELTAU) ;
}

/* Slectionne le sommet le plus proche de x,y */
int proxsom (double *px, double *py)
{
Seg_Traj *seg ;
double dmin, xa, ya, xb, yb, dx, dy, dist ;
int i;
 
    segsel = NULL ;
    xb = *px ; yb = *py ;
    dmin = 1.e+100 ;
    seg = Traj->first ;
	for (i = 0 ; seg != NULL ; i++)
    {
		xa = seg->pMref[0] ;
		ya = seg->pMref[1] ;
        dx = xa - xb ; dy = ya - yb ;
        dist = dx*dx + dy*dy ;
        if (dist < dmin) { dmin = dist ; segsel = seg ; *px = xa ; *py = ya ;}
        seg = seg->next ;
    }
	return segsel != NULL ? 0 : 1 ;
}

/* Slectionne lment le plus proche de x,y, parmi les 2 tangentes et les points */
/* Renvoi : 0 rien trouv
			1 tangente ini
			2 point
			3 tangente fin  */
int selelem (double *px, double *py)
{
Seg_Traj *seg ;
int code ;
double dist, dmin, dx, dy, xm = *px, ym= *py, x, y ;

	if (proxsom(px, py)) return 0 ;
	code = 2 ; /* Point ordinaire slectionn */
	if (Traj->Nbpt < 2) return code ; 
	dx = xm - *px ; dy = ym - *py ; dmin = dx*dx+dy*dy ;
	/* Extrmit tangente initiale */
	seg = Traj->first ;
	x = (int) (seg->pCoefs[0] + 30.*seg->pCoefs[0+NBCORD]) ;
	y = (int) (seg->pCoefs[1] + 30.*seg->pCoefs[1+NBCORD]) ;
	dx = xm - x ; dy = ym - y ; dist = dx*dx+dy*dy ;
	if (dist < dmin) {dmin = dist ; code = 1 ; *px = x ; *py = y; }

	/* Extrmit tangente finale */
	seg = Traj->last ;
	x = (int) (seg->pCoefs[0] + 30.*seg->pCoefs[0+NBCORD]) ;
	y = (int) (seg->pCoefs[1] + 30.*seg->pCoefs[1+NBCORD]) ;
	dx = xm - x ; dy = ym - y ; dist = dx*dx+dy*dy ;
	if (dist < dmin) {code = 3 ; *px = x ; *py = y; }
	/* Tangentes slectionnables que sur les arcs */
	if (code == 1 && Traj->first->type < 0) return 0 ;
	if (code == 3 && Traj->last->prev->type < 0) return 0 ;
	return code ;
}

/* Slectionne le point le plus proche de x,y et le segment portant ce point */
int proxseg(double *px, double *py)
{
Seg_Traj *seg ;
double u, du, dmin, dx, dy, dist, xm, ym, pt[NBCORD] ;
int n;
 
	segsel = Traj->first ;
    if (Traj->Nbpt < 1) return 1 ; 

	if (Traj->Nbpt == 1)
	{
		*px = segsel->pMref[0] ;
		*py = segsel->pMref[1] ;
		return 0 ;
	}

    xm = *px ; ym = *py ;
	dmin = 1.e+100 ;

	if (flabss)	inigens_trajlist (Traj) ;
	else		inigen_trajlist (Traj) ;
    du = flabss ? Traj->Sctot/NPT : Traj->Uctot/NPT ;

    for (n = 0 ; n <= NPT ; n++)
    {
		u = du*n ;
		if (flabss)
		{
			gens_trajlist (pt, u, Traj) ;
			seg = s2seg_trajlist (u, Traj) ;
		}
		else
		{
			gen_trajlist (pt, u, Traj) ;
			seg = u2seg_trajlist (u, Traj) ;
		}
		dx = pt[0] - xm ; dy = pt[1] - ym ;
		dist = dx*dx + dy*dy ;
		if (dist < dmin) 
		{ 
			dmin = dist ; 
			segsel = seg ; 
			*px = pt[0] ; 
			*py = pt[1] ; 
		}
	}
	return 0 ;
 }

/* Changemenet de mode continuite c2 <--> c3 */
void set_continuite(int fc)
{
    flagc3 = fc ;
}

/* Ajout d'un point en tete de la courbe */
void entete(double x, double y)
{
double vbid[NBCORD] ;

	vbid[0] = x ; vbid[1] = y ;
/*    vbid[2]=vbid[3]=0. ; Pour test trajquat  */
	segsel = insseg_trajlist (vbid, Traj->first, Traj) ;
	segsel->erry = 0. ; /* precision parfaite */
	typdeb = debdef ;
	misajour() ;
}

/* Ajout d'un point en queue de la courbe */
void ajoute(double x, double y)
{
double vbid[NBCORD] ;

	vbid[0] = x ; vbid[1] = y ;
/*    vbid[2]=vbid[3]=0. ;  Pour test trajquat */
	segsel = appseg_trajlist (vbid, Traj) ;
	segsel->erry = 0. ; /* precision parfaite */
	if (segsel == Traj->last) typdeb = debdef ;
	typfin = findef ;
	misajour() ;
}


/* Transformation courbe en droite et reciproquement */
int droite(double xs, double ys)
{
	if (Traj->Nbpt < 2) return 1;
    proxseg(&xs, &ys) ;
	if (segsel == NULL || segsel == Traj->last || segsel->type == CERCLE) return 1;

	if (segsel->type == DROITE) 
	{
		segsel->type = INDEF ;
	}
	else segsel->type = DROITE ;
	misajour() ;
	return 0 ;
}

#ifdef ORTHOQ
/* Transformation courbe en ortodromie et reciproquement */
int orthodrom(double xs, double ys)
{
	if (Traj->Nbpt < 2) return 1;
    proxseg(&xs, &ys) ;
	if (segsel == NULL || segsel == Traj->last || segsel->type == CERCLE) return 1;

	if (segsel->type == ORTHOQ) 
	{
		segsel->type = INDEF ;
	}
	else segsel->type = ORTHOQ ;
	misajour() ;
	return 0 ;
}
#endif ORTHOQ

/* Transformation courbe en cercle et reciproquement */
int cercle(double xs, double ys)
{
	if (Traj->Nbpt < 3) return 1;
    proxsom(&xs, &ys) ;

	if (segsel == NULL || segsel == Traj->first || segsel == Traj->last) return 1 ;

	if (segsel->type >0 && segsel->prev->type >=0)
	{ 
		segsel->type = CERCLE ; 
		segsel->prev->type = CERCLE ; 
	}
	else if(segsel->type == CERCLE && segsel->prev->type == CERCLE) 
	{ 
		segsel->type = INDEF ; 
		segsel->prev->type = INDEF ; 
	}
	else return 1 ;
    misajour() ;
	return 0 ;
}

/* Deplacement du point selectionn de la courbe */
void deplace(double x, double y)
{
	if (segsel == NULL) return ;
	if (segsel->pMref[0] == x && segsel->pMref[1] == y) return ;

	segsel->pMref[0] = x ; segsel->pMref[1] = y ;
	misajour() ;
}

/* Modification precision du point selectionn de la courbe */
void precision(double x, double y)
{
double dx, dy, d ;
	if (segsel == NULL) return ;
	dx = segsel->pMref[0] - x ;
	dy = segsel->pMref[1] - y ;
	d = sqrt(dx*dy+dy*dy) ;
	if (segsel->erry == d) return ;
	segsel->erry = d ;

	misajour() ;
}


/* Supprime sommet le plus proche de x,y */
int supprime (double *px, double *py)
{
Seg_Traj *seg ;

    proxsom(px, py) ;
	if (segsel == NULL) return 1 ;
	seg = segsel->next ;
	if (segsel == Traj->first) typdeb = debdef ;
	if (segsel == Traj->last) { seg = segsel->prev ; typfin = findef ;}
	if (segsel->type == CERCLE)
	{
		if (segsel->prev != NULL && segsel->prev->type == CERCLE)
			segsel->prev->type = INDEF ; 
		if (segsel->next != NULL && segsel->next->type == CERCLE)
			segsel->next->type = INDEF ; 
	}
	if (segsel->prev != NULL && segsel->prev->type == CERCLE &&
		segsel->prev->prev != NULL && segsel->prev->prev->type == CERCLE)
	{
		segsel->prev->type = INDEF ; 
		segsel->prev->prev->type = INDEF ; 
	}
	delseg_trajlist (segsel, Traj) ;
	segsel = seg ;
	misajour() ;
	return 0 ;
}

/* Insere un sommet au point le plus proche de x,y */
int insere(double *px, double *py)
{
Seg_Traj *seg ;
double pt[NBCORD] ;

    proxseg(px, py) ;
	if (segsel == NULL || segsel->next == NULL) return 1 ;
	pt[0] = *px ; pt[1] = *py ;
	seg = insseg_trajlist (pt, segsel->next, Traj) ;
	seg->erry = 0. ;
	if (seg == Traj->first) typdeb = debdef ;
	if (segsel->type == CERCLE && seg->next->type == CERCLE)
	{ seg->type = CERCLE ; segsel->type = INDEF ; }
	segsel = seg ;
	misajour() ;	
	return 0 ;
}

/* Initialisation tangente initiale */
void firstseg(int i)
{
	switch (i)
	{
	case TANGTINI :
		typdeb = TGTE ;
		break ;
	case NDFICINI :
		typdeb = NDFIC ;
		break ;
	case CRNULINI :
		typdeb = CRNUL ;
		break ;
	}
}

void tangini(double x, double y)
{
Seg_Traj *segp = Traj->first ;

	if (Traj->Nbpt < 2 || segp->type < 0) return ;
	segp->pCoefs[0+NBCORD] = (x - segp->pCoefs[0])/30. ;
	segp->pCoefs[1+NBCORD] = (y - segp->pCoefs[1])/30. ;
	misajour() ;
}

/* Initialisation tangente finale */
void lastseg(int i)
{
	switch (i)
	{
	case TANGTFIN :
		typfin = TGTE ;
		break ;
	case NDFICFIN :
		typfin = NDFIC ;
		break ;
	case CRNULFIN :
		typfin = CRNUL ;
		break ;
	}
}

void tangfin(double x, double y)
{
Seg_Traj *seg = Traj->last ;

	if (Traj->Nbpt < 2 || seg->prev->type < 0) return ;
	seg->pCoefs[0+NBCORD] = (x - seg->pCoefs[0])/30. ;
	seg->pCoefs[1+NBCORD] = (y - seg->pCoefs[1])/30. ;
	misajour() ;
}

/* Macros de trac */
#define moveto(x,y)       MoveToEx(hdc,(x),(y),NULL)
#define lineto(x,y)       LineTo(hdc,(x),(y))

void trace_croix (int x, int y)
{
	moveto (x-4, y) ; lineto (x+4, y) ;
	moveto (x, y-4) ; lineto (x, y+4) ;
}

void trace_erry(int x, int y, int r) 
{
int a, b ;
   
    if (r < 2) return ;
    a = (int) (0.867*r) ;
    b = r/2  ;
   
	moveto (x+a, y-b) ;
	lineto (x+a, y+b) ;
	lineto (x, y+r) ;
	lineto (x-a, y+b) ;
	lineto (x-a, y-b) ;
	lineto (x, y-r) ;
	lineto (x+a, y-b) ;
}

void tracespline(HDC hdcloc)
{
Seg_Traj *segp = Traj->first, *seg ;
double u, du, pt[NBCORD], ctt, stt, ct, st ;
int x, y, xx, yy, dx, dy, n ;

    if (Traj->Nbpt < 1) return ;
	hdc = hdcloc ;

	xx = (int) segp->pMref[0] ;
	yy = (int) segp->pMref[1] ;

    trace_croix(xx, yy) ; 
	if (flcont == CONT_C2LISS || flcont == CONT_C3LISS)
		trace_erry(xx, yy, (int) segp->erry) ;


    if (Traj->Nbpt < 2) return ;

	if (flabss)	inigens_trajlist (Traj) ;
	else		inigen_trajlist (Traj) ;

    du = flabss ? Traj->Sctot/NPT : Traj->Uctot/NPT ;

	x = (int) segp->pCoefs[0] ;
	y = (int) segp->pCoefs[1] ;
    moveto (x, y) ;

	/* Trac tangente dbut */
	{
		ctt = segp->pCoefs[0+NBCORD] ;
		stt = segp->pCoefs[1+NBCORD] ;
		u = ctt*ctt + stt*stt ; 
		u = sqrt(u) ; 
		ct = ctt/u ; 
		st = stt/u ;
		dx = (int) (8.*ct) ;
		dy = (int) (8.*st) ;
		xx = (int) (segp->pCoefs[0]+30.*ctt) ;
		yy = (int) (segp->pCoefs[1]+30.*stt) ;
		lineto (xx , yy) ;
		lineto (xx-dx-dy,yy-dy+dx) ; 
		moveto (xx, yy) ;
		lineto (xx-dx+dy,yy-dy-dx) ; 
        moveto (x, y) ;
	}

    for (n = 1 ; n <= NPT ; n++)
    {
		u = du*n ;
		if (flabss)
		{
			gens_trajlist (pt, u, Traj) ;
			seg = s2seg_trajlist (u, Traj) ; /* pour les croix */
		}
		else
		{
			gen_trajlist (pt, u, Traj) ;
			seg = u2seg_trajlist (u, Traj) ; /* pour les croix */
		}
		x = (int) pt[0] ;
		y = (int) pt[1] ;
		lineto (x, y) ;

		/* Trac croix au changement de segment */
		if (seg != segp)
		{
			segp = seg ;
			xx = (int) segp->pMref[0] ;
			yy = (int) segp->pMref[1] ;
			trace_croix (xx,yy) ;
			if (flcont == CONT_C2LISS || flcont == CONT_C3LISS)
				trace_erry(xx, yy, (int) segp->erry) ;
			moveto (x, y) ;
		}
    }
    seg = Traj->last ;
	x = (int) seg->pMref[0] ;
	y = (int) seg->pMref[1] ;
    trace_croix(x, y) ;
	if (flcont == CONT_C2LISS || flcont == CONT_C3LISS)
		trace_erry(x, y, (int) seg->erry) ;

	x = (int) seg->pCoefs[0] ;
	y = (int) seg->pCoefs[1] ;
	moveto(x,y) ;	
	/* Trac tangente fin */
	{
		ctt = seg->pCoefs[0+NBCORD] ;
		stt = seg->pCoefs[1+NBCORD] ;
		u = ctt*ctt+stt*stt ; u = sqrt(u) ; ct = ctt/u ; st = stt/u ;
		dx = (int) (8.*ct) ;
		dy = (int) (8.*st) ;
		xx = (int) (seg->pCoefs[0]+30.*ctt) ;
		yy = (int) (seg->pCoefs[1]+30.*stt) ;
		lineto (xx , yy) ;
		lineto (xx-dx-dy,yy-dy+dx) ; 
		moveto (xx, yy) ;
		lineto (xx-dx+dy,yy-dy-dx) ; 
	}
}
