#ifndef trajpolc3_h
#define trajpolc3_h

/*----------------------------------------------------------------------------*/
/* RACCORD c3 entre 2 points par un pol degre 7 (b1,c1,d1 et b2,c2,d2 donnes) */
/*----------------------------------------------------------------------------*/
void tr1p7_c3(double *A1, double d, double *A2, int n) ;


/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 3 points par 2 pols de degr 6 et 5           */
/*             Les coefficients b1,c1,d1 et b3,c3,d3 sont donns             */
/*---------------------------------------------------------------------------*/
int tr2p65_c3 (double *A1, double d1, double *A2, double d2,
                                                        double *A3, int n) ;

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 3 points par 2 pols de degr 5 et 6           */
/*             Les coefficients b1,c1,d1 et b3,c3,d3 sont donns             */
/*---------------------------------------------------------------------------*/
int tr2p56_c3 (double *A1, double d1, double *A2, double d2,
                                                        double *A3, int n) ;

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 4 points par 3 polynomes de degr 5           */
/*             Les coefficients b1,c1,d1 et b4,c4,d4 sont donns             */
/*---------------------------------------------------------------------------*/
int tr3p555_c3 (double *A1, double d1, double *A2, double d2,
                                  double *A3, double d3, double *A4, int n) ;

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 4 points par 3 polynomes de degr 6, 4 et 5   */
/*             Les coefficients b1,c1,d1 et b4,c4,d4 sont donns             */
/*---------------------------------------------------------------------------*/
int tr3p645_c3 (double *A1, double d1, double *A2, double d2,
                                  double *A3, double d3, double *A4, int n) ;

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 4 points par 3 polynomes de degr 5, 4 et 6   */
/*             Les coefficients b1,c1,d1 et b4,c4,d4 sont donns             */
/*---------------------------------------------------------------------------*/
int tr3p546_c3 (double *A1, double d1, double *A2, double d2,
                                  double *A3, double d3, double *A4, int n) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 entre N+1 points par N pols de degr 6,4,...,4,5         */
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*--------------------------------------------------------------------------*/
int trNp645_c3 (double **Coefs, double *pUf, int nbseg, int dimsp) ;


/*--------------------------------------------------------------------------*/
/*      RACCORD c3 entre N+1 points par N pols de degr 5,4,...,4,6         */
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*--------------------------------------------------------------------------*/
int trNp546_c3 (double **Coefs, double *pUf, int nbseg, int dimsp) ;


/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 6												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminCp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp) ;


/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 5												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminCp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 5 et 6 (externes)								*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminCp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double rx) ;

#if 0  /* Supprime */
/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 6 ou 5 selon la valeur de ver					*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int c3splinopt (double **Coefs, double *pUf, int nbseg, int dimsp, int ver) ;
#endif 

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur entre N+1 points			*/
/*       par N pols de degr 6												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminLp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur entre N+1 points			*/
/*       par N pols de degr 6 externes et 5 internes						*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminLp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double rx) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur entre N+1 points			*/
/*       par N pols de degr 5												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminLp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation mixte longueur-courbure entre N+1		*/
/*		points par N pols de degr 6										*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminLCp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double muc) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation mixte longueur-courbure entre N+1		*/
/*		points par N pols de degr 6 externes et 5 internes					*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminLCp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double muc) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation mixte longueur-courbure entre N+1		*/
/*		points par N pols de degr 5										*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminLCp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double muc) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec lissage aux points et minimisation de longueur		*/
/*		et/ou  courbure entre N+1 points par N pols de degr 6				*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminDLCp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, 
				                        double *dM2, double clis, double muc) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec lissage aux points et minimisation de longueur		*/
/*		et/ou  courbure entre N+1 points par N pols de degr 5				*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminDLCp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, 
				                        double *dM2, double clis, double muc) ;

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec lissage aux points et minimisation de longueur		*/
/*		et/ou  courbure entre N+1 points par N pols de degr 6 et 5 int.	*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminDLCp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, 
				                        double *dM, double clis, double muc) ;


#endif
