/****************************************************************************/
/* FICHIER TRAJUSEQ.C : Procedures de calcul de trajectoires sous forme     */
/*                      d'une chaine de segments pouvant comprendre des     */
/*                      droites, arcs de cercles et polynomiales.           */
/****************************************************************************/


// DONNEES MODIFIEES POUR LIBERER LE 4eme vecteur pour la derivee
// troisieme utilisee par les polynomes adjacents pour la continuite c3
// Le rayon et l'angle de l'arc sont repousss dans le 5eme vecteur pour
// CERCLE (decalage dimsp4) et le 6eme vecteur pour ORTHOQ (decalage dimsp5)

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "trajpol.h"
#include "trajuseq.h"
#include "trajpolc3.h"

/*--------------------------------------------------------------------------*/
/*  Declaration de procedures internes                                      */
/*--------------------------------------------------------------------------*/
static void segpol_rac1(Seg_Traj *seg, int typdeb, int typfin) ;
static void segpol_racn(Seg_Traj *seg1, int tdeb, int tfin, int n, double cl) ;

/*--------------------------------------------------------------------------*/
/*             CREATION ET DESTRUCTION D'UN SEGMENT DE TRAJECTOIRE          */
/*--------------------------------------------------------------------------*/
Seg_Traj *new_segtraj (int dimsp, double *pt)
/*--------------------------------------------------------------------------*/
{
/* Alloue une structure Seg_Traj et un buffer pour un pol de degre DEGMAX   */
/* et initialise la structure                                               */
int i ;
Seg_Traj *seg = malloc (sizeof (Seg_Traj)) ;

    if (seg == NULL)
    { puts ("Echec creation segment de trajectoire") ; exit(1) ; }
    /* DEGMAX+1 pour degr 0  DEGMAX et encore +1 pour pMref -> DEGMAX+2 */
    seg->pCoefs = malloc ((DEGMAX+2)*dimsp*sizeof(double) ) ;
    if (seg->pCoefs == NULL)
	{ puts ("Echec creation segment de trajectoire") ; exit(1) ; }
    seg->dimsp = dimsp ;
    seg->type = INDEF ;
    seg->next = NULL ;
    seg->prev = NULL ;
    seg->pMref = seg->pCoefs + (DEGMAX+1)*dimsp ;
	seg->erry = 0. ; /* prcision parfaite a priori */

/* Recopie les coordonnees du point */
    for (i = 0 ; i < dimsp ; i++)
	seg->pMref[i] = pt[i] ;

    return seg ;
}

/*--------------------------------------------------------------------------*/
void del_segtraj (Seg_Traj *seg)
{ free (seg->pCoefs) ; free(seg) ; }
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*                         CREATION D'UNE TRAJECTOIRE                       */
/*--------------------------------------------------------------------------*/
Traj_List *new_trajlist (int dimsp)
/*--------------------------------------------------------------------------*/
{
Traj_List *Traj = malloc (sizeof (Traj_List)) ;

    if (Traj == NULL)
    { puts ("Echec creation trajectoire") ; exit(1) ; }
    Traj->dimsp = dimsp ;
    Traj->Nbpt = 0 ;
    Traj->first = NULL ;
    Traj->last = NULL ;
    return Traj ;
}

/*--------------------------------------------------------------------------*/
/*                      DESTRUCTION TRAJECTOIRE                             */
/*--------------------------------------------------------------------------*/
void del_trajlist (Traj_List *Traj)
/*--------------------------------------------------------------------------*/
{
    vide_trajlist (Traj) ;
    free (Traj) ;
}

/*--------------------------------------------------------------------------*/
/*                      VIDANGE D'UNE TRAJECTOIRE                           */
/*--------------------------------------------------------------------------*/
void vide_trajlist (Traj_List *Traj)
/*--------------------------------------------------------------------------*/
{
Seg_Traj *seg, *segs ;

    if (Traj != NULL && Traj->Nbpt)
    {
	for ( seg = Traj->first ; seg != NULL; )
	{
	    segs = seg->next ;
	    del_segtraj (seg) ;
	    seg = segs ;
	}
	Traj->first = Traj->last = NULL ;
	Traj->Nbpt = 0 ;
    }
}

/*--------------------------------------------------------------------------*/
/*       Ajout d'un segment de trajectoire en queue de la Trajectoire       */
/*--------------------------------------------------------------------------*/
Seg_Traj *appseg_trajlist (double *pt, Traj_List *Traj)
/*--------------------------------------------------------------------------*/
{
Seg_Traj *segp, *seg = new_segtraj (Traj->dimsp, pt) ;

/* Ajoute en queue de la Trajectoire */
    segp = Traj->last ;
    Traj->last = seg ;
    seg->prev = segp ;
    if ( segp != NULL) segp->next = seg ;
    else Traj->first = seg ;
    ++Traj->Nbpt ;
    return seg ;
}

/*--------------------------------------------------------------------------*/
/*         Insertion d'un segment dans la trajectoire avant segs            */
/*--------------------------------------------------------------------------*/
Seg_Traj *insseg_trajlist (double *pt, Seg_Traj *segs, Traj_List *Traj)
/*--------------------------------------------------------------------------*/
{
Seg_Traj *segp, *seg = new_segtraj (Traj->dimsp, pt) ;

/* Insere le point avant le point segsel dans la trajectoire */
    seg->next = segs ;
    if ( segs != NULL)
    {
	segp = segs->prev ;
	segs->prev = seg ;
    }
    else
    {
	segp = Traj->last ;
	Traj->last = seg ;
    }
    seg->prev = segp ;
    if ( segp != NULL) segp->next = seg ;
    else Traj->first = seg ;
    ++Traj->Nbpt ;
    return seg ;
}

/*--------------------------------------------------------------------------*/
/*               Enleve un segment de la liste                              */
/*--------------------------------------------------------------------------*/
void delseg_trajlist (Seg_Traj *seg, Traj_List *Traj)
/*--------------------------------------------------------------------------*/
{
Seg_Traj *segp = seg->prev,  *segs = seg->next ;

    if (segp != NULL) segp->next = segs ;
    else Traj->first = segs ;

    if (segs != NULL) segs->prev = segp ;
    else Traj->last  = segp ;

    --Traj->Nbpt ;
    del_segtraj (seg) ;
}

/*----------------------------------------------------*/
/*  Calcul du point d'un segment situe a l'abscisse u */
/*----------------------------------------------------*/
void ptu_segtraj (double *pt, double u, Seg_Traj *seg)
{
double *coef = seg->pCoefs ;
int dimsp = seg->dimsp, type = seg->type ;

    if (type > 0) ptu_trajpol (pt, u, coef, dimsp, type) ;
    else switch (type)
    {
#ifdef ORTHOQ
	case ORTHOQ :
	    qu_othdrom (pt, u, coef, &coef[4*dimsp], seg->uf,
					coef[5*dimsp], coef[5*dimsp+1]) ;
	    break ;
#endif
	case DROITE :
	{
	int i ;
	    for (i = 0 ; i < dimsp ; i++)
		pt[i] = coef[i] + u * coef[i+dimsp] ;
	}
	break ;
	case CERCLE :
	{
	int i ;
	int ddimsp = dimsp+dimsp, dddimsp = ddimsp + dimsp, dimsp4 = dddimsp+dimsp ;
	double a, msa, mca , af = coef[dimsp4+1] ;
	double u2a = af/seg->uf ;
	    a = u * u2a ;
	    msa = sin (a) / u2a ;
	    mca = 2. * (1. - cos (a))/(u2a*u2a) ;
	    for (i = 0 ; i < dimsp ; i++)
		pt[i] = coef[i] + msa * coef[i+dimsp] + mca * coef[i+ddimsp] ;
	}
	break ;
	default :
	puts ("Type de trajectoire inconnu dans ptu_segtraj") ;
	exit(1) ;
	break ;
    }
}

/*----------------------------------------------------------*/
/* Calcul de la tangente d'un segment au point d'abscisse u */
/*----------------------------------------------------------*/
void d1Mdu_segtraj (double *vec, double u, Seg_Traj *seg)
{
double *coef = seg->pCoefs ;
int dimsp = seg->dimsp, type = seg->type ;

    if (type > 0) d1Mdu_trajpol (vec, u, coef, dimsp, type) ;
    else switch (type)
    {
#ifdef ORTHOQ
    case ORTHOQ :
	d1qu_othdrom (vec, u, coef, &coef[4*dimsp], seg->uf,
					coef[5*dimsp], coef[5*dimsp+1]) ;
	break ;
#endif
    case DROITE :
	{
	int i ;
		for (i = 0 ; i < dimsp ; i++)
		vec[i] = coef[i+dimsp] ;
	}
	break ;
	case CERCLE :
	{
	int i ;
	int ddimsp = dimsp+dimsp, dddimsp = ddimsp + dimsp, dimsp4 = dddimsp+dimsp ;
	double a, tsa, tca, af = coef[dimsp4+1] ;
	double u2a = af/seg->uf;
	    a = u * u2a ;
	    tsa = 2. * sin (a) / u2a ;
	    tca = cos (a) ;
	    for (i = 0 ; i < dimsp ; i++)
		vec[i] = tca * coef[i+dimsp] + tsa * coef[i+ddimsp] ;
	}
	break ;
    default :
	puts ("Type de trajectoire inconnu dans d1Mdu_segtraj") ;
	exit(1) ;
	break ;
    }
}

/*-----------------------------------------------------------------*/
/*    Calcul de derivee seconde d'un segment au point d'abscisse u */
/*-----------------------------------------------------------------*/
void d2Mdu_segtraj (double *vec, double u, Seg_Traj *seg)
{
double *coef = seg->pCoefs ;
int dimsp = seg->dimsp, type = seg->type ;

    if (type > 0) d2Mdu_trajpol (vec, u, coef, dimsp, type) ;
    else switch (type)
    {
#ifdef ORTHOQ
    case ORTHOQ :
	d2qu_othdrom (vec, u, coef, &coef[4*dimsp], seg->uf,
					coef[5*dimsp], coef[5*dimsp+1]) ;
	break ;
#endif
    case DROITE :
	{
	int i ;
	    for (i = 0 ; i < dimsp ; i++) vec[i] = 0. ;
	}
	break ;
    case CERCLE :
	{
	int i ;
	int ddimsp = dimsp+dimsp, dddimsp = ddimsp + dimsp, dimsp4 = dddimsp+dimsp ;
	double a, csa, cca ;
	double af = coef[dimsp4+1] ;
	double u2a = af/seg->uf ;
	    a = u * u2a ;
	    csa = u2a* sin (a) ;
	    cca = 2. * cos (a) ;
	    for (i = 0 ; i < dimsp ; i++)
		vec[i] = cca * coef[i+ddimsp] - csa * coef[i+dimsp] ;
	}
	break ;
    default :
	puts ("Type de trajectoire inconnu dans d2Mdu_segtraj") ;
	exit(1) ;
	break ;
    }
}

/*-------------------------------------------------------------------*/
/*    Calcul de derivee troisieme d'un segment au point d'abscisse u */
/*-------------------------------------------------------------------*/
void d3Mdu_segtraj (double *vec, double u, Seg_Traj *seg)
{
double *coef = seg->pCoefs ;
int dimsp = seg->dimsp, type = seg->type ;

    if (type > 0) d3Mdu_trajpol (vec, u, coef, dimsp, type) ;
    else switch (type)
    {
#ifdef ORTHOQ
    case ORTHOQ :
	d3qu_othdrom (vec, u, coef, &coef[4*dimsp], seg->uf,
					coef[5*dimsp], coef[5*dimsp+1]) ;
	break ;
#endif
    case DROITE :
	{
	int i ;
	    for (i = 0 ; i < dimsp ; i++) vec[i] = 0. ;
	}
	break ;
    case CERCLE :
	{
	int i ;
	int ddimsp = dimsp+dimsp, dddimsp = ddimsp + dimsp, dimsp4 = dddimsp+dimsp ;
	double a, tsa, tca, af = coef[dimsp4+1] ;
	double u2a = af/seg->uf;
	    a = u * u2a ;
	    tsa = -2. * u2a * sin (a) ;
	    tca = -u2a * u2a * cos (a) ;
	    for (i = 0 ; i < dimsp ; i++)
		   vec[i] = tca * coef[i+dimsp] + tsa * coef[i+ddimsp] ;
	}
	break ;
    default :
	puts ("Type de trajectoire inconnu dans d3Mdu_segtraj") ;
	exit(1) ;
	break ;
    }
}

/*-----------------------------------------------------------------*/
/*    Calcul de derivee seconde curviligne au point d'abscisse   u */
/*-----------------------------------------------------------------*/
void d2Mdsu_segtraj (double *vec, double u, Seg_Traj *seg)
{
double *coef = seg->pCoefs ;
int dimsp = seg->dimsp, type = seg->type ;

    if (type > 0) d2Mdsu_trajpol (vec, u, coef, dimsp, type) ;
    else switch (type)
    {
#ifdef ORTHOQ
    case ORTHOQ :
	d2qs_othdrom (vec, u, coef, &coef[4*dimsp], seg->uf,
					coef[5*dimsp], coef[5*dimsp+1]) ;
	break ;
#endif
    case DROITE :
	{ int i ; for (i = 0 ; i < dimsp ; i++) vec[i] = 0. ; }
	break ;
    case CERCLE :
	{
	int i ;
	int ddimsp = dimsp+dimsp , dddimsp = ddimsp + dimsp, dimsp4 = dddimsp+dimsp ;
	double a, csa, cca ;
	double ray = coef[dimsp4], af = coef[dimsp4+1] ;
	double u2a = af/seg->uf, bid ;
	    a = u * u2a ;
	    bid = u2a*ray*ray ;
	    csa = sin (a) / bid ;
	    cca = 2. * cos (a) / (bid*u2a) ;
	    for (i = 0 ; i < dimsp ; i++)
		vec[i] = cca * coef[i+ddimsp] - csa * coef[i+dimsp] ;
	}
	break ;
    default :
	puts ("Type de trajectoire inconnu dans d2Mdsu_segtraj") ;
	exit(1) ;
	break ;
    }
}

/*-----------------------------------------------------------------*/
/*    Calcul de derivee troisieme curviligne au point d'abscisse u */
/*-----------------------------------------------------------------*/
void d3Mdsu_segtraj (double *vec, double u, Seg_Traj *seg)
{
double *coef = seg->pCoefs ;
int dimsp = seg->dimsp, type = seg->type ;

    if (type > 0) d3Mdsu_trajpol (vec, u, coef, dimsp, type) ;
    else switch (type)
    {
#ifdef ORTHOQ
    case ORTHOQ :
	d3qs_othdrom (vec, u, coef, &coef[4*dimsp], seg->uf,
					coef[5*dimsp], coef[5*dimsp+1]) ;
	break ;
#endif
    case DROITE :
	{ int i ; for (i = 0 ; i < dimsp ; i++) vec[i] = 0. ; }
	break ;
    case CERCLE :
	{
	int i ;
	int ddimsp = dimsp+dimsp, dddimsp = ddimsp + dimsp, dimsp4 = dddimsp+dimsp ;
	double a, tsa, tca, ray = coef[dimsp4], af = coef[dimsp4+1] ;
	double u2a = af/seg->uf;
	double ray3asu ;
		a = u * u2a ;
		ray3asu = ray*ray*ray*u2a ;
		tsa = - 2. * sin (a) / (ray3asu*u2a) ;
		tca = - cos (a)/(ray3asu);
		for (i = 0 ; i < dimsp ; i++)
		    vec[i] = tca * coef[i+dimsp] + tsa * coef[i+ddimsp] ;
	}
	break ;
    default :
	puts ("Type de trajectoire inconnu dans d2Mdsu_segtraj") ;
	exit(1) ;
	break ;
    }
}

/* Recherche segment contenant l'abscisse u */
Seg_Traj *u2seg_trajlist (double u, Traj_List *Traj)
{
Seg_Traj *seg = Traj->first, *derseg = Traj->last->prev ;

    while (u > seg->Uci + seg->uf && seg != derseg) seg = seg->next ;
    return seg ;
}

/* Initialisation de la generation des points d'une trajectoire f(u) */
void inigen_trajlist(Traj_List *Traj)
{ Traj->segc = Traj->first ; }

/* Generation des points d'une trajectoire
  !!! Il est obligatoire d'appeler inigen_trajlist avant la generation !!! */
void gen_trajlist(double *pt, double u, Traj_List *Traj)
{
    while (u > Traj->segc->Uci + Traj->segc->uf && Traj->segc != Traj->last->prev)
		Traj->segc = Traj->segc->next ;
    ptu_segtraj (pt, u - Traj->segc->Uci, Traj->segc) ;
}

/* Calcul longueur d'un segment de trajectoire avec le pas duiss */
double long_segtraj (Seg_Traj *seg, double duiss)
{
    if (seg->type > 1)
    {
	seg->duiss = duiss ;
	seg->sf = u2s_trajpol (seg->uf, seg->pCoefs, seg->dimsp, seg->type, (int) (seg->uf/duiss+0.49)) ;
    }
    else switch (seg->type)
    {
    case 1 :
    case DROITE :
	{
	int i, dimsp = seg->dimsp, ddimsp = dimsp+dimsp ; double d = 0. ;
	    for (i = dimsp ; i < ddimsp ; i++) d += seg->pCoefs[i]*seg->pCoefs[i] ;
	    seg->sf = seg->uf * sqrt (d) ;
	}
	break ;
    case CERCLE :
	seg->sf = seg->pCoefs[seg->dimsp*4] * seg->pCoefs[seg->dimsp*4+1] ;
	break ;
#ifdef ORTHOQ
    case ORTHOQ :
    {
	int i, dimsp = seg->dimsp ; 
    double a=0. , b = 0. ;
        for (i = 0 ; i < dimsp ; i++) 
        { 
            a += seg->pCoefs[i]*seg->pCoefs[i] ; 
            b += seg->pCoefs[4*dimsp+i]*seg->pCoefs[4*dimsp+i] ; 
        }
        seg->sf = seg->pCoefs[seg->dimsp*5]*sqrt(sqrt(a*b)) ;
    }
	break ;
#endif
    default :
	puts ("Type de trajectoire non traite dans long_segtraj.");
	exit (1) ;
	break ;
    }
    return seg->sf ;
}

/* Calcul u correspondant a s dans un segment de trajectoire */
double s2u_segtraj (double s, Seg_Traj *seg)
{
double u;

    if (seg->type > 1)
	u = s2u_trajpol (s, seg->uf, seg->pCoefs, seg->dimsp, seg->type, (int) (seg->uf/seg->duiss+0.49)) ;
    else switch (seg->type)
    {
    case 1 :
    case DROITE :
    case CERCLE :
#ifdef ORTHOQ
    case ORTHOQ :
#endif
	u = seg->uf * s / seg->sf ;
	break ;
    default :
	puts ("Type de trajectoire non traite dans u2s_segtraj.");
	exit (1) ;
	break ;
    }
    return u ;
}

/* Calcul longueur curviligne d'une trajectoire avec le pas duiss */
double long_trajlist (Traj_List *Traj, double duiss)
{
Seg_Traj *seg = Traj->first ;
double Sctot = 0. ;

    if (Traj->Nbpt < 2) return 0. ;
    while (seg != Traj->last)
    {
	seg->Sci = Sctot ;
	Sctot += long_segtraj (seg, duiss) ;
	seg = seg->next ;
    }
    Traj->Sctot = Sctot ;
    return Sctot ;
}

/* Recherche segment contenant l'abscisse curviligne s */
Seg_Traj *s2seg_trajlist (double s, Traj_List *Traj)
{
Seg_Traj *seg = Traj->first, *derseg = Traj->last->prev ;

    while (s > seg->Sci + seg->sf && seg != derseg) seg = seg->next ;
    return seg ;
}

/* Calcul u correspondant a s pour une trajectoire */
double s2u_trajlist (double s, Traj_List *Traj)
{
Seg_Traj *seg = s2seg_trajlist (s, Traj) ;
    return  s2u_segtraj (s - seg->Sci, seg) + seg->Uci ;
}

/* Initialisation de la generation des points d'une trajectoire f(s) */
void inigens_trajlist(Traj_List *Traj)
{
    Traj->segc = Traj->first ;
    Traj->u2 = Traj->s2 = Traj->duSURds = 0. ;
    Traj->duiss = Traj->first->duiss ;
}

/* Generation des points d'une trajectoire a partir de l'abscisse curviligne
  !!! Il est obligatoire d'appeler inigens_trajlist avant la generation !!! */
void gens_trajlist(double *pt, double s, Traj_List *Traj)
{
double u, temp ;

    while (s - Traj->segc->Sci > Traj->segc->sf && Traj->segc != Traj->last->prev)
    {
	Traj->segc = Traj->segc->next ;
	/* Initialisations relatives au changement de segment */
	if (Traj->segc->type > 1)
	{   Traj->u2 = Traj->s2 = 0. ; Traj->duiss = Traj->segc->duiss ; }
    }
    s -= Traj->segc->Sci ;

    if (Traj->segc->type > 1)
    {
	/* recherche s1 et s2 tels que s1 < s <= s2 */
	while (s > Traj->s2)
	{
	/* Ajustement du = duiss sauf dernier segment */
	    temp = Traj->segc->uf - Traj->u2 ;   /* Distance a la fin */
	    if (Traj->duiss > temp)                 /* intervalle trop grand ou uc > uf */
		Traj->duiss = temp > 0. ? temp : Traj->segc->duiss ;
	    temp = dsdu_trajpol (Traj->u2 + Traj->duiss/2., Traj->segc->pCoefs, Traj->segc->dimsp, Traj->segc->type) * Traj->duiss ;
	    Traj->duSURds = Traj->duiss / temp ;
	    Traj->s2 += temp ;
	    Traj->u2 += Traj->duiss ;
	}
	u = Traj->u2 - (Traj->s2 - s) * Traj->duSURds ;
    }
    else switch (Traj->segc->type)
    {
	case 1 :
	case DROITE :
	case CERCLE :
#ifdef ORTHOQ
	case ORTHOQ :
#endif
		u = Traj->segc->uf * s / Traj->segc->sf ;
	    break ;
	default :
	    puts ("Type de trajectoire non traite dans gens_trajlist.");
	    exit (1) ;
	    break ;
    }
    ptu_segtraj (pt, u, Traj->segc) ;
}

/*--------------------------------------------------------------------------*/
/*                     UTILITAIRES INTERNES A TRAJLIST.C                    */
/*--------------------------------------------------------------------------*/


/* Calcul d'un segment de trajectoire raccord */
static void segpol_rac1(Seg_Traj *seg, int typdeb, int typfin)
{
// Le cas c3 est traite identique aux deux bouts
   

    switch (typdeb)
    {
    case NDFIC :
    case CRNUL : /* Neant au debut */
		switch (typfin)
		{
		case NDFIC :
		case CRNUL : /* neant debut et fin */
			seg->type = 1 ;
			trajpol_drte00 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp) ;
			return ;
		case 1 : /* neant debut, tgte fin */
			seg->type = 2 ;
			trajpol_bole01 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp);
			return ;
		case 2 : /* neant debut, courbure fin */
			seg->type = 3 ;
			trajpol_cubi02 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp);
			return ;
		case 3 : /* Cas 0 - 3 : On impose des contraintes bidon au debut */
			{
			int dimsp = seg->dimsp, i ;
			Seg_Traj *segs = seg->next ;
			double d, cor = 0., lseg2 = 0. ;
			    // tangente symetrique, derivees 2 et 3 nulles au depart
				for (i = 0 ; i < dimsp; i++)
				{
					d = segs->pCoefs[i] - seg->pCoefs[i] ;
					lseg2 += d*d ;
					cor += d*segs->pCoefs[dimsp+i] ;
				}
				cor *= 2./lseg2 ;
				for (i = 0 ; i < dimsp; i++)
				{ 
					seg->pCoefs[dimsp+i] = - segs->pCoefs[dimsp+i]  
						+ cor*(segs->pCoefs[i] - seg->pCoefs[i]) ;
					seg->pCoefs[dimsp*2+i] = 0. ;
					seg->pCoefs[dimsp*3+i] = 0. ;
				}
			}
			break ;
		}
		break ;
    case 1 : /* Tangente imposee au debut */
		switch (typfin)
		{
		case NDFIC :
		case CRNUL : /* tgte debut , neant fin */
			seg->type = 2 ;
			trajpol_bole10 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp);
			return ;
		case 1 : /* tgte debut , tgte fin */
			seg->type = 3 ;
			trajpol_cubi11 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp);
			return ;
		case 2 : /* tgte debut , courbure fin */
			seg->type = 4 ;
			trajpol_quad12 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp);
			return ;
		case 3 : /* Cas 1 - 3 : On impose des contraintes bidon au debut */
			{
			int dimsp = seg->dimsp, i ;
			    // derivees 2 et 3 nulles au depart
				for (i = 0 ; i < dimsp; i++)
				{ 
					seg->pCoefs[dimsp*2+i] = 0. ;
					seg->pCoefs[dimsp*3+i] = 0. ;
				}
			}
			break ;
		}
		break ;
    case 2 : /* courbure debut */
		switch (typfin)
		{
		case NDFIC :
		case CRNUL : /* courbure debut, neant fin */
			seg->type = 3 ;
			trajpol_cubi20 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp);
			return ;
		case 1 : /* courbure debut, tgte fin */
			seg->type = 4 ;
			trajpol_quad21 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp) ;
			return ;
		case 2 : /* courbure debut, courbure fin */
			seg->type = 5 ;
			trajpol_five22 (seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp) ;
			return ;
		case 3 : /* Cas 2 - 3 : On impose des contraintes bidon au debut */
			{
			int dimsp = seg->dimsp, i ;
			    // derivees 3 nulle au depart
				for (i = 0 ; i < dimsp; i++)
				{ 
					seg->pCoefs[dimsp*3+i] = 0. ;
				}
			}
			break ;
		}
		break ;
	case 3 : /* C3 au debut */
		switch (typfin)
		{
		case NDFIC :
		case CRNUL : /* Cas 3 - 0 : On impose des contraintes bidon a la fin */
			{ 
			int dimsp = seg->dimsp, i ;
			Seg_Traj *segs = seg->next ;
			double d, cor = 0., lseg2 = 0. ;
			    // tangente symetrique, derivees 2 et 3 nulles a la fin
				for (i = 0 ; i < dimsp; i++)
				{
					d = segs->pCoefs[i] - seg->pCoefs[i] ;
					lseg2 += d*d ;
					cor += d*seg->pCoefs[dimsp+i] ;
				}
				cor *= 2./lseg2 ;
				for (i = 0 ; i < dimsp; i++)
				{ 
					segs->pCoefs[dimsp+i] = - seg->pCoefs[dimsp+i]  
						 + cor*(segs->pCoefs[i] - seg->pCoefs[i]) ;
					segs->pCoefs[dimsp*2+i] = 0. ;
					segs->pCoefs[dimsp*3+i] = 0. ;
				}
			}
			break ;
		case 1 : /* Cas 3 - 1 : On impose des contraintes bidon a la fin */
			{
			int dimsp = seg->dimsp, i ;
			Seg_Traj *segs = seg->next ;
			    // derivees 2 et 3 nulles a la fin
				for (i = 0 ; i < dimsp; i++)
				{ 
					segs->pCoefs[dimsp*2+i] = 0. ;
					segs->pCoefs[dimsp*3+i] = 0. ;
				}
			}
			break ;
		case 2 : /* Cas 3 - 2 : On impose des contraintes bidon a la fin */
			{
			int dimsp = seg->dimsp, i ;
			Seg_Traj *segs = seg->next ;
			    // derivees 3 nulle a la fin
				for (i = 0 ; i < dimsp; i++)
				{ 
					segs->pCoefs[dimsp*3+i] = 0. ;
				}
			}
			break ;
		case 3 :
			break ;
		}
		break ;
    }
	seg->type = 7 ;
	tr1p7_c3(seg->pCoefs, seg->uf, seg->next->pCoefs, seg->dimsp) ;
}

/* Calcul de n segments de trajectoire raccord */
char *texter[3]= {"erreur donnes : nbseg < ?",
		        "echec allocation",
		        "echec resolution LU" } ;


static void segpol_racnc3(Seg_Traj *seg1, int tdeb, int tfin, int n, int typsplin,
						  	int mode2, int mode3, int moden, 
							double ck1N1, double cmuc, double cliss)
{
Seg_Traj *seg, *seg2 = seg1 ;
double **A, *d, *T, *M, *P, dd, *dM ;
int i, k, nfig, dimsp = seg1->dimsp, resnok=0 ;

    if (tdeb >= 2) n++ ; /* Le 1er n'etait pas compte */
    if (tfin >= 2) n++ ; /* Le dernier n'etait pas compte */

    A = (double **) malloc ((n+1)*sizeof(double *)) ;
    d = (double *) malloc ((n+1)*sizeof(double)) ;

	/* Initialisation points et etendues parametriques des n segments */
    for (i = 0 ; i < n ; i++)
    { A[i] = seg2->pCoefs ; d[i] = seg2->uf ; seg2 = seg2->next ; }
	A[n] = seg2->pCoefs ;

	if (typsplin==1 || typsplin==3) /* Recupere precision de passage au point */
	{
		dM = (double *) malloc ((n+1)*sizeof(double)) ;
		seg = seg1 ;
		for (i = 0 ; i <= n ; i++)
		{ dM[i] = seg->erry ; seg = seg->next ; }
	}


	/* Cas des splines classiques en continuit C2 */
	if (tdeb < 3 && tfin < 3 && typsplin < 2)
	{
		seg1->type = (tdeb > 1)? 4 : 3 ;
		seg = seg1->next ;
		for (i = 1 ; i < n-1 ; i++) { seg->type = 3 ; seg = seg->next ; }
		seg->type = (tfin > 1)? 4 : 3;
		if (typsplin == 0)
		    trisplin (A, d, tdeb, tfin, n, dimsp) ;
		else
		    lispline (A, d, tdeb, tfin, n, dimsp, dM, cliss) ;
		free (A) ;
		free (d) ;
		return ;
	}

	/* Cas des splines en continuit C3 */
	/* Raz vecteurs courbure et deriv3 non fournies */ 
	for (k = (tdeb >= 1) ? tdeb+1 : 2 ; k < 4; k++)
	{ T = seg1->pCoefs + dimsp*k ; for (i = 0 ; i < dimsp; i++) T[i] = 0. ; }

	for (k = (tfin >= 1) ? tfin+1 : 2 ; k < 4; k++)
	{ T = seg2->pCoefs + dimsp*k ; for (i = 0 ; i < dimsp; i++) T[i] = 0. ; }
	

	/* Initialisation tangentes debut et/ou fin si necessaire */
	if (tdeb < 1 && seg1->type == INDEF) /* tdeb a initialiser */
	{
		M = seg1->pCoefs ; P = seg1->next->pCoefs ; 
		T = M + dimsp ; dd = seg1->uf ;
		for (i = 0 ; i < dimsp; i++) T[i] = (P[i]-M[i])/dd ;
	}

    if (tfin < 1 /* && seg2->prev->type == INDEF*/) /* tfin a initialiser */
	{
		M = seg2->pCoefs ; P = seg2->prev->pCoefs ; 
		T = M + dimsp ; dd = seg2->prev->uf ;
		for (i = 0 ; i < dimsp; i++) T[i] = -(P[i]-M[i])/dd ;
	}
	/* Traitement suivant nombre de segments de la trajectoire */
	nfig = n ;
	if (n == 2 && mode2 == 0) nfig = 4 ;
	if (n == 3 && mode3 == 0) nfig = 4 ;
	switch (nfig)
	{
	case 1 : /* Un seul segment */
        segpol_rac1(seg1, tdeb, tfin) ;
		break ;
	case 2 : /* Deux segments */
		/* Traitement suivant mode2 : degres 56 ou 65 */
		switch (mode2)
		{
		case 1 :
			seg1->type = 5 ;
			seg2->prev->type = 6 ;
			tr2p56_c3 (seg1->pCoefs, seg1->uf, seg1->next->pCoefs, seg1->next->uf,
								seg2->pCoefs, dimsp) ;
			break ;
		case 2 :
			seg1->type = 6 ;
			seg2->prev->type = 5 ;
			tr2p65_c3 (seg1->pCoefs, seg1->uf, seg1->next->pCoefs, seg1->next->uf,
								seg2->pCoefs, dimsp) ;
			break ;
		}
		break ;
	case 3 : /* Trois segments */
		/* Traitement suivant mode 3 : degre 546, ou 555 ou 645 */
		switch (mode3)
		{
		case 1 :
			seg1->type = 5 ;
			seg1->next->type = 4 ;
			seg2->prev->type = 6 ;
			tr3p546_c3 (seg1->pCoefs, seg1->uf, seg1->next->pCoefs, seg1->next->uf,
								seg2->prev->pCoefs, seg2->prev->uf, seg2->pCoefs, dimsp) ;
			break ;
		case 2 :
			seg1->type = 5 ;
			seg1->next->type = 5 ;
			seg2->prev->type = 5 ;
			tr3p555_c3 (seg1->pCoefs, seg1->uf, seg1->next->pCoefs, seg1->next->uf,
								seg2->prev->pCoefs, seg2->prev->uf, seg2->pCoefs, dimsp) ;
			break;
		case 3 :
			seg1->type = 6 ;
			seg1->next->type = 4 ;
			seg2->prev->type = 5 ;
			tr3p645_c3 (seg1->pCoefs, seg1->uf, seg1->next->pCoefs, seg1->next->uf,
								seg2->prev->pCoefs, seg2->prev->uf, seg2->pCoefs, dimsp) ;
			break ;
		}
		break ;
	default : 
		/* traitement suivant moden : 
		   degre 64.45, 54.46, 66.66opt, 55.55opt, 65.56opt ..........
		*/
		k = moden ;
		/* cas 645 et 546 : n doit etre >= 3 */
		if ((moden == 0 || moden == 1) && n < 3) 
			k = 2 ;	/* trait en 6C */

		/* cas 5C, 5L ou 5LC : n doit etre >= 4 */
		if (moden == 3 || moden == 6 || moden == 9)
		{
			if(n==3) k = moden+1 ; /* trait en 656 */
			if(n<=2) k = moden-1 ; /* trait en 6 */
		}

		/* cas 656C, 656L ou 656LC : n doit etre >= 3 */
		if ((moden == 4 || moden == 7 || moden == 10) && n < 3) 
			k = moden-2 ; /* trait en 6 */

		switch (k)
		{
		case 0 : /* cas degre 64...45 */
			seg1->type = 6 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 4 ; seg = seg->next ; }
			seg->type = 5 ;
			resnok = trNp645_c3 (A, d, n, dimsp) ;
			break ;
		case 1 : /* cas degre 54...46 */
			seg1->type = 5 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 4 ; seg = seg->next ; }
			seg->type = 6 ;
			resnok = trNp546_c3 (A, d, n, dimsp) ;
			break ;
		case 2 : /* cas degre 66...66 avec optimisation courbure */
			seg1->type = 6 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 6 ; seg = seg->next ; }
			seg->type = 6 ;
			resnok = trNminCp6_c3 (A, d, n, dimsp) ;
			break ;
		case 3 : /* cas degre 55...55 avec optimisation courbure */
			seg1->type = 5 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 5 ; seg = seg->next ; }
			seg->type = 5 ;
			resnok = trNminCp5_c3 (A, d, n, dimsp) ;
			break ;
		case 4 : /* cas degre 65...56 avec optimisation courbure */
			seg1->type = 6 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 5 ; seg = seg->next ; }
			seg->type = 6 ;
			resnok = trNminCp656_c3 (A, d, n, dimsp, ck1N1) ;
			break ;
		case 5 : /* cas degre 66...66 avec optimisation longueur */
			seg1->type = 6 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 6 ; seg = seg->next ; }
			seg->type = 6 ;
			resnok = trNminLp6_c3 (A, d, n, dimsp) ;
			break ;
		case 6 : /* cas degre 55...55 avec optimisation longueur */
			seg1->type = 5 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 5 ; seg = seg->next ; }
			seg->type = 5 ;
			resnok = trNminLp5_c3 (A, d, n, dimsp) ;
			break ;
		case 7 : /* cas degre 65...56 avec optimisation longueur */
			seg1->type = 6 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 5 ; seg = seg->next ; }
			seg->type = 6 ;
			resnok = trNminLp656_c3 (A, d, n, dimsp, ck1N1) ;
			break ;
		case 8 : /* cas degre 6 avec optimisation mixte longueur-courbure */
			seg1->type = 6 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 6 ; seg = seg->next ; }
			seg->type = 6 ;
			if (typsplin == 1 || typsplin == 3) 
				resnok = trNminDLCp6_c3 (A, d, n, dimsp, dM, cliss, cmuc) ;
			else 
				resnok = trNminLCp6_c3 (A, d, n, dimsp, cmuc) ;
			break ;
			break ;
		case 9 : /* cas degre 5 avec optimisation mixte longueur-courbure */
			seg1->type = 5 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 5 ; seg = seg->next ; }
			seg->type = 5 ;
			if (typsplin == 1 || typsplin == 3) 
				resnok = trNminDLCp5_c3 (A, d, n, dimsp, dM, cliss, cmuc) ;
			else 
				resnok = trNminLCp5_c3 (A, d, n, dimsp, cmuc) ;
			break ;
		case 10 : /* cas degre 65..6 avec optimisation mixte longueur-courbure */
			seg1->type = 6 ;
			seg = seg1->next ;
			for (i = 1 ; i < n-1 ; i++) { seg->type = 5 ; seg = seg->next ; }
			seg->type = 6 ;
			if (typsplin == 1 || typsplin == 3) 
				resnok = trNminDLCp656_c3 (A, d, n, dimsp, dM, cliss, cmuc) ;
			else 
				resnok = trNminLCp656_c3 (A, d, n, dimsp, cmuc) ;
			break ;
		}
	}
	free (A) ;
    free (d) ;
	if (resnok)
	{ puts ("Erreur calcul trajectoire") ; exit(1) ; }

}

/*--------------------------------------------------------------------------*/
/*  CALCUL D'UNE TRAJECTOIRE SOUS FORME DE LISTE DE SEGMENTS                */
/*--------------------------------------------------------------------------*/
void calc_trajlistc3 (Traj_List *Traj, int typdeb, int typfin, int typsplin,
					  		int mode2, int mode3, int moden, 
							double ck1N1, double cmuc, double cliss)
/*--------------------------------------------------------------------------*/
{
Seg_Traj *segp, *seg, *segs, *seg1, *seglast ;
int i, n, dimsp = Traj->dimsp ;
int ddimsp = dimsp + dimsp, dimsp3 = ddimsp+dimsp, dimsp4 = dimsp3+dimsp ;
int *kdeb ;     /* Indicateur du nb de conditions imposees au point */
double Uf, bid, *W ;
int makesplin, nsplin ;

    if (Traj->Nbpt < 2) return ;
    seglast = Traj->last->prev ;

/* Allocation et initialisation des contraites aux points */
    kdeb = malloc (Traj->Nbpt*sizeof(int)) ;
    kdeb[0] = typdeb ;
    for (i = 1 ; i < Traj->Nbpt-1 ; i++) kdeb[i] = 0 ;
    kdeb[Traj->Nbpt-1] = typfin ;

/* Mise en place des points de passage */
    seg = Traj->first ;
    for (n = 0 ; n < Traj->Nbpt ; n++)
    {
	for (i = 0 ; i < dimsp ; i++) seg->pCoefs[i] = seg->pMref[i] ;
	seg = seg->next ;
    }

/* 1er passage : calcul des valeurs de Uci, Uf, Uctot et des droites rang n
   des cerlces n, n+1 et des orthodromies                                 */
    seg = Traj->first ; Traj->Uctot = 0. ;
    for (n = 0 ; seg != Traj->last ; n++)
    {
		segs = seg->next ;
		Uf = 0. ;
		for (i = 0 ; i < dimsp ; i++)
		{
			bid = segs->pCoefs[i] - seg->pCoefs[i] ;
			Uf += bid * bid ;
		}
		Uf = sqrt (Uf) ;
		if (seg->type < 0) switch (seg->type) /* types imposs */
		{
		case DROITE :
		/* Calcul vecteur tangent */
			for (i = 0 ; i < dimsp ; i++)
			{
				seg->pCoefs[i+dimsp] =
					(segs->pCoefs[i] - seg->pCoefs[i])/Uf ;
				seg->pCoefs[i+ddimsp] = 0. ; /* fixe courbure debut */
			    seg->pCoefs[i+dimsp3] = 0. ; /* fixe derivee troisieme debut */
			}
		/* Contraint segment suivant */
			for (i = 0 ; i < dimsp ; i++)
			{
				segs->pCoefs[i+dimsp] = seg->pCoefs[i+dimsp] ; /* fixe tgte fin */
				segs->pCoefs[i+ddimsp] = 0. ; /* fixe courbure fin */
			    segs->pCoefs[i+dimsp3] = 0. ; /* fixe derivee troisieme fin */
			}
			kdeb[n]   = typsplin >= 2 ? 3 : 2 ;
			kdeb[n+1] = typsplin >= 2 ? 3 : 2 ;
			break ;
		case CERCLE :
		/* !!!  PAR HYPOTHESE : CERCLE DEFINI PAR 2 SEGMENTS SUCCESSIFS !!! */
		{
		double ufa, ufb, cosa, cosb, ra, rb, arca, arcb, dcosa, ah, temp ;
		double *Ua = &seg->pCoefs[ddimsp], *Ub = &segs->pCoefs[ddimsp] ;
		double *Ta = &seg->pCoefs[dimsp], *Tb = &segs->pCoefs[dimsp] ;

			ufa = Uf ;               /* Pour le moment */
		/* Calcule corde segment suivant */
			ufb = 0. ;
			for (i = 0 ; i < dimsp ; i++)
			{
			bid = segs->next->pCoefs[i] - segs->pCoefs[i] ;
			ufb += bid * bid ;
			}
			ufb = sqrt (ufb) ;      /* Pour le moment */

		/* Calcul cordes unitaires */
			bid = cosa = cosb = 0. ;
			for (i = 0 ; i < dimsp ; i++)
			{
			Ua[i] = (segs->pCoefs[i] - seg->pCoefs[i])/ufa ;
			Ub[i] = (segs->next->pCoefs[i] - segs->pCoefs[i])/ufb ;
			Tb[i] = ufb * Ua[i] + ufa * Ub[i] ;
			/* Longueur Tb */
			bid += Tb[i] * Tb[i] ;
			/* Produit scalaire Ua * Tb et Ub * Tb */
			cosa += Ua[i] * Tb[i] ;
			cosb += Ub[i] * Tb[i] ;
			}
			bid = sqrt (bid) ; cosa /= bid ; cosb /= bid ;
			temp = 2.*cosa ; dcosa = ufa * cosa ;
			ah = 0. ;
			for (i = 0 ; i < dimsp ; i++)
			{
			/* Vecteurs unitaires tangents au cercle */
			Tb[i] /= bid ;                        /* Tb unitaire */
			Ta[i] =  temp * Ua[i] - Tb[i] ;       /* Ta unitaire */
			Ua[i] =  ufa * Ua[i] - dcosa * Ta[i] ; /* vecteur AH dans Ua */
			ah += Ua[i] * Ua[i] ;
			}
			/* Rayons et arcs */
			ah = sqrt (ah) ;
			ra = rb = 0.5 * ufa * ufa / ah ;
			arca = 2. * acos (cosa) ;
			arcb = 2. * acos (cosb) ;

		/* Calcul AC et BC */
			temp = ra / ah ;
			for (i = 0 ; i < dimsp ; i++)
			{
			   Ua[i] *= temp ;  /* AC */
			   /* BC = BA + AC */
			   Ub[i] = seg->pCoefs[i] - segs->pCoefs[i] + Ua[i] ;
			}
		/* Modification du choix des Uf pris egaux a la longueur des arcs */
			ufa = ra * arca ; ufb = rb * arcb ; /* ligne facultative */
		/* Mise en place coefficients B et C */
			bid = arca / ufa ;
			temp = 0.5 * bid * bid ;
			bid *= ra ;
			for (i = 0 ; i < dimsp ; i++)
			{
			   Ta[i] *= bid ;  /* B = (R.arca/Ufa).Ta */
			   Ua[i] *= temp ;  /* C = 0.5 (arca/Ufa)2. AC    */
			}
			bid = arcb / ufb ;
			temp = 0.5 * bid * bid ;
			bid *= rb ;
			for (i = 0 ; i < dimsp ; i++)
			{
			   Tb[i] *= bid ;
			   Ub[i] *= temp ;
			}
		/* Mise en place alfa et Rayon */
			seg->pCoefs[dimsp4] = ra ;     segs->pCoefs[dimsp4] = rb ;
			seg->pCoefs[dimsp4+1] = arca ; segs->pCoefs[dimsp4+1] = arcb ;
		/* Contraint segment precedent */
			W = seg->pCoefs + dimsp3 ;
			d3Mdu_segtraj (W, 0., seg) ; for (i = 0 ; i < dimsp ; i++) W[i] /= 6. ;
			kdeb[n] = typsplin >= 2 ? 3 : 2 ;
		/* Contraint segment suivant */
			segs->uf = ufb ; /* Anticipation ncessaire pour les 2 routines qui suivent */
			d1Mdu_segtraj (segs->next->pCoefs+dimsp, ufb, segs) ;
			W = segs->next->pCoefs + ddimsp ;
			d2Mdu_segtraj (W, ufb, segs) ; for (i = 0 ; i < dimsp ; i++) W[i] /= 2. ;
			W = segs->next->pCoefs + dimsp3 ;
			d3Mdu_segtraj (W, ufb, segs) ; for (i = 0 ; i < dimsp ; i++) W[i] /= 6. ;
			kdeb[n+2] = typsplin >= 2 ? 3 : 2 ;
		/* Saute calcul segment suivant car deja fait */
			seg->uf = ufa ;
			seg->Uci = Traj->Uctot ;
			Traj->Uctot += seg->uf ;
			Uf = ufb ;

			seg = segs ;
			segs = seg->next ;
			n++ ;
		}
			break ;
#ifdef ORTHOQ
		case ORTHOQ :
		{
		double ts2, sts2 ;
			if (dimsp != 4)
			{
			puts ("Orthodromie en quaternion en dimension 4 uniquement !") ;
			break ;
			}
			ts2 = ang_othdrom (seg->pCoefs, segs->pCoefs) ;
			sts2 = sin (ts2) ;
		/* Calcul vecteur tangent et courbure initiaux et finaux */
			d1qu_othdrom (&seg->pCoefs[dimsp], 0.,
					seg->pCoefs, segs->pCoefs, Uf, ts2, sts2) ;
			d2qu_othdrom (&seg->pCoefs[ddimsp], 0.,
					seg->pCoefs, segs->pCoefs, Uf, ts2, sts2) ;
			d3qu_othdrom (&seg->pCoefs[dimsp3], 0.,
						seg->pCoefs, segs->pCoefs, Uf, ts2, sts2) ;

			d1qu_othdrom (&segs->pCoefs[dimsp], Uf,
					seg->pCoefs, segs->pCoefs, Uf, ts2, sts2) ;
			d2qu_othdrom (&segs->pCoefs[ddimsp], Uf,
					seg->pCoefs, segs->pCoefs, Uf, ts2, sts2) ;
			d3qu_othdrom (&segs->pCoefs[dimsp3], Uf,
						seg->pCoefs, segs->pCoefs, Uf, ts2, sts2) ;
			for (i = 0 ; i < 4 ; i++)
			{
				seg->pCoefs[i+ddimsp] /= 2. ; /* 1/2 de d2M/du2 en C */
				seg->pCoefs[i+dimsp3] /= 6. ; /* 1/6 de d3M/du3 en C */

				seg->pCoefs[i+dimsp4] = segs->pCoefs[i] ; /* point final */


				segs->pCoefs[i+ddimsp] /= 2. ; /* 1/2 de d2M/du2 en C */
				segs->pCoefs[i+dimsp3] /= 6. ; /* 1/6 de d3M/du3 en C */
			}
			seg->pCoefs[dimsp4+dimsp] = ts2 ;  /* Longueur */
			seg->pCoefs[dimsp4+dimsp+1] = sts2 ; /* Sinus longueur */

			kdeb[n] = typsplin >= 2 ? 3 : 2 ;   /* tgte et courbure imposees au debut */
			kdeb[n+1] = typsplin >= 2 ? 3 : 2 ; /* tgte et courbure imposee a la fin */
		}
        break ;
#endif
		default :
			puts ("Type d'arc de trajectoire non traite par calc_trajlist !") ;
			break ;
		}
		seg->uf = Uf ;
		seg->Uci = Traj->Uctot ;
		Traj->Uctot += Uf ;
		seg = segs ;
	}

/* Deuxieme passage : Examen des splines */
/* seg1 pointera sur le debut d'un raccord par spline
/* nsplin compte le nombre de segments splines sauf 1er et dernier si contraintes > 1 :
	 si <  0  pas encore commence
	 si == 0  trouve un premier raccord avec contrainte C2 ou C3
	 si >0    trouve au moins un raccord ordinaire 
*/
    seg = NULL ;
    segs = Traj->first ;
    nsplin = -1 ;
    makesplin = 0 ;
    for (n = 0 ; segs != Traj->last ; n++)
    {
		segp = seg ;
		seg = segs ;
		segs = seg->next ;
		if (seg->type >= 0)
		{ /* Cas raccord */
			if (nsplin <= 0) /* Si pas encore trouve de raccord ordinaire de degr 3 */
			{ /* cad soit 1er raccord, soit trouve une quadrique */
				switch (kdeb[n]) 
				{
				case CRNUL :
				case NDFIC : /* pas de contrainte spciale au debut */
					switch (kdeb[n+1])
					{
					case CRNUL :
					case NDFIC : /* pas de contrainte speciale a la fin */
					  if (seg == seglast && nsplin < 0)
						  /* segment isole 0-0 : segpol_rac1 fixe seg->type */
						segpol_rac1 (seg, 0, 0) ;
					  else /* Ce n'est pas un segment unique */
					  {
						  if (nsplin < 0) /* C'est le premier */
							  seg1 = seg ;
						  nsplin = 1 ; 
						  if (seg == seglast) /* C'est le dernier */
							  makesplin = 1 ;
					  }
					  break ;
					case 1 :
					  if (nsplin == 0) /* quadrique + 1 raccord spline */
					  {
						  nsplin = 1 ; makesplin = 1 ;
					  }
					  else 
						  segpol_rac1 (seg, 0, 1) ;
					  break ;
					case 2 :
					case 3 :
					  if (nsplin == 0)  /* 2 segments contraints c2 ou c3 */
					  { 
						  nsplin = 2 ;
						  if (kdeb[n-1] >= 2) nsplin-- ; /* Le 1er n'est pas compte */
						  if (kdeb[n+1] >= 2) nsplin-- ; /* Le dernier n'est pas compte */
				          segpol_racnc3 (seg->prev, kdeb[n-1], kdeb[n+1], nsplin, typsplin,
							  					  mode2, mode3, moden, ck1N1, cmuc, cliss) ;
						  nsplin = -1 ; 
					  }
					  else 
						  segpol_rac1 (seg, 0, kdeb[n+1]) ;
					  break ;
					} // sortie du switch (kdeb[n+1])
				break ;
				case 1 : /* c'est forcement kdeb[0] = 1 */
					if ((kdeb[n+1] == NDFIC || kdeb[n+1] == CRNUL)
									 && seg != seglast)
					{ /* Debut de spline par raccord */
						seg1 = seg ;
						nsplin = 1 ;
						typdeb = 1 ;
					}
					else 
						segpol_rac1(seg, 1, kdeb[n+1]) ;
					break ;
				case 2 :
				case 3 :
					if ((kdeb[n+1] == NDFIC || kdeb[n+1] == CRNUL)
									 && seg != seglast)
					{ /* Debut de spline par raccord type 2 ou 3 */
						seg1 = seg ;
						nsplin = 0 ;
						typdeb = kdeb[n] ;
					}
					else 
						segpol_rac1(seg, kdeb[n], kdeb[n+1]) ;
					break ;
				}
			}
			else /* Cas ou nsplin > 0 */
			{ /* elements suivant de la spline */
				switch (kdeb[n+1]) 
				{
				case CRNUL :
				case NDFIC :
					nsplin++ ; 
					if (seg == seglast)
					{
					   if ((typdeb == NDFIC || typdeb == CRNUL) && nsplin == 2)
					   /* Cas particulier double parabole 0-0 valable en c2 et c3 */
					   { 
						  segp->type = 2 ;
						  seg->type = 2 ;
						  trajpol_2boles00 (segp->pCoefs, segp->uf, seg->pCoefs, seg->uf,
								seg->next->pCoefs, segp->dimsp) ;
						  nsplin = -1 ; 
					   }
					   else makesplin = 1 ;
					}
					break ;
				case 1 : /* Fin spline x-1 */
					nsplin++ ; makesplin = 1 ;
					break ;
				case 2 : /* Fin d'une spline par contraint >= 2 */
				case 3 :
					makesplin = 1 ;
					break ;
				} /* Fin switch examen kdeb fin segment */
			} /* fin else du if nsplin == 0 */
			if (makesplin)
			{
				segpol_racnc3 (seg1, typdeb, kdeb[n+1], nsplin, typsplin,
							  	mode2, mode3, moden, ck1N1, cmuc, cliss) ;
				makesplin = 0 ; nsplin = -1 ;
			}
		} /* fin examen d'un raccord */
    } /* fin boucle d'examen des raccords */
    free (kdeb) ;
}

