/****************************************************************************/
/* FICHIER TRAJUSEQ.H : A inclure pour utiliser les procdures de calcul    */
/*                      de trajectoires organises sous forme de listes     */
/*                      chainees de segments.                               */
/****************************************************************************/

#ifndef trajuseq_h
#define trajuseq_h

#define INDEF   0
#define DROITE -1
#define CERCLE -2

/* NE PAS DEFINIR SI QUATERNIONS INUTILISES 
#define ORTHOQ  -3 */

#ifdef ORTHOQ
#include "trajquat.h"
#endif

/*--------------------------------------------------------------------------*/
/*                      STRUCTURE SEGMENT DE TRAJECTOIRE                    */
/*--------------------------------------------------------------------------*/
typedef struct seg_traj Seg_Traj ;
struct seg_traj
{
	Seg_Traj *prev ;         /* Segment precedent                */
	Seg_Traj *next ;         /* Segment suivant                  */
	int dimsp ;		        /* Dimension trajectoire            */
	int type ;		        /* Type du segment ou degre         */
	double Uci ;		        /* Valeur initiale parametre        */
	double uf ;	     	        /* Variation parametre              */
	double *pCoefs ;	                /* Adresse des coefs                */
/* Si utilisation abscisse curviligne */
	double duiss ;		        /* Pas d'integration de la longueur */
	double Sci ;                     /* Abscisse curviligne initiale     */
	double sf ;                     /* Longueur curviligne segment      */
/* Si utilisation splines lissantes */
	double *pMref ;			/* Point de reference */
	double erry ;			/* Precision de passage au point */
} ;


/*--------------------------------------------------------------------------*/
/*                        STRUCTURE TRAJECTOIRE                             */
/*--------------------------------------------------------------------------*/
typedef struct
{
	Seg_Traj *first ;	       /* Premier segment de la trajectoire */
	Seg_Traj *last  ;	       /* Dernier segment "fictif"          */
	double Uctot ;	               /* Longueur curviligne totale        */
	int Nbpt ;	               /* Nombre de points                  */
	int dimsp ;		       /* Dimension de la trajectoire       */
/* Pour generation */
	Seg_Traj *segc ;	       /* Segment en cours de generation    */
/* Si utilisation abscisse curviligne */
	double Sctot ;                  /* Longueur curviligne segment       */
	double s2 ;		       /* Absc. curv. fin d'interpolation   */
	double u2 ;		       /* Absc. para. fin d'interpolation   */
	double duSURds ;		       /* Pente interpolation               */
	double duiss ;                    /* Pas d'interpolation               */
} Traj_List ;
/**/
/****************************************************************************/
/*                PROCEDURES MANIPULANT DES SEGMENTS                        */
/****************************************************************************/
/* Creation d'un segment de trajectoire depuis le point pt.                 */
/*--------------------------------------------------------------------------*/
Seg_Traj *new_segtraj (int dimsp, double *pt) ;

/*--------------------------------------------------------------------------*/
/* Destruction d'un segment de trajectoire.                                 */
/*--------------------------------------------------------------------------*/
void del_segtraj (Seg_Traj *seg) ;

/*--------------------------------------------------------------------------*/
/* Calcul du point d'un segment en u                                        */
/*--------------------------------------------------------------------------*/
void ptu_segtraj (double *pt, double u, Seg_Traj *seg) ;

/*--------------------------------------------------------------------------*/
/* Calcul de la tangente d'un segment en u                                  */
/*--------------------------------------------------------------------------*/
void d1Mdu_segtraj (double *vec, double u, Seg_Traj *seg) ;

/*--------------------------------------------------------------------------*/
/* Calcul de derivee seconde en u                                           */
/*--------------------------------------------------------------------------*/
void d2Mdu_segtraj (double *vec, double u, Seg_Traj *seg) ;

/*--------------------------------------------------------------------------*/
/* Calcul de derivee troisieme en u                                         */
/*--------------------------------------------------------------------------*/
void d3Mdu_segtraj (double *vec, double u, Seg_Traj *seg) ;

/*--------------------------------------------------------------------------*/
/* Calcul de la courbure d'un segment en u                                  */
/*--------------------------------------------------------------------------*/
void d2Mdsu_segtraj (double *vec, double u, Seg_Traj *seg) ;

/*--------------------------------------------------------------------------*/
/* Calcul de la derive curligne troisieme d'un segment en u                */
/*--------------------------------------------------------------------------*/
void d3Mdsu_segtraj (double *vec, double u, Seg_Traj *seg) ;

/*--------------------------------------------------------------------------*/
/* Calcul longueur d'un segment de trajectoire avec le pas duiss           */
/*--------------------------------------------------------------------------*/
double long_segtraj (Seg_Traj *seg, double duiss) ;

/*--------------------------------------------------------------------------*/
/* Calcul u correspondant a s dans un segment de trajectoire                */
/*--------------------------------------------------------------------------*/
double s2u_segtraj (double s, Seg_Traj *seg) ;
/**/
/****************************************************************************/
/*              PROCEDURES MANIPULANT DES TRAJECTOIRES                      */
/****************************************************************************/
/* Creation d'une trajectoire                                               */
/*--------------------------------------------------------------------------*/
Traj_List *new_trajlist (int dimsp) ;

/*--------------------------------------------------------------------------*/
/* Destruction d'une trajectoire                                            */
/*--------------------------------------------------------------------------*/
void del_trajlist (Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Vidange d'une trajectoire                                                */
/*--------------------------------------------------------------------------*/
void vide_trajlist (Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Ajout d'un point en queue de la Trajectoire : cree un segment            */
/*--------------------------------------------------------------------------*/
Seg_Traj *appseg_trajlist (double *pt, Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Insertion d'un point dans la trajectoire avant segs : cree un segment    */
/*--------------------------------------------------------------------------*/
Seg_Traj *insseg_trajlist (double *pt, Seg_Traj *segs, Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Detruit un point et son segment associ dans la trajectoire              */
/*--------------------------------------------------------------------------*/
void delseg_trajlist (Seg_Traj *seg, Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Recherche du segment contenant l'abscisse paramtrique u                 */
/*--------------------------------------------------------------------------*/
Seg_Traj *u2seg_trajlist (double u, Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Initialisation de la generation des points d'une trajectoire f(u)        */
/*--------------------------------------------------------------------------*/
void inigen_trajlist(Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Generation des points d'une trajectoire                                  */
/* !! Il est obligatoire d'appeler inigen_trajlist avant la generation !!!  */
/*--------------------------------------------------------------------------*/
void gen_trajlist(double *pt, double u, Traj_List *Traj) ;
/**/
/*--------------------------------------------------------------------------*/
/* Calcul longueur curviligne d'une trajectoire avec le pas duiss          */
/*--------------------------------------------------------------------------*/
double long_trajlist (Traj_List *Traj, double duiss) ;

/*--------------------------------------------------------------------------*/
/* Recherche segment contenant l'abscisse curviligne s                      */
/*--------------------------------------------------------------------------*/
Seg_Traj *s2seg_trajlist (double s, Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Calcul u correspondant a s pour une trajectoire                          */
/*--------------------------------------------------------------------------*/
double s2u_trajlist (double s, Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Initialisation de la generation des points d'une trajectoire f(s)        */
/*--------------------------------------------------------------------------*/
void inigens_trajlist(Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Generation des points d'une trajectoire a partir de l'abscisse curviligne*/
/* !! Il est obligatoire d'appeler inigens_trajlist avant la generation !!! */
/*--------------------------------------------------------------------------*/
void gens_trajlist(double *pt, double s, Traj_List *Traj) ;

/*--------------------------------------------------------------------------*/
/* Calcul d'une trajectoire sous forme de liste de segments                 */
/*--------------------------------------------------------------------------*/
void calc_trajlistc3 (Traj_List *Traj, int typdeb, int typfin, int typspli,
					  		int mode2, int mode3, int moden, 
							double ck1N1, double cmuc, double cliss) ;

#endif