#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifndef M_PI_2
#define M_PI_2 1.57079632679489661923
#endif
/* #include <windows.h> */
#include "trajpolc3.h"

/* prototype solv_lu */
int solv_lu (double *XY, double *A, int n, int ns, int trp) ;



/*-------------------------------------------------------------------------*/
/* RACCORD c3 entre 2 points par un polseven (b1,c1,d1 et b2,c2,d2 donnes) */
/*-------------------------------------------------------------------------*/
/*							MODULE REPUTE MARCHER						   */
void tr1p7_c3(double *A1, double d, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul du polynome du 7eme degre qui passe par les points A1 et A2,
    avec les tangentes B1, B2 les demi-courbures C1,C2 et les 1/6ime
	des jerks D1, D2 donnes.

    On a M(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4 + F1 u5 + G1 u6 + H1 u7

    La procedure calcule les coefficients E1, F1, G1 et H1 */
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n, *E1 = D1 + n, *F1 = E1 + n ;
double *G1 = F1 + n, *H1 = G1 + n ;
double *B2 = A2 + n , *C2 = B2 + n , *D2 = C2 + n ;
double d2, d3, d4, d5, d6, d7 ;

    d2 = d * d   ; d3 = d2 * d ; d4 = d3 * d ; 
	d5 = d4 * d  ; d6 = d5 * d ; d7 = d6 * d ;

    while (n--)
    {
        E1[n] = 35.*(A2[n]-A1[n])/d4 - (15.*B2[n]+20.*B1[n])/d3 
			          + (5.*C2[n]-10.*C1[n])/d2 - (D2[n]+4.*D1[n])/d ;
        F1[n] = -84.*(A2[n]-A1[n])/d5 + (39.*B2[n]+45.*B1[n])/d4 
			          - (14.*C2[n]-20.*C1[n])/d3 + (3.*D2[n]+6.*D1[n])/d2 ;
        G1[n] = 70.*(A2[n]-A1[n])/d6 - (34.*B2[n]+36.*B1[n])/d5 
			          + (13.*C2[n]-15.*C1[n])/d4 - (3.*D2[n]+4.*D1[n])/d3 ;
        H1[n] = -20.*(A2[n]-A1[n])/d7 + 10.*(B2[n]+B1[n])/d6 
			          -4.*(C2[n]-C1[n])/d5 + (D2[n]+D1[n])/d4 ;
    }
}

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 3 points par 2 pols de degr 6 et 5           */
/*             Les coefficients b1,c1,d1 et b3,c3,d3 sont donns             */
/*---------------------------------------------------------------------------*/
/*							MODULE REPUTE MARCHER						     */
int tr2p65_c3 (double *A1, double d1, double *A2, double d2,
                                                        double *A3, int dim)
/*--------------------------------------------------------------------------*/
/*
   Calcul des 2 polynomes du 6eme et 5eme degres qui passent par les points 
   A1, A2 et A3 avec les tangentes B1, B3, les demi-courbures C1, C3 et les
   1/6ime de jerk D1 et D3 donns.

    On a
        M1(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4 + F1 u5 + G1 u6
        M2(u) = A2 + B2 u + C2 u2 + D2 u3 + E2 u4 + F2 u5                   */
/*--------------------------------------------------------------------------*/
{
int n = dim ;
double *B1 = A1+n, *C1 = B1+n, *D1 = C1+n, *E1 = D1+n, *F1 = E1+n, *G1 = F1+n;
double *B2 = A2+n, *C2 = B2+n, *D2 = C2+n, *E2 = D2+n, *F2 = E2+n;
double *B3 = A3+n, *C3 = B3+n, *D3 = C3+n;
double d12 = d1*d1, d13 = d12*d1, d14 = d13*d1, d15 = d14*d1, d16 = d15*d1 ;
double d22 = d2*d2, d23 = d22*d2, d24 = d23*d2, d25 = d24*d2 ;

         /* Par / a conv mll les matrices sont transposees dans ce sp. */
double Pm1[] = { 15./d14, -5./d13,  1./d12, 
                -24./d15,  9./d14, -2./d13, 
			     10./d16, -4./d15,  1./d14} ;

double Vm1[] = {  2./d22,   -2./d2,     1., 
                 -2./d23,   2.5/d22, -1.5/d2, 
			     0.6/d24,  -0.8/d23,  0.6/d22} ;

double RPm1[] = {20./d13, -10./d12, 4./d1} ;

double TVm1[] = {-2./d22,   2./d2,  -1.,
				 0.6*d2, -0.3*d22, 0.1*d23} ;

double Wm1[] = {-0.5*d12*d24, 10.*(d1+2.*d2)*d1*d2,
				 2.*d12*d23, 50.*d22-10.*d12} ;

double Wcte = d22*(d1+d2)*(3.*d1+5.*d2) ;
double SM[8], Y[2] ;
int i ;


#if 1 /* Version debugge */

    /* Valeur exacte Wm1 */
    for (i = 0 ; i < 4 ; i++) Wm1[i] /= Wcte ;

    while (n--)
    {
		SM[0] = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[1] = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[2] = - (C1[n]+3.*D1[n]*d1) ;
        SM[3] = -D1[n] ;
		SM[4] = A3[n] - A2[n] ;
		SM[5] = B3[n] ;
		SM[6] = C3[n] ;
		SM[7] = D3[n] ;
   
		/* Calcul S2 - RPm1*S1 -TVm1*S3 dans S2 */
		SM[3] -= RPm1[0]*SM[0]+RPm1[1]*SM[1]+RPm1[2]*SM[2]+
				 TVm1[0]*SM[5]+TVm1[1]*SM[6]+TVm1[2]*SM[7] ;
		SM[4] -= TVm1[3]*SM[5]+TVm1[4]*SM[6]+TVm1[5]*SM[7] ;

		/* Calcul Y = Wm1*(...) */
		B2[n] = Y[0] = Wm1[0]*SM[3] + Wm1[1]*SM[4] ;
		C2[n] = Y[1] = Wm1[2]*SM[3] + Wm1[3]*SM[4] ;

		/* Calcul S1-Q*Y dans S1 */
		SM[1] += Y[0] ;
		SM[2] += Y[1] ;
		
		/* Calcul X = Pm1*(S1-Q*Y) */
		E1[n] = Pm1[0]*SM[0] + Pm1[1]*SM[1] + Pm1[2]*SM[2] ;
		F1[n] = Pm1[3]*SM[0] + Pm1[4]*SM[1] + Pm1[5]*SM[2] ;
		G1[n] = Pm1[6]*SM[0] + Pm1[7]*SM[1] + Pm1[8]*SM[2] ;

		/* Calcul de S3-U*Y dans S3 */
		SM[5] -= Y[0] + 2.*d2*Y[1] ;
		SM[6] -= Y[1] ;

		/* Calcul Z = Vm1*(S3-U*Y) */
		D2[n] = Vm1[0]*SM[5] +Vm1[1]*SM[6] + Vm1[2]*SM[7] ;
		E2[n] = Vm1[3]*SM[5] +Vm1[4]*SM[6] + Vm1[5]*SM[7] ;
		F2[n] = Vm1[6]*SM[5] +Vm1[7]*SM[6] + Vm1[8]*SM[7] ;
    }

#else  /* Version LU qui marche egalement */
	/* Ici c'est la transposee au sens de MLL */
double Mat[] = {
   d14,    d15,    d16, 0., 0.   , 0.    , 0.    , 0.    ,
4.*d13, 5.*d14, 6.*d15,-1., 0.   , 0.    , 0.    , 0.    ,
6.*d12,10.*d13,15.*d14, 0.,-1.   , 0.    , 0.    , 0.    ,
4.*d1 ,10.*d12,20.*d13, 0., 0.   ,-1.    , 0.    , 0.    ,
0.    , 0.    , 0.    , d2,   d22,    d23,    d24,    d25,
0.    , 0.    , 0.    , 1., 2.*d2, 3.*d22, 4.*d23, 5.*d24,
0.    , 0.    , 0.    , 0., 1.   , 3.*d2 , 6.*d22,10.*d23,
0.    , 0.    , 0.    , 0., 0.   , 1.    , 4.*d2 ,10.*d22} ;
   
int resok, n8 ;
double *SS = (double *) calloc (dim*8, sizeof(double)) ;

    for (n = 0 ; n < dim ; n++)
    {
		n8 = n*8 ;
		SS[n8+0]  = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SS[n8+1]  = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SS[n8+2]  = - (C1[n]+3.*D1[n]*d1) ;
        SS[n8+3]  = -D1[n] ;
		SS[n8+4]  = A3[n] - A2[n] ;
		SS[n8+5]  = B3[n] ;
		SS[n8+6]  = C3[n] ;
		SS[n8+7]  = D3[n] ;
	}
		
		/* Resolution Wx=S */
	resok = solv_lu(SS, Mat, 8, dim, 1) ;
	if (!resok) { free (SS) ; return 1 ; }
    for (n = 0 ; n < dim; n++)
    {
		n8 = n*8 ;
		E1[n] = SS[n8+0] ;
		F1[n] = SS[n8+1] ;
		G1[n] = SS[n8+2] ;
        B2[n] = SS[n8+3] ;
		C2[n] = SS[n8+4] ;
		D2[n] = SS[n8+5] ;
		E2[n] = SS[n8+6] ;
		F2[n] = SS[n8+7] ;
	}
	free(SS) ;
#endif

	return 0 ;
}

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 3 points par 2 pols de degr 5 et 6           */
/*             Les coefficients b1,c1,d1 et b3,c3,d3 sont donns             */
/*---------------------------------------------------------------------------*/
/*					MODULE REPUTE MARCHER dans sa version LU			     */
int tr2p56_c3 (double *A1, double d1, double *A2, double d2,
                                                        double *A3, int dim)
/*--------------------------------------------------------------------------*/
/*
   Calcul des 2 polynomes du 5eme et 6eme degres qui passent par les points 
   A1, A2 et A3 avec les tangentes B1, B3, les demi-courbures C1, C3 et les
   1/6ime de jerk D1 et D3 donns.

    On a
        M1(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4 + F1 u5 
        M2(u) = A2 + B2 u + C2 u2 + D2 u3 + E2 u4 + F2 u5 + G2 u6           */
/*--------------------------------------------------------------------------*/
{
int resok, n = dim ;
double *B1 = A1+n, *C1 = B1+n, *D1 = C1+n, *E1 = D1+n, *F1 = E1+n;
double *B2 = A2+n, *C2 = B2+n, *D2 = C2+n, *E2 = D2+n, *F2 = E2+n, *G2 = F2+n;
double *B3 = A3+n, *C3 = B3+n, *D3 = C3+n;
double d12 = d1*d1, d13 = d12*d1, d14 = d13*d1, d15 = d14*d1 ;
double d22 = d2*d2, d23 = d22*d2, d24 = d23*d2, d25 = d24*d2, d26 = d25*d2 ;

#if 0 /* Version a debugger */
         /* Par / a conv mll les matrices sont transposees */
double Pm1[] = { 5./d14, -1./d13 
                -4./d15,  1./d14} ;

double Vm1[] = { 2.5/d23, -2./d22, 0.75/d2, 
                 -3./d24, 2.8/d23, -1.2/d22, 
			      1./d25, -1./d24,  0.5/d23} ;

double RPm1[] = {-10./d12, 4./d1
				 -20./d13, 6./d12} ;

double TVm1[] = {0.5*d2, -0.2*d22, 0.05*d23} ;

double Wm1[] = {2.*d12*d22          , 0.5*d12*d23         ,  10.*d12,
				-d2*(3.*d22+5.*d12) ,  2.*d23*d1          ,  40.*d1,
				12.*d22             ,-d2*(8.*d2+5.*d1)*d1 , 60.      } ;

double Wcte = d2*(d1+d2)*(5.*d1+3.*d2) ;
double SM[8], Y[3] ;
int i ;

    /* Valeur exacte Wm1 */
    for (i = 0 ; i < 9 ; i++) Wm1[i] /= Wcte ;

    while (n--)
    {
		SM[0] = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[1] = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[2] = - (C1[n]+3.*D1[n]*d1) ;
        SM[3] = -D1[n] ;
		SM[4] = A3[n] - A2[n] ;
		SM[5] = B3[n] ;
		SM[6] = C3[n] ;
		SM[7] = D3[n] ;
   
		/* Calcul S2 - RPm1*S1 -TVm1*S3 dans S2 */
		SM[2] -= RPm1[0]*SM[0]+RPm1[1]*SM[1] ;
		SM[3] -= RPm1[2]*SM[0]+RPm1[3]*SM[1] ;
		SM[4] -= TVm1[0]*SM[5]+TVm1[1]*SM[6]+TVm1[2]*SM[7] ;

		/* Calcul Y = Wm1*(...) */
		B2[n] = Y[0] = Wm1[0]*SM[2] + Wm1[1]*SM[3] + Wm1[2]*SM[4] ;
		C2[n] = Y[1] = Wm1[3]*SM[2] + Wm1[4]*SM[3] + Wm1[5]*SM[4] ;
		D2[n] = Y[2] = Wm1[6]*SM[2] + Wm1[7]*SM[3] + Wm1[8]*SM[4] ;

		/* Calcul S1-Q*Y dans S1 */
		SM[1] += Y[0] ;
		
		/* Calcul X = Pm1*(S1-Q*Y) */
		E1[n] = Pm1[0]*SM[0] + Pm1[1]*SM[1] ;
		F1[n] = Pm1[2]*SM[0] + Pm1[3]*SM[1] ;

		/* Calcul de S3-U*Y dans S3 */
		SM[5] -= Y[0] + 2.*d2*Y[1] + 3.*d22*Y[2] ;
		SM[6] -=              Y[1] + 3.*d2 *Y[2] ;
		SM[7] -=                            Y[2] ;

		/* Calcul Z = Vm1*(S3-U*Y) */
		E2[n] = Vm1[0]*SM[5] +Vm1[1]*SM[6] + Vm1[2]*SM[7] ;
		F2[n] = Vm1[3]*SM[5] +Vm1[4]*SM[6] + Vm1[5]*SM[7] ;
		G2[n] = Vm1[6]*SM[5] +Vm1[7]*SM[6] + Vm1[8]*SM[7] ;
    }
#else  /* Version par resolution LU */
	/* Ici c'est la transposee au sens de MLL */
double Mat[] = {
   d14,    d15, 0., 0.   , 0.    , 0.    , 0.    , 0.    ,
4.*d13, 5.*d14,-1., 0.   , 0.    , 0.    , 0.    , 0.    ,
6.*d12,10.*d13, 0.,-1.   , 0.    , 0.    , 0.    , 0.    ,
4.*d1 ,10.*d12, 0., 0.   ,-1.    , 0.    , 0.    , 0.    ,
0.    , 0.    , d2,   d22,    d23,    d24,    d25,    d26,
0.    , 0.    , 1., 2.*d2, 3.*d22, 4.*d23, 5.*d24, 6.*d25,
0.    , 0.    , 0., 1.   , 3.*d2 , 6.*d22,10.*d23,15.*d24,
0.    , 0.    , 0., 0.   , 1.    , 4.*d2 ,10.*d22,20.*d23} ;

int n8 ;
double *SM = (double *) calloc (dim*8, sizeof(double)) ;

    for (n = 0 ; n < dim ; n++)
    {
		n8 = n*8 ;
		SM[n8+0]  = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[n8+1]  = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[n8+2]  = - (C1[n]+3.*D1[n]*d1) ;
        SM[n8+3]  = -D1[n] ;
		SM[n8+4]  = A3[n] - A2[n] ;
		SM[n8+5]  = B3[n] ;
		SM[n8+6]  = C3[n] ;
		SM[n8+7]  = D3[n] ;
	}
		
		/* Resolution Wx=S */
	resok = solv_lu(SM, Mat, 8, dim, 1) ;
	if (!resok) {free(SM) ; return 1 ;}
    for (n = 0 ; n < dim; n++)
    {
		n8 = n*8 ;
		E1[n] = SM[n8+0] ;
		F1[n] = SM[n8+1] ;
        B2[n] = SM[n8+2] ;
		C2[n] = SM[n8+3] ;
		D2[n] = SM[n8+4] ;
		E2[n] = SM[n8+5] ;
		F2[n] = SM[n8+6] ;
		G2[n] = SM[n8+7] ;
	}
	free(SM) ;
#endif
	return 0 ;
}

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 4 points par 3 polynomes de degr 5           */
/*             Les coefficients b1,c1,d1 et b4,c4,d4 sont donns             */
/*---------------------------------------------------------------------------*/
/*            MODULE REPUTE MARCHER dans sa version   LU                     */
int tr3p555_c3 (double *A1, double d1, double *A2, double d2,
                                  double *A3, double d3, double *A4, int dim)
/*--------------------------------------------------------------------------*/
/*
   Calcul des 3 polynomes de degr 5 qui passent par les points 
   A1, A2,A3 et A4 avec les tangentes B1, B4, les demi-courbures C1, C4 et
   les 1/6ime de jerk D1 et D4 donns.

    On a
        M1(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4 + F1 u5 
        M2(u) = A2 + B2 u + C2 u2 + D2 u3 + E2 u4 + F2 u5           
        M3(u) = A3 + B3 u + C3 u2 + D3 u3 + E3 u4 + F3 u5                   */
/* Retour : 0 ok
		    1 erreur resolution                                             */
/*--------------------------------------------------------------------------*/
{
int n = dim, resok ;
double *B1 = A1+n, *C1 = B1+n, *D1 = C1+n, *E1 = D1+n, *F1 = E1+n;
double *B2 = A2+n, *C2 = B2+n, *D2 = C2+n, *E2 = D2+n, *F2 = E2+n;
double *B3 = A3+n, *C3 = B3+n, *D3 = C3+n, *E3 = D3+n, *F3 = E3+n;
double *B4 = A4+n, *C4 = B4+n, *D4 = C4+n;
double d12 = d1*d1, d13 = d12*d1, d14 = d13*d1, d15 = d14*d1 ;
double d22 = d2*d2, d23 = d22*d2, d24 = d23*d2, d25 = d24*d2 ;
double d32 = d3*d3, d33 = d32*d3, d34 = d33*d3, d35 = d34*d3;

#if 0 /* Version a debugger */

double R1T1QP2m1T2[] = {d14, 4.*d13,  6.*d12,  4.*d1,
                        d15, 5.*d14, 10.*d13, 10.*d12,
                         0.,    d24, -4.*d23,  6.*d22};

double P2m1[]={ 4./d2 , -6./d22,  4./d23, -1/d24,
			   -3.    ,  6./d2 , -4./d22, 1./d23,
				2.*d2 , -5.    ,  4./d2, -1./d22
				  -d22,  3.*d2 , -3.,     1./d2     };

double P3m1[]={ 4./d3 , -6./d32,  4./d33, -1/d34,
			   -3.    ,  6./d3 , -4./d32, 1./d33,
				2.*d3 , -5.    ,  4./d3, -1./d32
				  -d32,  3.*d3 , -3.,     1./d3     };

double QP2m1[]={0., -4./d2 , 6./d22, -4./d23,
				0.,  3.,    -6./d2,   4./d22,
				0., -2.*d2 , 5.,     -4./d2,
				0.,     d22,-3.*d2,   3.     };

double QP3m1[]={0., -4./d3 , 6./d32, -4./d33,
				0.,  3.,    -6./d3,   4./d32,
				0., -2.*d3 , 5.,     -4./d3,
				0.,     d32,-3.*d3,   3.     };

double T2[] = {d25, 5.*d24, 10.*d23, 10.*d22};
double T3[] = {d35, 5.*d34, 10.*d33, 10.*d32};

double QP3m1T3[]={0.,    d34,  -4.*d33,  6.*d32};

double QP2m1QP3m1T3[4], W[16], SM[12] ;

double Work1[4], Work2[4] ;

/* Plutot que d'inverser W on resoud le systeme. Valable uniquement si
   l'espace compte moins de 4 composantes. Pour 4 composantes c'est a peu
   pres la meme chose. Au dela il vaudrait mieux calculer l'inverse */

    /* Calcul QP2m1QP3m1T3 */
    mult_mat(QP2m1QP3m1T3, QP2m1, QP3m1T3,4,4,1) ;

    while (n--)
    {
		SM[0] = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[1] = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[2] = - (C1[n]+3.*D1[n]*d1) ;
        SM[3] = -D1[n] ;
		SM[4] = A3[n] - A2[n] ;
		SM[8] = A4[n] - A3[n] ;
		SM[9] = B4[n] ;
		SM[10] = C4[n] ;
		SM[11] = D4[n] ;
		
		/* ans = QP3m1*S3 */
        mult_mat(Work2, QP3m1, SM+8, 4,4,1) ;
		/* ans = QP2m1 * ans */
        mult_mat(Work1, QP2m1, Work2, 4,4,1) ;
        /* ans = QP2m1*S2 */
		fact_vect(Work2, SM[4], QP2m1, 4) ;
		/* ans = S1 - QP2m1*S2 + QP2m1*QP3m1*S3 */
		sub_vect(SM,SM,Work2,4) ;
		add_vect(SM,SM,Work1,4) ;

		/* Formation matrice W */
		cpy_vect(W, R1T1QP2m1T2, 12) ;
		cpy_vect(W+12, QP2m1QP3m1T3, 4) ;

		/* Resolution Wx=S1 */
		resok = solv_lu(SM, W, 4, 1, 0) ;
		if (!resok) return 1 ; 
		E1[n] = SM[0] ;
		F1[n] = SM[1] ;
		F2[n] = SM[2] ;
		F3[n] = SM[3] ;

		/* ans = S3-T3*F3 */
		cum_vect (SM+8, SM+8,-SM[3], T3,4) ;
		/* ans = P3m1 * (S3-T3*F3) */
		mult_mat(Work2,P3m1,SM+8,4,4,1);
		B3[n] = Work2[0] ;
		C3[n] = Work2[1] ;
		D3[n] = Work2[2] ;
		E3[n] = Work2[3] ;
   
		/* ans = S2 + Work2(1:3)-T2*F2 */
		cpy_vect(SM+5,Work2+1,3) ;
		cum_vect (SM+4, SM+4,-SM[2], T2,4) ;
		/* ans = P2m1*(S2+..) */
		mult_mat(Work1,P2m1,SM+4,4,4,1) ;
		B2[n] = Work1[0] ;
		C2[n] = Work1[1] ;
		D2[n] = Work1[2] ;
		E2[n] = Work1[3] ;
	}
#else  /* Version par resolution LU */
/* Ici c'est la transposee au sens de MLL */
double Mat[] = {
   d14,    d15, 0., 0.   , 0.    , 0.    ,     0., 0., 0.  , 0.    , 0.    ,     0.,
4.*d13, 5.*d14,-1., 0.   , 0.    , 0.    ,     0., 0., 0.  , 0.    , 0.    ,     0.,
6.*d12,10.*d13, 0.,-1.   , 0.    , 0.    ,     0., 0., 0.  , 0.    , 0.    ,     0.,
4.*d1 ,10.*d12, 0., 0.   ,-1.    , 0.    ,     0., 0., 0.  , 0.    , 0.    ,     0.,
0.    , 0.    , d2,   d22,    d23,    d24,    d25, 0.,  0. , 0.    , 0.    ,     0.,
0.    , 0.    , 1., 2.*d2, 3.*d22, 4.*d23, 5.*d24,-1., 0.  , 0.    , 0.    ,     0.,
0.    , 0.    , 0., 1.   , 3.*d2 , 6.*d22,10.*d23, 0.,-1.  , 0.    , 0.    ,     0.,
0.    , 0.    , 0., 0.   , 1.    , 4.*d2 ,10.*d22, 0., 0.  ,-1.    , 0.    ,     0.,
0.    , 0.    , 0., 0.   , 0.    ,     0., 0.    , d3,  d32,    d33,    d34,    d35,
0.    , 0.    , 0., 0.   , 0.    ,     0., 0.    , 1.,2.*d3, 3.*d32, 4.*d33, 5.*d34,
0.    , 0.    , 0., 0.   , 0.    ,     0., 0.    , 0.,1.   , 3.*d3 , 6.*d32,10.*d33,
0.    , 0.    , 0., 0.   , 0.    ,     0., 0.    , 0.,0.   , 1.    , 4.*d3 ,10.*d32 } ;

int n12 ;
double *SM = (double *) calloc (dim*12, sizeof(double)) ;

    for (n = 0 ; n < dim ; n++)
    {
		n12 = n*12 ;
		SM[n12+0]  = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[n12+1]  = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[n12+2]  = - (C1[n]+3.*D1[n]*d1) ;
        SM[n12+3]  = -D1[n] ;
		SM[n12+4]  = A3[n] - A2[n] ;
		SM[n12+5]  = 0. ;
		SM[n12+6]  = 0. ;
		SM[n12+7]  = 0. ;
		SM[n12+8]  = A4[n] - A3[n] ;
		SM[n12+9]  = B4[n] ;
		SM[n12+10] = C4[n] ;
		SM[n12+11] = D4[n] ;
	}
		
		/* Resolution Wx=S */
	resok = solv_lu(SM, Mat, 12, dim, 1) ;
	if (!resok) { free(SM) ; return 1 ;}
    for (n = 0 ; n < dim; n++)
    {
		n12 = n*12 ;
		E1[n] = SM[n12+0] ;
		F1[n] = SM[n12+1] ;
        B2[n] = SM[n12+2] ;
		C2[n] = SM[n12+3] ;
		D2[n] = SM[n12+4] ;
		E2[n] = SM[n12+5] ;
		F2[n] = SM[n12+6] ;
		B3[n] = SM[n12+7] ;
		C3[n] = SM[n12+8] ;
		D3[n] = SM[n12+9] ;
		E3[n] = SM[n12+10] ;
		F3[n] = SM[n12+11] ;
	}
	free(SM) ;
#endif
	return 0 ;
}

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 4 points par 3 polynomes de degr 6, 4 et 5   */
/*             Les coefficients b1,c1,d1 et b4,c4,d4 sont donns             */
/*---------------------------------------------------------------------------*/
/*					MODULE REPUTE MARCHER dans sa version LU			     */
int tr3p645_c3 (double *A1, double d1, double *A2, double d2,
                                  double *A3, double d3, double *A4, int dim)
/*--------------------------------------------------------------------------*/
/*
   Calcul des 3 polynomes de degr 6, 4 et 5 qui passent par les points 
   A1, A2,A3 et A4 avec les tangentes B1, B4, les demi-courbures C1, C4 et
   les 1/6ime de jerk D1 et D4 donns.

    On a
        M1(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4 + F1 u5 + G1 u6 
        M2(u) = A2 + B2 u + C2 u2 + D2 u3 + E2 u4            
        M3(u) = A3 + B3 u + C3 u2 + D3 u3 + E3 u4 + F3 u5                   */
/* Retour : 0 ok
		    1 erreur resolution                                             */
/*--------------------------------------------------------------------------*/
{
int resok, n = dim ;
double *B1 = A1+n, *C1 = B1+n, *D1 = C1+n, *E1 = D1+n, *F1 = E1+n, *G1=F1+n;
double *B2 = A2+n, *C2 = B2+n, *D2 = C2+n, *E2 = D2+n;
double *B3 = A3+n, *C3 = B3+n, *D3 = C3+n, *E3 = D3+n, *F3 = E3+n ;
double *B4 = A4+n, *C4 = B4+n, *D4 = C4+n;
double d12 = d1*d1, d13 = d12*d1, d14 = d13*d1, d15 = d14*d1, d16 = d15*d1 ;
double d22 = d2*d2, d23 = d22*d2, d24 = d23*d2 ;
double d32 = d3*d3, d33 = d32*d3, d34 = d33*d3, d35 = d34*d3 ;

#if 0
/* Version a debugger */

double R1[] = {d14, 4.*d13,  6.*d12, 4.*d1,
	                d15, 5.*d14, 10.*d13, 10.*d12,
				    d16, 6.*d15, 15.*d14, 20.*d13};

double P2m1[]={ 4./d2 , -6./d22,  4./d23, -1/d24,
			   -3.    ,  6./d2 , -4./d22, 1./d23,
				2.*d2 , -5.    ,  4./d2, -1./d22
				  -d22,  3.*d2 , -3.,     1./d2     };

double P3m1[]={ 4./d3 , -6./d32,  4./d33, -1/d34,
			   -3.    ,  6./d3 , -4./d32, 1./d33,
				2.*d3 , -5.    ,  4./d3, -1./d32
				  -d32,  3.*d3 , -3.,     1./d3     };

double QP2m1[]={0., -4./d2 , 6./d22, -4./d23,
				0.,  3.,    -6./d2,   4./d22,
				0., -2.*d2 , 5.,     -4./d2,
				0.,     d22,-3.*d2,   3.     };

double QP3m1[]={0., -4./d3 , 6./d32, -4./d33,
				0.,  3.,    -6./d3,   4./d32,
				0., -2.*d3 , 5.,     -4./d3,
				0.,     d32,-3.*d3,   3.     };

double T3[] = {d35, 5.*d34, 10.*d33, 10.*d32};

double QP3m1T3[]={0.,    d34,  -4.*d33,  6.*d32};

double QP2m1QP3m1T3[4], W[16], SM[12] ;

double Work1[4], Work2[4] ;

/* Plutot que d'inverser W on resoud le systeme. Valable uniquement si
   l'espace compte moins de 4 composantes. Pour 4 composantes c'est a peu
   pres la meme chose. Au dela il vaudrait mieux calculer l'inverse */

    /* Calcul QP2m1QP3m1T3 */
    mult_mat(QP2m1QP3m1T3, QP2m1, QP3m1T3,4,4,1) ;

    while (n--)
    {
		SM[0] = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[1] = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[2] = - (C1[n]+3.*D1[n]*d1) ;
        SM[3] = -D1[n] ;
		SM[4] = A3[n] - A2[n] ;
	/*	SM[5] = SM[6] = SM[7] = 0. ; inutile car ecrases plus loin */
		SM[8] = A4[n] - A3[n] ;
		SM[9] = B4[n] ;
		SM[10] = C4[n] ;
		SM[11] = D4[n] ;
		
		/* ans = QP3m1*S3 */
        mult_mat(Work2, QP3m1, SM+8, 4,4,1) ;
		/* ans = QP2m1 * ans */
        mult_mat(Work1, QP2m1, Work2, 4,4,1) ;
        /* ans = QP2m1*S2 */
		fact_vect(Work2, SM[4], QP2m1, 4) ;
		/* ans = S1 - QP2m1*S2 + QP2m1*QP3m1*S3 */
		sub_vect(SM,SM,Work2,4) ;
		add_vect(SM,SM,Work1,4) ;

		/* Formation matrice W */
		cpy_vect(W, R1, 12) ;
		cpy_vect(W+12, QP2m1QP3m1T3, 4) ;

		/* Resolution Wx=S1 */
		resok = solv_lu(SM, W, 4, 1, 0) ;
		if (!resok) return 1 ;
		E1[n] = SM[0] ;
		F1[n] = SM[1] ;
		G1[n] = SM[2] ;
		F3[n] = SM[3] ;

		/* ans = S3-T3*F3 */
		cum_vect (SM+8, SM+8,-SM[3], T3,4) ;
		/* ans = P3m1 * (S3-T3*F3) */
		mult_mat(Work2,P3m1,SM+8,4,4,1);
		B3[n] = Work2[0] ;
		C3[n] = Work2[1] ;
		D3[n] = Work2[2] ;
		E3[n] = Work2[3] ;
   
   
		/* ans = S2 + Work2(0:2) */
		cpy_vect(SM+5,Work2,3) ;
		/* ans = P2m1*(S2+..) */
		mult_mat(Work1,P2m1,SM+4,4,4,1) ;
		B2[n] = Work1[0] ;
		C2[n] = Work1[1] ;
		D2[n] = Work1[2] ;
		E2[n] = Work1[3] ;
	}
#else  /* version par resolution LU */
/* Ici c'est la transposee au sens de MLL */
double Mat[] = {
   d14,    d15,    d16, 0., 0.   , 0.    , 0.    , 0., 0.  , 0.    , 0.    ,     0.,
4.*d13, 5.*d14, 6.*d15,-1., 0.   , 0.    , 0.    , 0., 0.  , 0.    , 0.    ,     0.,
6.*d12,10.*d13,15.*d14, 0.,-1.   , 0.    , 0.    , 0., 0.  , 0.    , 0.    ,     0.,
4.*d1 ,10.*d12,20.*d13, 0., 0.   ,-1.    , 0.    , 0., 0.  , 0.    , 0.    ,     0.,
0.    , 0.    , 0.    , d2,   d22,    d23,    d24, 0.,  0. , 0.    , 0.    ,     0.,
0.    , 0.    , 0.    , 1., 2.*d2, 3.*d22, 4.*d23,-1., 0.  , 0.    , 0.    ,     0.,
0.    , 0.    , 0.    , 0., 1.   , 3.*d2 , 6.*d22, 0.,-1.  , 0.    , 0.    ,     0.,
0.    , 0.    , 0.    , 0., 0.   , 1.    , 4.*d2 , 0., 0.  ,-1.    , 0.    ,     0.,
0.    , 0.    , 0.    , 0., 0.   , 0.    , 0.    , d3,  d32,    d33,    d34,    d35,
0.    , 0.    , 0.    , 0., 0.   , 0.    , 0.    , 1.,2.*d3, 3.*d32, 4.*d33, 5.*d34,
0.    , 0.    , 0.    , 0., 0.   , 0.    , 0.    , 0.,1.   , 3.*d3 , 6.*d32,10.*d33,
0.    , 0.    , 0.    , 0., 0.   , 0.    , 0.    , 0.,0.   , 1.    , 4.*d3 ,10.*d32 } ;

int n12 ;
double *SM = (double *) calloc (dim*12, sizeof(double)) ;

    for (n = 0 ; n < dim ; n++)
    {
		n12 = n*12 ;
		SM[n12+0]  = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[n12+1]  = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[n12+2]  = - (C1[n]+3.*D1[n]*d1) ;
        SM[n12+3]  = -D1[n] ;
		SM[n12+4]  = A3[n] - A2[n] ;
		SM[n12+5]  = 0. ;
		SM[n12+6]  = 0. ;
		SM[n12+7]  = 0. ;
		SM[n12+8]  = A4[n] - A3[n] ;
		SM[n12+9]  = B4[n] ;
		SM[n12+10] = C4[n] ;
		SM[n12+11] = D4[n] ;
	}
		
		/* Resolution Wx=S */
	resok = solv_lu(SM, Mat, 12, dim, 1) ;
	if (!resok) { free(SM) ; return 1 ;}
    for (n = 0 ; n < dim; n++)
    {
		n12 = n*12 ;
		E1[n] = SM[n12+0] ;
		F1[n] = SM[n12+1] ;
		G1[n] = SM[n12+2] ;
        B2[n] = SM[n12+3] ;
		C2[n] = SM[n12+4] ;
		D2[n] = SM[n12+5] ;
		E2[n] = SM[n12+6] ;
		B3[n] = SM[n12+7] ;
		C3[n] = SM[n12+8] ;
		D3[n] = SM[n12+9] ;
		E3[n] = SM[n12+10] ;
		F3[n] = SM[n12+11] ;
	}
	free(SM) ;
#endif
	return 0 ;
}

/*---------------------------------------------------------------------------*/
/*            RACCORD c3 entre 4 points par 3 polynomes de degr 5, 4 et 6   */
/*             Les coefficients b1,c1,d1 et b4,c4,d4 sont donns             */
/*---------------------------------------------------------------------------*/
/*					MODULE REPUTE MARCHER dans sa version LU			     */
int tr3p546_c3 (double *A1, double d1, double *A2, double d2,
                                  double *A3, double d3, double *A4, int dim)
/*--------------------------------------------------------------------------*/
/*
   Calcul des 3 polynomes de degr 5, 4 et 6 qui passent par les points 
   A1, A2,A3 et A4 avec les tangentes B1, B4, les demi-courbures C1, C4 et
   les 1/6ime de jerk D1 et D4 donns.

    On a
        M1(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4 + F1 u5 
        M2(u) = A2 + B2 u + C2 u2 + D2 u3 + E2 u4            
        M3(u) = A3 + B3 u + C3 u2 + D3 u3 + E3 u4 + F3 u5 + G3 u6           */
/* Retour : 0 ok
		    1 erreur resolution                                             */
/*--------------------------------------------------------------------------*/
{
int resok, n = dim ;
double *B1 = A1+n, *C1 = B1+n, *D1 = C1+n, *E1 = D1+n, *F1 = E1+n;
double *B2 = A2+n, *C2 = B2+n, *D2 = C2+n, *E2 = D2+n;
double *B3 = A3+n, *C3 = B3+n, *D3 = C3+n, *E3 = D3+n, *F3 = E3+n, *G3=F3+n ;
double *B4 = A4+n, *C4 = B4+n, *D4 = C4+n;
double d12 = d1*d1, d13 = d12*d1, d14 = d13*d1, d15 = d14*d1 ;
double d22 = d2*d2, d23 = d22*d2, d24 = d23*d2 ;
double d32 = d3*d3, d33 = d32*d3, d34 = d33*d3, d35 = d34*d3, d36 = d35*d3 ;

#if 0
/* Version a debugger */
double R1[] = {d14, 4.*d13,  6.*d12, 4.*d1,
	           d15, 5.*d14, 10.*d13, 10.*d12};

double P2m1[]={ 4./d2 , -6./d22,  4./d23, -1/d24,
			   -3.    ,  6./d2 , -4./d22, 1./d23,
				2.*d2 , -5.    ,  4./d2, -1./d22
				  -d22,  3.*d2 , -3.,     1./d2     };

double P3m1[]={ 4./d3 , -6./d32,  4./d33, -1/d34,
			   -3.    ,  6./d3 , -4./d32, 1./d33,
				2.*d3 , -5.    ,  4./d3, -1./d32
				  -d32,  3.*d3 , -3.,     1./d3     };

double QP2m1[]={0., -4./d2 , 6./d22, -4./d23,
				0.,  3.,    -6./d2,   4./d22,
				0., -2.*d2 , 5.,     -4./d2,
				0.,     d22,-3.*d2,   3.     };

double QP3m1[]={0., -4./d3 , 6./d32, -4./d33,
				0.,  3.,    -6./d3,   4./d32,
				0., -2.*d3 , 5.,     -4./d3,
				0.,     d32,-3.*d3,   3.     };

double T3[] = {d35, 5.*d34, 10.*d33, 10.*d32,
	           d36, 6.*d35, 15.*d34, 20.*d33};

double QP3m1T3[]={0.,    d34,  -4.*d33,  6.*d32,
                  0., 4.*d35, -15.*d34, 20.*d33};

double QP2m1QP3m1T3[8], W[16], SM[12] ;

double Work1[4], Work2[4] ;

/* Plutot que d'inverser W on resoud le systeme. Valable uniquement si
   l'espace compte moins de 4 composantes. Pour 4 composantes c'est a peu
   pres la meme chose. Au dela il vaudrait mieux calculer l'inverse */

    /* Calcul QP2m1QP3m1T3 */
    mult_mat(QP2m1QP3m1T3, QP2m1, QP3m1T3,4,4,2) ;

    while (n--)
    {
		SM[0] = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[1] = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[2] = - (C1[n]+3.*D1[n]*d1) ;
        SM[3] = -D1[n] ;
		SM[4] = A3[n] - A2[n] ;
		SM[5] = SM[6] = SM[7] = 0. ;
		SM[8] = A4[n] - A3[n] ;
		SM[9] = B4[n] ;
		SM[10] = C4[n] ;
		SM[11] = D4[n] ;
		
		/* ans = QP3m1*S3 */
        mult_mat(Work2, QP3m1, &SM[8], 4,4,1) ;
		/* ans = QP2m1 * ans */
        mult_mat(Work1, QP2m1, Work2, 4,4,1) ;
        /* ans = QP2m1*S2 */
		fact_vect(Work2, SM[4], QP2m1, 4) ;
		/* ans = S1 - QP2m1*S2 + QP2m1*QP3m1*S3 */
		sub_vect(SM,SM,Work2,4) ;
		add_vect(SM,SM,Work1,4) ;

		/* Formation matrice W */
		cpy_vect(W, R1, 8) ;
		cpy_vect(W+8, QP2m1QP3m1T3, 8) ;

		/* Resolution Wx=S1 */
		resok = solv_lu(SM, W, 4, 1, 0) ;
		if (!resok) return 1 ; 
		E1[n] = SM[0] ;
		F1[n] = SM[1] ;
		F3[n] = SM[2] ;
		G3[n] = SM[3] ;

		/* ans = S3-T3*Z3 */
		mult_mat(Work1,T3,&SM[2],4,2,1);
		sub_vect(SM+8,SM+8,Work1,4);
		/* ans = P3m1 * (S3-T3*Z3) */
		mult_mat(Work2,P3m1,SM+8,4,4,1);
		B3[n] = Work2[0] ;
		C3[n] = Work2[1] ;
		D3[n] = Work2[2] ;
		E3[n] = Work2[3] ;
   
		/* ans = S2 + Work(0:2) */
		cpy_vect(SM+5,Work2,3) ;
		/* ans = P2m1*(S2+..) */
		mult_mat(Work1,P2m1,SM+4,4,4,1) ;
		B2[n] = Work1[0] ;
		C2[n] = Work1[1] ;
		D2[n] = Work1[2] ;
		E2[n] = Work1[3] ;
	}
	return 0 ;
#endif  /* version par resolution LU */
/* Ici c'est la transposee au sens de MLL */
double Mat[] = {
   d14,    d15, 0., 0.   , 0.    , 0.    , 0., 0.   , 0.    , 0.    ,     0., 0.   ,
4.*d13, 5.*d14,-1., 0.   , 0.    , 0.    , 0., 0.   , 0.    , 0.    ,     0., 0.   ,
6.*d12,10.*d13, 0.,-1.   , 0.    , 0.    , 0., 0.   , 0.    , 0.    ,     0., 0.   ,
4.*d1 ,10.*d12, 0., 0.   ,-1.    , 0.    , 0., 0.   , 0.    , 0.    ,     0., 0.   ,
0.    , 0.    , d2,   d22,    d23,    d24, 0., 0.   , 0.    , 0.    ,     0., 0.   ,
0.    , 0.    , 1., 2.*d2, 3.*d22, 4.*d23,-1., 0.   , 0.    , 0.    ,     0., 0.   ,
0.    , 0.    , 0., 1.   , 3.*d2 , 6.*d22, 0.,-1.   , 0.    , 0.    ,     0., 0.   ,
0.    , 0.    , 0., 0.   , 1.    , 4.*d2 , 0., 0.   ,-1.    , 0.    ,     0., 0.   ,
0.    , 0.    , 0., 0.   , 0.    , 0.    , d3,  d32 ,    d33,    d34,    d35,    d36, 
0.    , 0.    , 0., 0.   , 0.    , 0.    , 1., 2.*d3, 3.*d32, 4.*d33, 5.*d34, 6.*d35,
0.    , 0.    , 0., 0.   , 0.    , 0.    , 0., 1.   , 3.*d3 , 6.*d32,10.*d33,15.*d34,
0.    , 0.    , 0., 0.   , 0.    , 0.    , 0., 0.   , 1.    , 4.*d3 ,10.*d32,20.*d33} ;

int n12 ;
double *SM = (double *) calloc (dim*12, sizeof(double)) ;

    for (n = 0 ; n < dim ; n++)
    {
		n12 = n*12 ;
		SM[n12+0]  = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		SM[n12+1]  = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		SM[n12+2]  = - (C1[n]+3.*D1[n]*d1) ;
        SM[n12+3]  = -D1[n] ;
		SM[n12+4]  = A3[n] - A2[n] ;
		SM[n12+5]  = 0. ;
		SM[n12+6]  = 0. ;
		SM[n12+7]  = 0. ;
		SM[n12+8]  = A4[n] - A3[n] ;
		SM[n12+9]  = B4[n] ;
		SM[n12+10] = C4[n] ;
		SM[n12+11] = D4[n] ;
	}
		
		/* Resolution Wx=S */
	resok = solv_lu(SM, Mat, 12, dim, 1) ;
	if (!resok) { free(SM) ; return 1 ; }
    for (n = 0 ; n < dim; n++)
    {
		n12 = n*12 ;
		E1[n] = SM[n12+0] ;
		F1[n] = SM[n12+1] ;
        B2[n] = SM[n12+2] ;
		C2[n] = SM[n12+3] ;
		D2[n] = SM[n12+4] ;
		E2[n] = SM[n12+5] ;
		B3[n] = SM[n12+6] ;
		C3[n] = SM[n12+7] ;
		D3[n] = SM[n12+8] ;
		E3[n] = SM[n12+9] ;
		F3[n] = SM[n12+10] ;
		G3[n] = SM[n12+11] ;
	}
	free(SM) ; 
	return 0 ;
}


/*--------------------------------------------------------------------------*/
/*      RACCORD c3 entre N+1 points par N pols de degr 6,4,...,4,5         */
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*--------------------------------------------------------------------------*/
int trNp645_c3 (double **Coefs, double *pUf, int nbseg, int dimsp)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 5 coefficients
		(avec dimsp composantes) des polynomes de degre 4, sauf pour :
		le 1	7 coefs (degre 6)
		le nbseg 6 coefs (degre 5)
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : echec allocation
*/
/*--------------------------------------------------------------------------*/
{
#if O   /* Version a debugger qui ne marche pas */

int resok, k, n=dimsp, jB=n, jC=jB+n, jD=jC+n, jE=jD+n, jF=jE+n;

double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, d16=d15*d1;

double R1[] = {d14, 4.*d13,  6.*d12,  4.*d1,
			   d15, 5.*d14,	10.*d13, 10.*d12,
			   d16, 6.*d15, 15.*d14, 20.*d13};

double d2=pUf[1], d22=d2*d2, d23=d22*d2;

double Mat44[] = {0.,  4./d2 , -6./d22,  4./d23,
				  0., -3.,      6./d2,  -4./d22,
				  0.,  2.*d2 , -5.,      4./d2,
				  0.,    -d22,  3.*d2,  -3.     }; 

double dN=pUf[nbseg-1], dN2=dN*dN, dN3=dN2*dN, dN4=dN3*dN, dN5=dN4*dN ;

double TN[] = {dN5, 5.*dN4, 10.*dN3, 10.*dN2};

double PNm1[]={ 4./dN , -6./dN2,  4./dN3, -1/dN4,
			   -3.    ,  6./dN , -4./dN2, 1./dN3,
				2.*dN , -5.    ,  4./dN, -1./dN2
				  -dN2,  3.*dN , -3.,     1./dN     };

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n, *G1=F1+n ; 

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1], *BN=AN+n, *CN=BN+n, *DN=CN+n, *EN=DN+n, *FN=EN+n ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek ;

double W[16], Work[4], Vk[16], *bufV2k, *V2k, *bufPkm1, *Pkm1 ;

double *SM, *S1, *Sk, *SN ;

double dk, dk2, dk3, dk4 ;

double *buffer = (double *) malloc (sizeof(double)*(
	4*nbseg + /* pour SM */
	16*(nbseg-1) + /* pour V2k de V22 a V2N */
	16*(nbseg-2)   /* pour Pkm1 de P2m1 * PNm1m1 */
									                )) ;


    if (buffer == NULL) return 1 ;

	SM = buffer ;
	bufV2k = buffer + 4*nbseg ;
	bufPkm1 = bufV2k + 16*(nbseg-1) ;

	S1 = SM ;
    SN = SM+4*(nbseg-1);

	/* Precalculs des matrices */

	V2k = bufV2k ;  /* Pointe sur V22 = V2 */
	cpy_vect(V2k, Mat44, 16) ; /* Memo V22 */


	Pkm1 = bufPkm1-16 ;
	for (k = 2 ; k < nbseg ; k++)
	{
		V2k += 16 ;  /* de V23 a V2N */
		Pkm1 += 16 ; /* de P2m1 a PNm1m1

		/* Calcul de Vk et Pkm1 */
		dk = pUf[k] ; dk2 = dk*dk ; dk3 = dk2*dk, dk4 = dk3*dk ;

		Vk[13] = Pkm1[12] = -dk2; 
		Vk[14] = Pkm1[13] = 3.*dk ; 
		Vk[9] = Pkm1[8] =  2.*dk ; 
		Vk[0] = Vk[4] = Vk[8] = Vk[12]= 0. ; 
		Vk[5] = Vk[15] = Pkm1[4] = Pkm1[14] = -3. ;
		Vk[10]= Pkm1[9] = -5. ; 
		Pkm1[15]= 1./dk ;
		Vk[1] =  Vk[11]= Pkm1[0] = Pkm1[10]= 4./dk ;
		Vk[6] = Pkm1[5] = 6./dk ;
		Pkm1[11] = -1./dk2;
		Vk[2] = Pkm1[1] = -6./dk2 ; 
		Vk[7] = Pkm1[6] = -4./dk2 ;
		Pkm1[7] = 1./dk3;
		Vk[3] = Pkm1[2] =  4./dk3 ;
		Pkm1[3] = -1/dk4 ;

		/* V2k = V2*V3*..*Vk */
		mult_mat(V2k, Mat44, Vk, 4, 4, 4) ;
	    cpy_vect(Mat44, V2k, 16) ; /* Inutile au dernier tour */
	}
	mult_mat (Work, Mat44, TN, 4, 4, 1) ;
	cpy_vect (W, R1, 12) ;
	cpy_vect (W + 12, Work, 4) ;

	/* Resolution pour les differentes coordonnees */
    n = dimsp ;
    while(n--)
    {
        /* Second membre de dimension 4*nbseg */
		S1[0] = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		S1[1] = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		S1[2] = - (C1[n]+3.*D1[n]*d1) ;
        S1[3] = -D1[n] ;
		for (Sk = S1+4, k = 1 ; k < nbseg-1 ; k++, Sk += 4)
		{
			Sk[0] = Coefs[k+1][n] - Coefs[k][n];
			Sk[1] = 0. ;
			Sk[2] = 0. ;
			Sk[3] = 0. ;
		}
		SN[0] = AN1[n] - AN[n] ;
		SM[1] = BN1[n] ;
		SM[2] = CN1[n] ;
		SM[3] = DN1[n] ;	
	}


	/* S1 = S1 + V2*S2 */
	V2k = bufV2k ;  /* Pointe sur V22 = V2 */
	mult_mat(Work, V2k, S1+4, 4, 4, 1) ;
	add_vect(S1, S1, Work, 4) ;

	for (Sk = S1+4, k = 2 ; k < nbseg ; k++, Sk += 4)
	{		
		/*  V2k*Sk */
		V2k += 16 ; /* de V23 a V2N */
		mult_mat(Work, V2k, Sk, 4, 4, 1) ;
		/* S1 = S1 + V2k*Sk */
		add_vect(S1, S1, Work, 4) ;
	}

			/* Resolution Wx=S1 */
	cpy_vect (Vk, W, 16) ;
	resok = solv_lu(S1, Vk, 4, 1, 0) ;
	if (!resok) { free(buffer) ; return 1 ; }
	E1[n] = S1[0] ;
	F1[n] = S1[1] ;
	G1[n] = S1[2] ;
	FN[n] = S1[3] ;

		/* ans = SN-TN*FN */
	cum_vect (SN, SN,-FN[n], TN, 4) ;
	/* ans = PNm1 * (SN-TN*FN) */
	mult_mat(Work, PNm1, SN, 4, 4, 1);
	BN[n] = Work[0] ;
	CN[n] = Work[1] ;
	DN[n] = Work[2] ;
	EN[n] = Work[3] ;

	Pkm1 = bufPkm1 + 16*(nbseg-3) ; /* dernier cad P(N-1)m1 */
	for (Sk = SN-4, k = nbseg-2; k >= 1 ; k--, Sk -= 4)
	{
		/* ans = Sk + Work(1:3) */
		cpy_vect(Sk+1, Work+1, 3) ;
		/* ans = P2m1*(Sk + ..) */
		mult_mat(Work, Pkm1, Sk, 4, 4, 1) ;
		Pkm1 -= 16 ;

		Ak = Coefs[k] ; Bk=Ak+dimsp; Ck=Bk+dimsp ; Dk=Ck+dimsp ; Ek=Dk+dimsp ;
		Bk[n] = Work[0] ;
		Ck[n] = Work[1] ;
		Dk[n] = Work[2] ;
		Ek[n] = Work[3] ;
	}

    free(buffer) ;
	return 0 ;

#else  /* Version par decomposition LU  REPUTEE MARCHER */

double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1 ;

double dk, dk2, dk3, dk4, dk5;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1 = AN1+n, *CN1 = BN1+n, *DN1 = CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk ;
   
int dimS = 4*nbseg ;			/* Dimension S : Nb de contraintes */

int resok, k, m, m1;

double *X, *Mat, *Sec, *buffer ;

    if (nbseg < 3) return 1 ;
    
	buffer = (double *) calloc (dimS*(dimS+dimsp),sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + dimS*dimS ;

    /* 1ere colonne M1 */
    m = m1 = 0 ;
	Mat[m++] = d14 ;
	Mat[m++] = 4.*d13 ;
	Mat[m++] = 6.*d12 ;
	Mat[m  ] = 4.*d1 ;

    /* 2eme colonne M1 */
	m1 += dimS ; m = m1 ;
	Mat[m++] = d15 ;
	Mat[m++] = 5.*d14 ;
	Mat[m++] = 10.*d13 ;
	Mat[m  ] = 10.*d12 ;

    /* 3eme colonne M1 */
	m1 += dimS ; m = m1 ;
	Mat[m++] = d16 ;
	Mat[m++] = 6.*d15 ;
	Mat[m++] = 15.*d14 ;
	Mat[m  ] = 20.*d13 ;

	m1 += dimS ; m = m1 ; /* Pour debut de Q */

	for (k = 1; k < nbseg ; k++)
	{
		dk   = pUf[k] ;
		dk2  = dk *dk ;
		dk3  = dk2*dk ;
		dk4  = dk3*dk ;
		
		/* Elements de Q */
		Mat[m+1] = -1 ;
		Mat[m+dimS+2] = -1 ;
		Mat[m+2*dimS+3] = -1 ;


		/* Elements de Mk */
		m1 += 4 ; m = m1 ;
		/* 1ere colonne Mk */
		Mat[m++] = dk ;
		Mat[m++] = 1. ;
		Mat[m++] = 0. ;
		Mat[m  ] = 0. ;
		
		/* 2eme colonne Mk */
	    m1 += dimS ; m = m1 ;
		Mat[m++] = dk2 ;
		Mat[m++] = 2.*dk ;
		Mat[m++] = 1. ;
		Mat[m  ] = 0.;
		
		/* 3eme colonne Mk */
	    m1 += dimS ; m = m1 ;
		Mat[m++] = dk3 ;
		Mat[m++] = 3.*dk2 ;
		Mat[m++] = 3.*dk ;
		Mat[m  ] = 1.  ;
 
		/* 4eme colonne Mk */
	    m1 += dimS ; m = m1 ;
		Mat[m++] = dk4 ;
		Mat[m++] = 4.*dk3 ;
		Mat[m++] = 6.*dk2 ;
		Mat[m  ] = 4.*dk ;

	  if (k == nbseg-1)
	  {
		dk5  = dk4*dk ; 
		/* 5eme colonne Mk */
		m1 += dimS ; m = m1 ;
		Mat[m++] = dk5 ;
		Mat[m++] = 5.*dk4 ;
		Mat[m++] = 10.*dk3 ;
		Mat[m  ] = 10.*dk2 ;
	  }
	    m1 += dimS ; m = m1 ; /* Pour debut de Q */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	X = Sec ;
	while(n--)
	{
		/* Second membre de dimension 4*nbseg */
		*X++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*X++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*X++ = - (C1[n]+3.*D1[n]*d1) ;
		*X++ = - D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, X += 4)
		{
			*X = Coefs[k+1][n] - Coefs[k][n];
		}
		*X++ = AN1[n] - AN[n] ;
		*X++ = BN1[n] ;
		*X++ = CN1[n] ;
		*X++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, dimS, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

	n = dimsp ;
	X = Sec ;
	while(n--)
	{

		E1[n] = *X++ ;
		F1[n] = *X++ ;
		G1[n] = *X++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk = Ak + dimsp ;
			Ck = Bk + dimsp ; 
			Dk = Ck + dimsp ;
			Ek = Dk + dimsp ;

			Bk[n] = *X++ ;
			Ck[n] = *X++ ;
			Dk[n] = *X++ ;
			Ek[n] = *X++ ;
	        if (k == nbseg-1) {Fk = Ek + dimsp ; Fk[n] = *X++ ;  }
		}
	}

    free(buffer) ;
	return 0 ;
#endif
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 entre N+1 points par N pols de degr 5,4,...,4,6         */
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*--------------------------------------------------------------------------*/
int trNp546_c3 (double **Coefs, double *pUf, int nbseg, int dimsp)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 5 coefficients
		(avec dimsp composantes) des polynomes de degre 4, sauf pour :
		le 1	6 coefs (degre 5)
		le nbseg 7 coefs (degre 6)
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : echec allocation
*/
/*--------------------------------------------------------------------------*/
{
#if 0   /* Version resolution par blocs a debugger */

int resok, k, n=dimsp /* jB=n, jC=jB+n, jD=jC+n, jE=jD+n, jF=jE+n */;

double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1;

double R1[] = {d14, 4.*d13,  6.*d12,  4.*d1,
			   d15, 5.*d14,	10.*d13, 10.*d12};

double d2=pUf[1], d22=d2*d2, d23=d22*d2;

double Mat44[] = {0.,  4./d2 , -6./d22,  4./d23,
				  0., -3.,      6./d2,  -4./d22,
				  0.,  2.*d2 , -5.,      4./d2,
				  0.,    -d22,  3.*d2,  -3.     }; 

double dN=pUf[nbseg-1], dN2=dN*dN, dN3=dN2*dN, dN4=dN3*dN, dN5=dN4*dN, dN6=dN5*dN ;

double TN[] = {dN5, 5.*dN4, 10.*dN3, 10.*dN2,
			   dN6, 6.*dN5, 15.*dN4, 20.*dN3};

double PNm1[]={ 4./dN , -6./dN2,  4./dN3, -1/dN4,
			   -3.    ,  6./dN , -4./dN2, 1./dN3,
				2.*dN , -5.    ,  4./dN, -1./dN2
				  -dN2,  3.*dN , -3.,     1./dN     };

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n; 

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1], *BN=AN+n, *CN=BN+n, *DN=CN+n, *EN=DN+n, 
       *FN=EN+n, *GN=FN+n ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek ;

double W[16], Work[4], Vk[16], *bufV2k, *V2k, *bufPkm1, *Pkm1 ;

double *SM, *S1, *Sk, *SN ;

double dk, dk2, dk3, dk4 ;

double *buffer = (double *) malloc (sizeof(double)*(
	4*nbseg + /* pour SM */
	16*(nbseg-1) + /* pour V2k de V22 a V2N */
	16*(nbseg-2)   /* pour Pkm1 de P2m1 * PNm1m1 */
									                )) ;


    if (buffer == NULL) return 1 ;

	SM = buffer ;
	bufV2k = buffer + 4*nbseg ;
	bufPkm1 = bufV2k + 16*(nbseg-1) ;

	S1 = SM ;
    SN = SM+4*(nbseg-1);

	/* Precalculs des matrices */

	V2k = bufV2k ;  /* Pointe sur V22 = V2 */
	cpy_vect(V2k, Mat44, 16) ; /* Memo V22 */


	Pkm1 = bufPkm1-16 ;
	for (k = 2 ; k < nbseg ; k++)
	{
		V2k += 16 ;  /* de V23 a V2N */
		Pkm1 += 16 ; /* de P2m1 a PNm1m1

		/* Calcul de Vk et Pkm1 */
		dk = pUf[k] ; dk2 = dk*dk ; dk3 = dk2*dk, dk4 = dk3*dk ;

		Vk[13] = Pkm1[12] = -dk2; 
		Vk[14] = Pkm1[13] = 3.*dk ; 
		Vk[9] = Pkm1[8] =  2.*dk ; 
		Vk[0] = Vk[4] = Vk[8] = Vk[12]= 0. ; 
		Vk[5] = Vk[15] = Pkm1[4] = Pkm1[14] = -3. ;
		Vk[10]= Pkm1[9] = -5. ; 
		Pkm1[15]= 1./dk ;
		Vk[1] =  Vk[11]= Pkm1[0] = Pkm1[10]= 4./dk ;
		Vk[6] = Pkm1[5] = 6./dk ;
		Pkm1[11] = -1./dk2;
		Vk[2] = Pkm1[1] = -6./dk2 ; 
		Vk[7] = Pkm1[6] = -4./dk2 ;
		Pkm1[7] = 1./dk3;
		Vk[3] = Pkm1[2] =  4./dk3 ;
		Pkm1[3] = -1/dk4 ;

		/* V2k = V2*V3*..*Vk */
		mult_mat(V2k, Mat44, Vk, 4, 4, 4) ;
	    cpy_vect(Mat44, V2k, 16) ; /* Inutile au dernier tour */
	}
	mult_mat (Work, Mat44, TN, 4, 4, 2) ;
	cpy_vect (W, R1, 8) ;
	cpy_vect (W + 8, Work, 8) ;

	/* Resolution pour les differentes coordonnees */
    n = dimsp ;
    while(n--)
    {
        /* Second membre de dimension 4*nbseg */
		S1[0] = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		S1[1] = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		S1[2] = - (C1[n]+3.*D1[n]*d1) ;
        S1[3] = -D1[n] ;
		for (Sk = S1+4, k = 1 ; k < nbseg-1 ; k++, Sk += 4)
		{
			Sk[0] = Coefs[k+1][n] - Coefs[k][n];
			Sk[1] = 0. ;
			Sk[2] = 0. ;
			Sk[3] = 0. ;
		}
		SN[0] = AN1[n] - AN[n] ;
		SM[1] = BN1[n] ;
		SM[2] = CN1[n] ;
		SM[3] = DN1[n] ;	
	}


	/* S1 = S1 + V2*S2 */
	V2k = bufV2k ;  /* Pointe sur V22 = V2 */
	mult_mat(Work, V2k, S1+4, 4, 4, 1) ;
	add_vect(S1, S1, Work, 4) ;

	for (Sk = S1+4, k = 2 ; k < nbseg ; k++, Sk += 4)
	{		
		/*  V2k*Sk */
		V2k += 16 ; /* de V23 a V2N */
		mult_mat(Work, V2k, Sk, 4, 4, 1) ;
		/* S1 = S1 + V2k*Sk */
		add_vect(S1, S1, Work, 4) ;
	}

			/* Resolution Wx=S1 */
	cpy_vect (Vk, W, 16) ;
	resok = solv_lu(S1, Vk, 4, 1, 0) ;
	if (!resok) { free(buffer) ; return 1 ; }
	E1[n] = S1[0] ;
	F1[n] = S1[1] ;
	FN[n] = S1[2] ;
	GN[n] = S1[3] ;

		/* ans = SN-TN*ZN */
	mult_mat (Work, TN, S1+2, 4, 2, 1) ;
	add_vect (SN, SN, Work, 4) ;
	/* ans = PNm1 * (SN-TN*FN) */
	mult_mat(Work, PNm1, SN, 4, 4, 1);
	BN[n] = Work[0] ;
	CN[n] = Work[1] ;
	DN[n] = Work[2] ;
	EN[n] = Work[3] ;

	Pkm1 = bufPkm1 + 16*(nbseg-3) ; /* dernier cad P(N-1)m1 */
	for (Sk = SN-4, k = nbseg-2; k >= 1 ; k--, Sk -= 4)
	{
		/* ans = Sk + Work(1:3) */
		cpy_vect(Sk+1, Work+1, 3) ;
		/* ans = P2m1*(Sk + ..) */
		mult_mat(Work, Pkm1, Sk, 4, 4, 1) ;
		Pkm1 -= 16 ;

		Ak = Coefs[k] ; Bk=Ak+dimsp; Ck=Bk+dimsp ; Dk=Ck+dimsp ; Ek=Dk+dimsp ;
		Bk[n] = Work[0] ;
		Ck[n] = Work[1] ;
		Dk[n] = Work[2] ;
		Ek[n] = Work[3] ;
	}

    free(buffer) ;
	return 0 ;

#else  /* Version par decomposition LU   REPUTEE MARCHER */

double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1 ;

double dk, dk2, dk3, dk4, dk5, dk6;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1 = AN1+n, *CN1 = BN1+n, *DN1 = CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int dimS = 4*nbseg ;			/* Dimension S : Nb de contraintes */

int resok, k, m, m1;

double *X, *Mat, *Sec, *buffer ;

    if (nbseg < 3) return 1 ;
    
	buffer = (double *) calloc (dimS*(dimS+dimsp),sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + dimS*dimS ;

    /* 1ere colonne M1 */
    m = m1 = 0 ;
	Mat[m++] = d14 ;
	Mat[m++] = 4.*d13 ;
	Mat[m++] = 6.*d12 ;
	Mat[m  ] = 4.*d1 ;

    /* 2eme colonne M1 */
	m1 += dimS ; m = m1 ;
	Mat[m++] = d15 ;
	Mat[m++] = 5.*d14 ;
	Mat[m++] = 10.*d13 ;
	Mat[m  ] = 10.*d12 ;

	m1 += dimS ; m = m1 ; /* Pour debut de Q */

	for (k = 1; k < nbseg ; k++)
	{
		dk   = pUf[k] ;
		dk2  = dk *dk ;
		dk3  = dk2*dk ;
		dk4  = dk3*dk ;
		
		/* Elements de Q */
		Mat[m+1] = -1 ;
		Mat[m+dimS+2] = -1 ;
		Mat[m+2*dimS+3] = -1 ;


		/* Elements de Mk */
		m1 += 4 ; m = m1 ;
		/* 1ere colonne Mk */
		Mat[m++] = dk ;
		Mat[m++] = 1. ;
		Mat[m++] = 0. ;
		Mat[m  ] = 0. ;
		
		/* 2eme colonne Mk */
	    m1 += dimS ; m = m1 ;
		Mat[m++] = dk2 ;
		Mat[m++] = 2.*dk ;
		Mat[m++] = 1. ;
		Mat[m  ] = 0.;
		
		/* 3eme colonne Mk */
	    m1 += dimS ; m = m1 ;
		Mat[m++] = dk3 ;
		Mat[m++] = 3.*dk2 ;
		Mat[m++] = 3.*dk ;
		Mat[m  ] = 1.  ;
 
		/* 4eme colonne Mk */
	    m1 += dimS ; m = m1 ;
		Mat[m++] = dk4 ;
		Mat[m++] = 4.*dk3 ;
		Mat[m++] = 6.*dk2 ;
		Mat[m  ] = 4.*dk ;

	  if (k == nbseg-1)
	  {
		dk5 = dk4*dk ; 
		dk6 = dk5*dk ;
		/* 5eme colonne Mk */
		m1 += dimS ; m = m1 ;
		Mat[m++] = dk5 ;
		Mat[m++] = 5.*dk4 ;
		Mat[m++] = 10.*dk3 ;
		Mat[m  ] = 10.*dk2 ;

		/* 6eme colonne Mk */
		m1 += dimS ; m = m1 ;
		Mat[m++] = dk6 ;
		Mat[m++] = 6.*dk5 ;
		Mat[m++] = 15.*dk4 ;
		Mat[m  ] = 20.*dk3 ;
	  }
	    m1 += dimS ; m = m1 ; /* Pour debut de Q */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	X = Sec ;
	while(n--)
	{
		/* Second membre de dimension 4*nbseg */
		*X++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*X++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*X++ = - (C1[n]+3.*D1[n]*d1) ;
		*X++ = - D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, X += 4)
		{
			*X = Coefs[k+1][n] - Coefs[k][n];
		}
		*X++ = AN1[n] - AN[n] ;
		*X++ = BN1[n] ;
		*X++ = CN1[n] ;
		*X++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, dimS, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

	n = dimsp ;
	X = Sec ;
	while(n--)
	{

		E1[n] = *X++ ;
		F1[n] = *X++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk = Ak + dimsp ;
			Ck = Bk + dimsp ; 
			Dk = Ck + dimsp ;
			Ek = Dk + dimsp ;

			Bk[n] = *X++ ;
			Ck[n] = *X++ ;
			Dk[n] = *X++ ;
			Ek[n] = *X++ ;
	        if (k == nbseg-1) 
			{
				Fk = Ek + dimsp ; Fk[n] = *X++ ;  
			    Gk = Fk + dimsp ; Gk[n] = *X++ ;
			}
		}
	}

    free(buffer) ;
	return 0 ;
#endif
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 6												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminCp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 2)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1;

double v, dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int deg1m3 = 3, degkm1 = 5, degnm1 = 5 ;

int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY,					/* dimension Y */
	dimX = nbseg-1,			/* dimension X */
    nbx,					/* ordre systeme */
	nbxx2, nbxx3, nbxx4, nbx2, nbxp1, nbxp2, nbxp3, nbxp4, nbxm4 ;

int resok, k, h, ht, m, mt, ml, r ;

double *S, *Sk, *Y, *Mat, *Sec, *buffer ;

    if (nbseg < 2) return 1 ;

    dimY = deg1m3 + degkm1*(nbseg-2) + degnm1 ;	/* dimension Y */
    nbx = dimS+dimY+dimX ; 	/* ordre systeme */
	nbxx2 = nbx*2 ;
	nbxx3 = nbx*3 ;
	nbxx4 = nbx*4 ;
	nbx2 = nbx*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxm4 = nbx-4 ;
    
	buffer = (double *) calloc (
	nbx2		/* pour Mat : Matrice du systeme a resoudre */
	+ nbx*dimsp	/* pour Sec : Seconds membres du systeme */
	,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx2 ;

	/* Constitution de la matrice H M 0 / 0 R -Mt / 0 0 Ht */

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne -M1t */
    m = nbx*dimX ;						/* pte debut M1 */
	mt = ml = nbx*(dimX+dimY)+dimS ;	/* pte debut -M1t */
	v = d14   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 6.*d12; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d1 ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

    /* 2eme colonne M1 et 2eme ligne -M1t */
	m += nbxm4 ; ml = mt+1 ;
	v = d15   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 5.*d14; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d12; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

    /* 3eme colonne M1 et 3eme ligne -M1t (si degre 6) */
	m += nbxm4 ; ml = mt+2 ;
	v =  d16  ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 6.*d15; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =15.*d14; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =20.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

	m += nbxm4 ; mt += deg1m3 ; /* Ready pour Q/M2  et -Qt -M2t */

	/* Elements de R1 */
    r = nbx*dimX + dimS ;
	Mat[r]                        = 7.2*d15 ;
	Mat[r+1] = Mat[r+nbx]         = 10.*d16 ;
	Mat[r+nbx+1]                  = 100.*d177 ;
	Mat[r+2] = Mat[r+nbxx2]       = 90.*d177 ;
	Mat[r+nbx+2] = Mat[r+nbxx2+1] = 18.75*d18 ;
	Mat[r+nbxx2+2]                = 25.*d19 ;

	r += nbxx3+3;		/*Ready pour R2 */
    
	for (k = 1, 
		 h = 0,
		 ht = nbx*(dimX+dimY)+nbx+dimS+dimY ;
		 k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de H et Ht */
		Mat[h+1] = Mat[ht] = -1 ;
		ht += nbxx3 ;
		Mat[h+4] = Mat[ht] = dk ;
		ht += nbx ;
		Mat[h+5] = Mat[ht] = 1 ;
		ht++ ; h += nbxp4 ;

		/* Elements de Q et -Qt */
		Mat[m+2] = -1 ;
		Mat[m+nbxp3] = -1 ;
		Mat[mt+nbxx2] = -1 ;
		Mat[mt+nbxx3+1] = -1 ;


		/* Elements de Mk et -Mkt */
		m += 4 ; mt += nbxx4 ; /* pte sur Mk et -Mkt */
		ml = mt ;
		v = dk2   ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 2.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 0.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		
		/* 2eme colonne Mk et 2eme ligne -Mkt */
		m += nbxm4 ; ml = mt+1 ;
		v = dk3   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
 
		/* 3eme colonne Mk et 3eme ligne -Mkt */
		m += nbxm4 ; ml = mt+2 ;
		v = dk4   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 6.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

		/* 4eme colonne Mk et 4eme ligne -Mkt */
		m += nbxm4 ; ml = mt+3 ;
		v = dk5   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 5.*dk4; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

		/* 5eme colonne Mk et meme ligne -Mkt (si degre > 5) */
		m += nbxm4 ; ml = mt+4 ;
		v =  dk6  ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 6.*dk5; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =15.*dk4; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =20.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

		m += nbxm4 ; mt += degkm1 ; /* Ready pour Q/Mk+1  et -Qt -M(k+1)t */

		/* Elements de Rk */
		Mat[r] = dk ;
		
		Mat[r+1] = Mat[r+nbx] = 0.5*dk2 ;
		Mat[r+nbxp1] = dk3/3. ;

		Mat[r+2] = Mat[r+nbxx2] = 2.*dk3 ;
		Mat[r+nbxp2] = Mat[r+nbxx2+1] = 1.5*dk4 ;
		Mat[r+nbxx2+2] = 7.2*dk5 ;

		Mat[r+3] = Mat[r+nbxx3] = 2.5*dk4 ;
		Mat[r+nbxp3] = Mat[r+nbxx3+1] = 2.*dk5 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = 10.*dk6 ;
		Mat[r+nbxx3+3] = 100.*dk77 ;

		Mat[r+4] = Mat[r+nbxx4] = 3.*dk5 ;
		Mat[r+nbxp4] = Mat[r+nbxx4+1] = 2.5*dk6 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = 90.*dk77 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = 18.75*dk8 ;
		Mat[r+nbxx4+4] = 25.*dk9 ;

		r += nbxp1*degkm1;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	Sk = Sec ;
	while(n--)
	{
		S = Sk ;
		/* Second membre de dimension 4*nbseg */
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	

		*S++ = -(2.*d13*C1[n]+1.5*d14*D1[n]) ;
		*S++ = -(2.5*d14*C1[n]+2.*d15*D1[n]) ;
		*S = -(3.*d15*C1[n]+2.5*d16*D1[n]) ;

		Sk += nbx ;
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Sk = Sec ;
	while(n--)
	{
		Y = Sk + dimX ;

		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		G1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; Gk=Fk+dimsp ;

			Bk[n] = Sk[k-1] ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
			Gk[n] = *Y++ ;
		}
		Sk += nbx ;
	}

    free(buffer) ; 
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 5												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                             MODULE REPUTE MARCHER                        */
int trNminCp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 6 coefficients
		(avec dimsp composantes) des polynomes de degre 5, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 4)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.;

double v, dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Akp1 ;
   
int deg1m3 = 2, degkm1 = 4, degnm1 = 4 ;

int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY,					/* dimension Y */
	dimX = nbseg-1,			/* dimension X */
    nbx,					/* ordre systeme */
	nbxx2, nbxx3, nbxx4, nbxx5, nbx2, nbxp1, nbxp2, nbxp3, nbxp4, nbxm3 ;

int resok, k, h, ht, m, mt, ml, r ;

double *S, *Sk, *Y, *Mat, *Sec, *buffer ;

    if (nbseg < 4) return 1 ;

    dimY = deg1m3 + degkm1*(nbseg-2) + degnm1 ;	/* dimension Y */
    nbx = dimS+dimY+dimX ; 	/* ordre systeme */
	nbxx2 = nbx*2 ;
	nbxx3 = nbx*3 ;
	nbxx4 = nbx*4 ;
	nbxx5 = nbx*5 ;
	nbx2 = nbx*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxm3 = nbx-3 ;
    
	buffer = (double *) calloc (
	nbx2		/* pour Mat : Matrice du systeme a resoudre */
	+ nbx*dimsp	/* pour Sec : Seconds membres du systeme */
	,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx2 ;

	/* Constitution de la matrice H M 0 / 0 R -Mt / 0 0 Ht */

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne -M1t */
    m = nbx*dimX ;						/* pte debut M1 */
	mt = ml = nbx*(dimX+dimY)+dimS ;	/* pte debut -M1t */
	v = d14   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 6.*d12; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d1 ; Mat[m] = v ; Mat[ml] = v ; 

    /* 2eme colonne M1 et 2eme ligne -M1t */
	m += nbxm3 ; ml = mt+1 ;
	v = d15   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 5.*d14; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d12; Mat[m] = v ; Mat[ml] = v ;

	m += nbxm3 ; mt += deg1m3 ; /* Ready pour Q/M2  et -Qt -M2t */

	/* Elements de R1 */
    r = nbx*dimX + dimS ;
	Mat[r] = 7.2*d15 ;
	Mat[r+1] = Mat[r+nbx] = 10.*d16 ;
	Mat[r+nbx+1] = 100.*d177 ;

	r += nbxx2+2;		/*Ready pour R2 */
    
	for (k = 1, 
		 h = 0,
		 ht = nbx*(dimX+dimY)+dimS+dimY ;
		 k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		
		/* Elements de H et Ht */
		Mat[h+1] = Mat[ht+nbx] = -1 ;
		Mat[h+4] = Mat[ht+nbxx4] = dk ;
		Mat[h+5] = Mat[ht+nbxx5] = 1 ;
		h += nbxp4 ; ht += nbxx4 + 1 ;

		/* Elements de Q et -Qt */
		Mat[m+2] = -1 ;
		Mat[m+nbxp3] = -1 ;
		Mat[mt+nbxx2] = -1 ;
		Mat[mt+nbxx3+1] = -1 ;


		/* Elements de Mk et -Mkt */
		m += 4 ; mt += nbxx4 ; /* pte sur Mk et -Mkt */
		ml = mt ;
		v = dk2   ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 2.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 0.    ; Mat[m] = v ; Mat[ml] = v ;
		
		/* 2eme colonne Mk et 2eme ligne -Mkt */
		m += nbxm3 ; ml = mt+1 ;
		v = dk3   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m] = v ; Mat[ml] = v ; 
 
		/* 3eme colonne Mk et 3eme ligne -Mkt */
		m += nbxm3 ; ml = mt+2 ;
		v = dk4   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 6.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk ; Mat[m] = v ; Mat[ml] = v ;

		/* 4eme colonne Mk et 4eme ligne -Mkt */
		m += nbxm3 ; ml = mt+3 ;
		v = dk5   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 5.*dk4; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk2; Mat[m] = v ; Mat[ml] = v ;

		m += nbxm3 ; mt += degkm1 ; /* Ready pour Q/Mk+1  et -Qt -M(k+1)t */

		/* Elements de Rk */
		Mat[r] = dk ;
		
		Mat[r+1] = Mat[r+nbx] = 0.5*dk2 ;
		Mat[r+nbxp1] = dk3/3. ;

		Mat[r+2] = Mat[r+nbxx2] = 2.*dk3 ;
		Mat[r+nbxp2] = Mat[r+nbxx2+1] = 1.5*dk4 ;
		Mat[r+nbxx2+2] = 7.2*dk5 ;

		Mat[r+3] = Mat[r+nbxx3] = 2.5*dk4 ;
		Mat[r+nbxp3] = Mat[r+nbxx3+1] = 2.*dk5 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = 10.*dk6 ;
		Mat[r+nbxx3+3] = 100.*dk77 ;

		r += nbxp1*degkm1;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	Sk = Sec ;
	while(n--)
	{
		S = Sk ;
		/* Second membre de dimension 4*nbseg */
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++)
		{
			Ak = Coefs[k] ; Akp1 = Coefs[k+1] ;
			*S = Akp1[n] - Ak[n];
			S += 4 ;
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	

		*S++ = -(2.*d13*C1[n]+1.5*d14*D1[n]) ;
		*S   = -(2.5*d14*C1[n]+2.*d15*D1[n]) ;

		Sk += nbx ;
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Sk = Sec ;
	while(n--)
	{
		Y = Sk + dimX ;

		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk = Ak + dimsp ; Ck = Bk + dimsp ; 
			Dk = Ck + dimsp ; Ek = Dk + dimsp ;
			Fk = Ek + dimsp ;

			Bk[n] = Sk[k-1] ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
		}
		Sk += nbx ;
	}

    free(buffer) ;
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 5 et 6 (externes)								*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminCp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double rx)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

  SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 3)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1;

double v, dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1 = AN1+n, *CN1 = BN1+n, *DN1 = CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int deg1m3 = 3, degkm1 = 4, degnm1 = 5 ;

int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY,					/* dimension Y */
	dimX = nbseg-1,			/* dimension X */
    nbx,					/* ordre systeme */
	nbxx2, nbxx3, nbxx4, nbx2, nbxp1, nbxp2, nbxp3, nbxp4, nbxm4 ;

int resok, k, h, ht, m, mt, ml, r ;

double *S, *Sk, *Y, *Mat, *Sec, *buffer ;

    if (nbseg < 3) return 1 ;

    dimY = deg1m3 + degkm1*(nbseg-2) + degnm1 ;	/* dimension Y */
    nbx = dimS+dimY+dimX ; 	/* ordre systeme */
	nbxx2 = nbx*2 ;
	nbxx3 = nbx*3 ;
	nbxx4 = nbx*4 ;
	nbx2 = nbx*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxm4 = nbx-4 ;
    
	buffer = (double *) calloc (
	nbx2		/* pour Mat : Matrice du systeme a resoudre */
	+ nbx*dimsp	/* pour Sec : Seconds membres du systeme */
	,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx2 ;

	/* Constitution de la matrice H M 0 / 0 R -Mt / 0 0 Ht */

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne -M1t */
    m = nbx*dimX ;						/* pte debut M1 */
	mt = ml = nbx*(dimX+dimY)+dimS ;	/* pte debut -M1t */
	v = d14   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 6.*d12; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d1 ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

    /* 2eme colonne M1 et 2eme ligne -M1t */
	m += nbxm4 ; ml = mt+1 ;
	v = d15   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 5.*d14; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d12; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

    /* 3eme colonne M1 et 3eme ligne -M1t */
	m += nbxm4 ; ml = mt+2 ;
	v =  d16  ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 6.*d15; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =15.*d14; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =20.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

	m += nbxm4 ; mt += deg1m3 ; /* Ready pour Q/M2  et -Qt -M2t */

	/* Elements de R1 */
    r = nbx*dimX + dimS ;
	Mat[r]                        = rx*7.2*d15 ;
	Mat[r+1] = Mat[r+nbx]         = rx*10.*d16 ;
	Mat[r+nbx+1]                  = rx*100.*d177 ;

	Mat[r+2] = Mat[r+nbxx2]       = rx*90.*d177 ;
	Mat[r+nbx+2] = Mat[r+nbxx2+1] = rx*18.75*d18 ;
	Mat[r+nbxx2+2]                = rx*25.*d19 ;

	r += nbxx3+3;		/*Ready pour R2 */
    
	for (k = 1, 
		 h = 0,
		 ht = nbx*(dimX+dimY)+nbx+dimS+dimY ;
		 k < nbseg ; k++)
	{
		dk   = pUf[k] ;
		dk2  = dk *dk ;
		dk3  = dk2*dk ;
		dk4  = dk3*dk ;
		dk5  = dk4*dk ; 
        dk6  = dk5*dk ;
		dk7  = dk6*dk ;
		dk77 = dk7/7. ;
		
		/* Elements de H et Ht */
		Mat[h+1] = Mat[ht] = -1 ;
		ht += nbxx3 ;
		Mat[h+4] = Mat[ht] = dk ;
		ht += nbx ;
		Mat[h+5] = Mat[ht] = 1 ;
		ht++ ; h += nbxp4 ;

		/* Elements de Q et -Qt */
		Mat[m+2] = -1 ;
		Mat[m+nbxp3] = -1 ;
		Mat[mt+nbxx2] = -1 ;
		Mat[mt+nbxx3+1] = -1 ;


		/* Elements de Mk et -Mkt */
		m += 4 ; mt += nbxx4 ; /* pte sur Mk et -Mkt */
		ml = mt ;
		v = dk2   ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 2.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 0.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		
		/* 2eme colonne Mk et 2eme ligne -Mkt */
		m += nbxm4 ; ml = mt+1 ;
		v = dk3   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
 
		/* 3eme colonne Mk et 3eme ligne -Mkt */
		m += nbxm4 ; ml = mt+2 ;
		v = dk4   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 6.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

		/* 4eme colonne Mk et 4eme ligne -Mkt */
		m += nbxm4 ; ml = mt+3 ;
		v = dk5   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 5.*dk4; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

		/* 5eme colonne Mk et meme ligne -Mkt (si degre > 5) */
	  if (k == nbseg-1)
	  {
		m += nbxm4 ; ml = mt+4 ;
		v =  dk6  ; Mat[m++] = Mat[ml] = v ; ml +=nbx;
		v = 6.*dk5; Mat[m++] = Mat[ml] = v ; ml +=nbx;
		v =15.*dk4; Mat[m++] = Mat[ml] = v ; ml +=nbx;
		v =20.*dk3; Mat[m++] = Mat[ml] = v ; ml +=nbx;
	  }

		m += nbxm4 ; mt += degkm1 ; /* Ready pour Q/Mk+1  et -Qt -M(k+1)t */

		/* Elements de Rk */
		if (k != nbseg-1)
		{
			Mat[r] = dk ;
			
			Mat[r+1] = Mat[r+nbx] = 0.5*dk2 ;
			Mat[r+nbxp1] = dk3/3. ;

			Mat[r+2] = Mat[r+nbxx2] = 2.*dk3 ;
			Mat[r+nbxp2] = Mat[r+nbxx2+1] = 1.5*dk4 ;
			Mat[r+nbxx2+2] = 7.2*dk5 ;

			Mat[r+3] = Mat[r+nbxx3] = 2.5*dk4 ;
			Mat[r+nbxp3] = Mat[r+nbxx3+1] = 2.*dk5 ;
			Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = 10.*dk6 ;
			Mat[r+nbxx3+3] = 100.*dk77 ;

		}
		else
		{
			dk8 = dk7*dk;
			dk9 = dk8*dk;

			Mat[r]                          = rx*dk ;
			
			Mat[r+1] = Mat[r+nbx]           = rx*0.5*dk2 ;
			Mat[r+nbxp1]                    = rx*dk3/3. ;

			Mat[r+2] = Mat[r+nbxx2]         = rx*2.*dk3 ;
			Mat[r+nbxp2] = Mat[r+nbxx2+1]   = rx*1.5*dk4 ;
			Mat[r+nbxx2+2]                  = rx*7.2*dk5 ;

			Mat[r+3] = Mat[r+nbxx3]         = rx*2.5*dk4 ;
			Mat[r+nbxp3] = Mat[r+nbxx3+1]   = rx*2.*dk5 ;
			Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = rx*10.*dk6 ;
			Mat[r+nbxx3+3]                  = rx*100.*dk77 ;

			Mat[r+4] = Mat[r+nbxx4]         = rx*3.*dk5 ;
			Mat[r+nbxp4] = Mat[r+nbxx4+1]   = rx*2.5*dk6 ;
			Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = rx*90.*dk77 ;
			Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = rx*18.75*dk8 ;
			Mat[r+nbxx4+4]                  = rx*25.*dk9 ;
		}

		r += nbxp1*degkm1;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	Sk = Sec ;
	while(n--)
	{
		S = Sk ;
		/* Second membre de dimension 4*nbseg */
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = - D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	

		*S++ = -rx*(2.*d13*C1[n]+1.5*d14*D1[n]) ;
		*S++ = -rx*(2.5*d14*C1[n]+2.*d15*D1[n]) ;
		*S   = -rx*(3.*d15*C1[n]+2.5*d16*D1[n]) ;

		Sk += nbx ;
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Sk = Sec ;
	while(n--)
	{
		Y = Sk + dimX ;

		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		G1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk = Ak + dimsp ;
			Ck = Bk + dimsp ; 
			Dk = Ck + dimsp ;
			Ek = Dk + dimsp ;
			Fk = Ek + dimsp ; 

			Bk[n] = Sk[k-1] ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
	        if (k == nbseg-1) {Gk = Fk + dimsp ; Gk[n] = *Y ;  }
		}
		Sk += nbx ;
	}

    free(buffer) ;
	return 0 ;
}
#if 0  /* Regroupement en 1 seul algo des 3 prcdents */
/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de courbure entre N+1 points			*/
/*       par N pols de degr 6 ou 5 selon la valeur de ver					*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int c3splinopt (double **Coefs, double *pUf, int nbseg, int dimsp, int ver)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

    ver		: version
				1 : polynmes internes de degre 5 (suppose nbseg >= 3)
				2 : tous les polynmes de degre 5 (suppose nbseg >= 4)
				autre : tous les polynomes de degre 6

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 2)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1;

double v, dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int deg1m3 = 3, degkm1 = 5, degnm1 = 5 ;

int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY,					/* dimension Y */
	dimX = nbseg-1,			/* dimension X */
    nbx,					/* ordre systeme */
	nbxx2, nbxx3, nbxx4, nbx2, nbxp1, nbxp2, nbxp3, nbxp4, nbxm4 ;

int resok, k, h, ht, m, mt, ml, r, test ;

double *S, *Sk, *Y, *Mat, *Sec, *buffer ;

    if (nbseg < 2) return 1 ;

    if (ver != 1 && ver != 2) ver = 0 ;
    if (ver == 1 && nbseg < 4) ver = 2 ;
	if (ver == 2 && nbseg < 3) ver = 0 ;
	if (ver) degkm1 = 4 ;
	if (ver == 2) {deg1m3 = 2; degnm1 = 4;}

    dimY = deg1m3 + degkm1*(nbseg-2) + degnm1 ;	/* dimension Y */
    nbx = dimS+dimY+dimX ; 	/* ordre systeme */
	nbxx2 = nbx*2 ;
	nbxx3 = nbx*3 ;
	nbxx4 = nbx*4 ;
	nbx2 = nbx*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxm4 = nbx-4 ;
    
	buffer = (double *) calloc (
	nbx2		/* pour Mat : Matrice du systeme a resoudre */
	+ nbx*dimsp	/* pour Sec : Seconds membres du systeme */
	,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx2 ;

	/* Constitution de la matrice H M 0 / 0 R -Mt / 0 0 Ht */

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne -M1t */
    m = nbx*dimX ;						/* pte debut M1 */
	mt = ml = nbx*(dimX+dimY)+dimS ;	/* pte debut -M1t */
	v = d14   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 6.*d12; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 4.*d1 ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

    /* 2eme colonne M1 et 2eme ligne -M1t */
	m += nbxm4 ; ml = mt+1 ;
	v = d15   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 5.*d14; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =10.*d12; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

    /* 3eme colonne M1 et 3eme ligne -M1t (si degre 6) */
  if (deg1m3 > 2)
  {
	m += nbxm4 ; ml = mt+2 ;
	v =  d16  ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v = 6.*d15; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =15.*d14; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	v =20.*d13; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
  }

	m += nbxm4 ; mt += deg1m3 ; /* Ready pour Q/M2  et -Qt -M2t */

	/* Elements de R1 */
    r = nbx*dimX + dimS ;
	Mat[r] = 7.2*d15 ;
	Mat[r+1] = Mat[r+nbx] = 10.*d16 ;
	Mat[r+nbx+1] = 100.*d177 ;
  if (deg1m3 > 2)
  {
	Mat[r+2] = Mat[r+nbxx2] = 90.*d177 ;
	Mat[r+nbx+2] = Mat[r+nbxx2+1] = 18.75*d18 ;
	Mat[r+nbxx2+2] = 25.*d19 ;
  }

  r += ver==2 ? nbxx2+2 : nbxx3+3;		/*Ready pour R2 */
    
	for (k = 1, 
		 h = 0,
		 ht = nbx*(dimX+dimY)+nbx+dimS+dimY ;
		 k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de H et Ht */
		Mat[h+1] = Mat[ht] = -1 ;
		ht += nbxx3 ;
		Mat[h+4] = Mat[ht] = dk ;
		ht += nbx ;
		Mat[h+5] = Mat[ht] = 1 ;
		ht++ ; h += nbxp4 ;

		/* Elements de Q et -Qt */
		Mat[m+2] = -1 ;
		Mat[m+nbxp3] = -1 ;
		Mat[mt+nbxx2] = -1 ;
		Mat[mt+nbxx3+1] = -1 ;


		/* Elements de Mk et -Mkt */
		m += 4 ; mt += nbxx4 ; /* pte sur Mk et -Mkt */
		ml = mt ;
		v = dk2   ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 2.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 0.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		
		/* 2eme colonne Mk et 2eme ligne -Mkt */
		m += nbxm4 ; ml = mt+1 ;
		v = dk3   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 3.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 1.    ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
 
		/* 3eme colonne Mk et 3eme ligne -Mkt */
		m += nbxm4 ; ml = mt+2 ;
		v = dk4   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 6.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 4.*dk ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

		/* 4eme colonne Mk et 4eme ligne -Mkt */
		m += nbxm4 ; ml = mt+3 ;
		v = dk5   ;	Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 5.*dk4; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =10.*dk2; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;

		/* 5eme colonne Mk et meme ligne -Mkt (si degre > 5) */
		test = k+1-nbseg ;
		test = (test && degnm1 > 4) || (!test && degkm1 > 4) ;
	  if (test)
	  {
		m += nbxm4 ; ml = mt+4 ;
		v =  dk6  ; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v = 6.*dk5; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =15.*dk4; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
		v =20.*dk3; Mat[m++] = v ; Mat[ml] = v ; ml +=nbx;
	  }

		m += nbxm4 ; mt += degkm1 ; /* Ready pour Q/Mk+1  et -Qt -M(k+1)t */

		/* Elements de Rk */
		Mat[r] = dk ;
		
		Mat[r+1] = Mat[r+nbx] = 0.5*dk2 ;
		Mat[r+nbxp1] = dk3/3. ;

		Mat[r+2] = Mat[r+nbxx2] = 2.*dk3 ;
		Mat[r+nbxp2] = Mat[r+nbxx2+1] = 1.5*dk4 ;
		Mat[r+nbxx2+2] = 7.2*dk5 ;

		Mat[r+3] = Mat[r+nbxx3] = 2.5*dk4 ;
		Mat[r+nbxp3] = Mat[r+nbxx3+1] = 2.*dk5 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = 10.*dk6 ;
		Mat[r+nbxx3+3] = 100.*dk77 ;

	  if (test)
	  {
		Mat[r+4] = Mat[r+nbxx4] = 3.*dk5 ;
		Mat[r+nbxp4] = Mat[r+nbxx4+1] = 2.5*dk6 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = 90.*dk77 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = 18.75*dk8 ;
		Mat[r+nbxx4+4] = 25.*dk9 ;
	  }

		r += nbxp1*degkm1;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	Sk = Sec ;
	while(n--)
	{
		S = Sk ;
		/* Second membre de dimension 4*nbseg */
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	

		*S++ = -(2.*d13*C1[n]+1.5*d14*D1[n]) ;
		*S = -(2.5*d14*C1[n]+2.*d15*D1[n]) ;
		if (ver != 2) {S++ ; *S = -(3.*d15*C1[n]+2.5*d16*D1[n]) ;}

		Sk += nbx ;
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Sk = Sec ;
	while(n--)
	{
		Y = Sk + dimX ;

		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		if (deg1m3 > 2) G1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; Gk=Fk+dimsp ;

			test = k+1-nbseg ;
		    test = (test && degnm1 > 4) || (!test && degkm1 > 4) ;


			Bk[n] = Sk[k-1] ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
		   if (test) 
			Gk[n] = *Y++ ;
		}
		Sk += nbx ;
	}

    free(buffer) ;
	return 0 ;
}
#endif
/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur entre N+1 points			*/
/*       par N pols de degr 6												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminLp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 2)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1, d110=d19*d1,
							 d111=d110*d1;

double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9, dk10, dk11 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,nbxx5,
	nbxp1, nbxp2, nbxp3, nbxp4, nbxp5 ;		/* Ordre du systme */

int resok, k, m, mc, mt, ml, r ;

double *S, *Y, *Mat, *Sec, *buffer ;

    if (nbseg < 2) return 1 ;

    dimY = 3 + 6*(nbseg-1) ;	/* dimension Y */
    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxx5 = 5*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxp5 = nbx+5 ;


	buffer = (double *) calloc (nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx*nbx ;

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne M1t */
    mc = dimY ;	mt = nbx*dimY ;	
	m = mc ; ml = mt ;
	Mat[m++] = Mat[ml] = d14    ; ml +=nbx;
	Mat[m++] = Mat[ml] = 4.*d13 ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d12 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 4.*d1  ;
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 2eme colonne M1 et 2eme ligne M1t */
	Mat[m++] = Mat[ml] = d15     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 5.*d14  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 10.*d13 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 10.*d12 ; 
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 3eme colonne M1 et 3eme ligne M1t */
	Mat[m++] = Mat[ml] = d16     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d15  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 15.*d14 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 20.*d13 ;

	mc += nbx ; mt++ ;  /* Ready pour Q/M2  et Qt/M2t */

	/* Elements de R1 */
    r = 0 ;
	Mat[r] = 16.*d177 ;
	Mat[r+1] = Mat[r+nbx] = 2.5*d18 ;
	Mat[r+nbx+1] = 25.*d19/9. ;
	Mat[r+2] = Mat[r+nbxx2] = 8.*d19/3. ;
	Mat[r+nbx+2] = Mat[r+nbxx2+1] = 3.*d110 ;
	Mat[r+nbxx2+2] = 36.*d111/11. ;

	r += nbxx3+3;		/*Ready pour R2 */
    
	for (k = 1; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		dk10 = dk9*dk ;
		dk11 = dk10*dk ;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 6eme colonne Mk et 6eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk6     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk5  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 15.*dk4 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 20.*dk3 ;

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  dk ;
		
		Mat[r+1]       = Mat[r+nbx]     = dk2 ;
		Mat[r+nbxp1]   =                  4.*dk3/3. ;

		Mat[r+2]       = Mat[r+nbxx2]   = dk3 ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = 1.5*dk4 ;
		Mat[r+nbxx2+2] =                  1.8*dk5 ;

		Mat[r+3]       = Mat[r+nbxx3]   = dk4 ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = 1.6*dk5 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = 2.*dk6 ;
		Mat[r+nbxx3+3] =                  16.*dk77 ;

		Mat[r+4]       = Mat[r+nbxx4]   = dk5 ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = 5.*dk6/3. ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = 15.*dk77 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = 2.5*dk8 ;
		Mat[r+nbxx4+4] =                  25.*dk9/9. ;

		Mat[r+5]       = Mat[r+nbxx5]   = dk6 ;
		Mat[r+nbxp5]   = Mat[r+nbxx5+1] = 12.*dk77 ;
		Mat[r+nbxx2+5] = Mat[r+nbxx5+2] = 2.25*dk8 ;
		Mat[r+nbxx3+5] = Mat[r+nbxx5+3] = 8.*dk9/3. ;
		Mat[r+nbxx4+5] = Mat[r+nbxx5+4] = 3.*dk10 ;
		Mat[r+nbxx5+5] =                  36.*dk11/11 ;

		r += nbxp1*6;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{

	    S[0] = -(2.*d14*B1[n]+3.2*d15*C1[n]+4.*d16*D1[n]) ;
		S[1] = -(2.*d15*B1[n]+10.*d16*C1[n]/3.+30.*d177*D1[n]) ;
		S[2] = -(2.*d16*B1[n]+24.*d177*C1[n]+4.5*d18*D1[n]) ;

		/* Second membre de dimension 4*nbseg */
		S += dimY ;
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		G1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; Gk=Fk+dimsp ;

			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
			Gk[n] = *Y++ ;
		}
		
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur entre N+1 points			*/
/*       par N pols de degr 6 externes et 5 internes						*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminLp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double rx)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord (>= 4)

    dimsp	: Dimension de l'espace de la trajectoire

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 3)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1, d110=d19*d1,
							 d111=d110*d1;

double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9, dk10, dk11 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,nbxx5,
	nbxp1, nbxp2, nbxp3, nbxp4, nbxp5 ;		/* Ordre du systme */

int resok, k, m, mc, mt, ml, r ;

double *S, *Y, *Mat, *Sec, *buffer ;

    if (nbseg < 3) return 1 ;

    dimY = 3 + 5*(nbseg-2) + 6 ;	/* dimension Y */
    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxx5 = 5*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxp5 = nbx+5 ;


	buffer = (double *) calloc (nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx*nbx ;

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne M1t */
    mc = dimY ;	mt = nbx*dimY ;	
	m = mc ; ml = mt ;
	Mat[m++] = Mat[ml] = d14    ; ml +=nbx;
	Mat[m++] = Mat[ml] = 4.*d13 ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d12 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 4.*d1  ;
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 2eme colonne M1 et 2eme ligne M1t */
	Mat[m++] = Mat[ml] = d15     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 5.*d14  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 10.*d13 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 10.*d12 ; 
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 3eme colonne M1 et 3eme ligne M1t */
	Mat[m++] = Mat[ml] = d16     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d15  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 15.*d14 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 20.*d13 ;

	mc += nbx ; mt++ ;  /* Ready pour Q/M2  et Qt/M2t */

	/* Elements de R1 */
    r = 0 ;
	Mat[r]                        = rx*16.*d177 ;
	Mat[r+1] = Mat[r+nbx]         = rx*2.5*d18 ;
	Mat[r+nbx+1]                  = rx*25.*d19/9. ;
	Mat[r+2] = Mat[r+nbxx2]       = rx*8.*d19/3. ;
	Mat[r+nbx+2] = Mat[r+nbxx2+1] = rx*3.*d110 ;
	Mat[r+nbxx2+2]                = rx*36.*d111/11. ;

	r += nbxx3+3;		/*Ready pour R2 */
    
	for (k = 1; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 

		if (k == nbseg-1) 
		{
			mc += nbx ; mt++ ;
			m = mc ; ml = mt ;


			/* 6eme colonne Mk et 6eme ligne Mkt */
			Mat[m++] = Mat[ml] = dk6     ; ml +=nbx;
			Mat[m++] = Mat[ml] = 6.*dk5  ; ml +=nbx;
			Mat[m++] = Mat[ml] = 15.*dk4 ; ml +=nbx;
			Mat[m  ] = Mat[ml] = 20.*dk3 ;
		}

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		if (k != nbseg-1)
		{
			Mat[r]         =                  dk ;
			
			Mat[r+1]       = Mat[r+nbx]     = dk2 ;
			Mat[r+nbxp1]   =                  4.*dk3/3. ;

			Mat[r+2]       = Mat[r+nbxx2]   = dk3 ;
			Mat[r+nbxp2]   = Mat[r+nbxx2+1] = 1.5*dk4 ;
			Mat[r+nbxx2+2] =                  1.8*dk5 ;

			Mat[r+3]       = Mat[r+nbxx3]   = dk4 ;
			Mat[r+nbxp3]   = Mat[r+nbxx3+1] = 1.6*dk5 ;
			Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = 2.*dk6 ;
			Mat[r+nbxx3+3] =                  16.*dk77 ;

			Mat[r+4]       = Mat[r+nbxx4]   = dk5 ;
			Mat[r+nbxp4]   = Mat[r+nbxx4+1] = 5.*dk6/3. ;
			Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = 15.*dk77 ;
			Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = 2.5*dk8 ;
			Mat[r+nbxx4+4] =                  25.*dk9/9. ;
		}
		else
		{
			dk10 = dk9*dk ;
			dk11 = dk10*dk ;

			Mat[r]							= rx*dk ;
			
			Mat[r+1]       = Mat[r+nbx]     = rx*dk2 ;
			Mat[r+nbxp1]					= rx*4.*dk3/3. ;

			Mat[r+2]       = Mat[r+nbxx2]   = rx*dk3 ;
			Mat[r+nbxp2]   = Mat[r+nbxx2+1] = rx*1.5*dk4 ;
			Mat[r+nbxx2+2]					= rx*1.8*dk5 ;

			Mat[r+3]       = Mat[r+nbxx3]   = rx*dk4 ;
			Mat[r+nbxp3]   = Mat[r+nbxx3+1] = rx*1.6*dk5 ;
			Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = rx*2.*dk6 ;
			Mat[r+nbxx3+3]					= rx*16.*dk77 ;

			Mat[r+4]       = Mat[r+nbxx4]   = rx*dk5 ;
			Mat[r+nbxp4]   = Mat[r+nbxx4+1] = rx*5.*dk6/3. ;
			Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = rx*15.*dk77 ;
			Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = rx*2.5*dk8 ;
			Mat[r+nbxx4+4]					= rx*25.*dk9/9. ;

			Mat[r+5]       = Mat[r+nbxx5]   = rx*dk6 ;
			Mat[r+nbxp5]   = Mat[r+nbxx5+1] = rx*12.*dk77 ;
			Mat[r+nbxx2+5] = Mat[r+nbxx5+2] = rx*2.25*dk8 ;
			Mat[r+nbxx3+5] = Mat[r+nbxx5+3] = rx*8.*dk9/3. ;
			Mat[r+nbxx4+5] = Mat[r+nbxx5+4] = rx*3.*dk10 ;
			Mat[r+nbxx5+5]					= rx*36.*dk11/11 ;
		}

		r += nbxp1*5;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{

	    S[0] = -rx*(2.*d14*B1[n]+3.2*d15*C1[n]+4.*d16*D1[n]) ;
		S[1] = -rx*(2.*d15*B1[n]+10.*d16*C1[n]/3.+30.*d177*D1[n]) ;
		S[2] = -rx*(2.*d16*B1[n]+24.*d177*C1[n]+4.5*d18*D1[n]) ;

		/* Second membre de dimension 4*nbseg */
		S += dimY ;
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		G1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; 

			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
			if (k == nbseg-1)
			{
				Gk=Fk+dimsp ;
				Gk[n] = *Y++ ;
			}
		}
		
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}
/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur entre N+1 points			*/
/*       par N pols de degr 5												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminLp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord (>= 4)

    dimsp	: Dimension de l'espace de la trajectoire

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 4)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
/* WORD duree ; */
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1 ;

double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk ;
   
int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,
	nbxp1, nbxp2, nbxp3, nbxp4 ;		/* Ordre du systme */

int resok, k, m, mc, mt, ml, r ;

double *S, *Y, *Mat, *Sec, *buffer ;

    if (nbseg < 4) return 1 ;

    dimY = 2 + 5*(nbseg-1) ;	/* dimension Y */
    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;


	buffer = (double *) calloc (nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx*nbx ;

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne M1t */
    mc = dimY ;	mt = nbx*dimY ;	
	m = mc ; ml = mt ;
	Mat[m++] = Mat[ml] = d14    ; ml +=nbx;
	Mat[m++] = Mat[ml] = 4.*d13 ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d12 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 4.*d1  ;
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 2eme colonne M1 et 2eme ligne M1t */
	Mat[m++] = Mat[ml] = d15     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 5.*d14  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 10.*d13 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 10.*d12 ; 

	mc += nbx ; mt++ ;  /* Ready pour Q/M2  et Qt/M2t */

	/* Elements de R1 */
    r = 0 ;
	Mat[r] = 16.*d177 ;
	Mat[r+1] = Mat[r+nbx] = 2.5*d18 ;
	Mat[r+nbx+1] = 25.*d19/9. ;

	r += nbxx2+2;		/*Ready pour R2 */
    
	for (k = 1; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  dk ;
		
		Mat[r+1]       = Mat[r+nbx]     = dk2 ;
		Mat[r+nbxp1]   =                  4.*dk3/3. ;

		Mat[r+2]       = Mat[r+nbxx2]   = dk3 ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = 1.5*dk4 ;
		Mat[r+nbxx2+2] =                  1.8*dk5 ;

		Mat[r+3]       = Mat[r+nbxx3]   = dk4 ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = 1.6*dk5 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = 2.*dk6 ;
		Mat[r+nbxx3+3] =                  16.*dk77 ;

		Mat[r+4]       = Mat[r+nbxx4]   = dk5 ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = 5.*dk6/3. ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = 15.*dk77 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = 2.5*dk8 ;
		Mat[r+nbxx4+4] =                  25.*dk9/9. ;

		r += nbxp1*5;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{

	    S[0] = -(2.*d14*B1[n]+3.2*d15*C1[n]+4.*d16*D1[n]) ;
		S[1] = -(2.*d15*B1[n]+10.*d16*C1[n]/3.+30.*d177*D1[n]) ;

		/* Second membre de dimension 4*nbseg */
		S += dimY ;
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}
/*	duree = GetTickCount() ; */
	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

/*	resok = choll(Sec, Mat, nbx, dimsp) ;
	if (resok < 0) { free(buffer) ; return 3 ; } */

/*	duree = GetTickCount() - duree ; */

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; 

			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
		}
		
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur et/ou courbure entre N+1	*/
/*      points par N pols de degr 6												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminLCp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double muc)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

    muc     : partie courbure de la minimisation  0 <= muc <= 1.
	           (1-muc)*crit_longueur + muc*crit_courbure 

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 2)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1, d110=d19*d1,
							 d111=d110*d1;

double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9, dk10, dk11 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,nbxx5,
	nbxp1, nbxp2, nbxp3, nbxp4, nbxp5 ;		/* Ordre du systme */

int resok, k, m, mc, mt, ml, r ;

double *S, *Y, *Mat, *Sec, *buffer ;
double mul, d2tot ;

    if (nbseg < 2) return 1 ;

	muc = muc < 0. ? 0. : muc ;
	muc = muc > 1. ? 1. : muc ;
	mul = 1. - muc ;

	d2tot = 0. ;
    for (m = 0 ; m < nbseg ; m++) d2tot += pUf[m]*pUf[m] ;
	muc *= d2tot/nbseg ;
	

    dimY = 3 + 6*(nbseg-1) ;	/* dimension Y */
    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxx5 = 5*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxp5 = nbx+5 ;


	buffer = (double *) calloc (nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx*nbx ;

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne M1t */
    mc = dimY ;	mt = nbx*dimY ;	
	m = mc ; ml = mt ;
	Mat[m++] = Mat[ml] = d14    ; ml +=nbx;
	Mat[m++] = Mat[ml] = 4.*d13 ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d12 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 4.*d1  ;
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 2eme colonne M1 et 2eme ligne M1t */
	Mat[m++] = Mat[ml] = d15     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 5.*d14  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 10.*d13 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 10.*d12 ; 
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 3eme colonne M1 et 3eme ligne M1t */
	Mat[m++] = Mat[ml] = d16     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d15  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 15.*d14 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 20.*d13 ;

	mc += nbx ; mt++ ;  /* Ready pour Q/M2  et Qt/M2t */

	/* Elements de R1 */
    r = 0 ;
	Mat[r]                        = mul*16.*d177    + muc*7.2*d15 ;
	Mat[r+1] = Mat[r+nbx]         = mul*2.5*d18     + muc*10.*d16 ;
	Mat[r+nbx+1]                  = mul*25.*d19/9.  + muc*100.*d177 ;
	Mat[r+2] = Mat[r+nbxx2]       = mul*8.*d19/3.   + muc*90.*d177;
	Mat[r+nbx+2] = Mat[r+nbxx2+1] = mul*3.*d110     + muc*18.75*d18;
	Mat[r+nbxx2+2]                = mul*36.*d111/11.+ muc*25.*d19 ;

	r += nbxx3+3;		/*Ready pour R2 */
    
	for (k = 1; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		dk10 = dk9*dk ;
		dk11 = dk10*dk ;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 6eme colonne Mk et 6eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk6     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk5  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 15.*dk4 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 20.*dk3 ;

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  mul*dk          ;
		
		Mat[r+1]       = Mat[r+nbx]     = mul*dk2         ;
		Mat[r+nbxp1]   =                  mul*4.*dk3/3.  + muc*dk ;

		Mat[r+2]       = Mat[r+nbxx2]   = mul*dk3         ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = mul*1.5*dk4    + muc*0.5*dk2;
		Mat[r+nbxx2+2] =                  mul*1.8*dk5    + muc*dk3/3 ;

		Mat[r+3]       = Mat[r+nbxx3]   = mul*dk4         ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = mul*1.6*dk5    + muc*2.*dk3 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = mul*2.*dk6     + muc*1.5*dk4 ;
		Mat[r+nbxx3+3] =                  mul*16.*dk77   + muc*7.2*dk5 ;

		Mat[r+4]       = Mat[r+nbxx4]   = mul*dk5         ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = mul*5.*dk6/3.  + muc*2.5*dk4 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = mul*15.*dk77   + muc*2.*dk5 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = mul*2.5*dk8    + muc*10.*dk6 ;
		Mat[r+nbxx4+4] =                  mul*25.*dk9/9. + muc*100.*dk77 ;
 
		Mat[r+5]       = Mat[r+nbxx5]   = mul*dk6         ;
		Mat[r+nbxp5]   = Mat[r+nbxx5+1] = mul*12.*dk77   + muc*3.*dk5 ;
		Mat[r+nbxx2+5] = Mat[r+nbxx5+2] = mul*2.25*dk8   + muc*2.5*dk6 ;
		Mat[r+nbxx3+5] = Mat[r+nbxx5+3] = mul*8.*dk9/3.  + muc*90.*dk77 ;
		Mat[r+nbxx4+5] = Mat[r+nbxx5+4] = mul*3.*dk10    + muc*18.75*dk8 ;
		Mat[r+nbxx5+5] =                  mul*36.*dk11/11.+ muc*25.*dk9 ;

		r += nbxp1*6;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{

	    S[0] = -mul*(2.*d14*B1[n]+3.2*d15*C1[n]+4.*d16*D1[n]) 
			-muc*(2.*d13*C1[n]+1.5*d14*D1[n]) ;
		S[1] = -mul*(2.*d15*B1[n]+10.*d16*C1[n]/3.+30.*d177*D1[n]) 
			-muc*(2.5*d14*C1[n]+2.*d15*D1[n]) ;
		S[2] = -mul*(2.*d16*B1[n]+24.*d177*C1[n]+4.5*d18*D1[n]) 
			-muc*(3.*d15*C1[n]+2.5*d16*D1[n]) ;

		/* Second membre de dimension 4*nbseg */
		S += dimY ;
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		G1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; Gk=Fk+dimsp ;

			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
			Gk[n] = *Y++ ;
		}
		
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur et/ou courbure entre N+1	*/
/*      points par N pols de degr 5												*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminLCp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double muc)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

    muc     : partie courbure de la minimisation  0 <= muc <= 1.
	           (1-muc)*crit_longueur + muc*crit_courbure 

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 4)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1;

double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk ;
   
int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,
	nbxp1, nbxp2, nbxp3, nbxp4;		/* Ordre du systme */

int resok, k, m, mc, mt, ml, r ;

double *S, *Y, *Mat, *Sec, *buffer ;
double mul, d2tot ;

    if (nbseg < 4) return 1 ;

	muc = muc < 0. ? 0. : muc ;
	muc = muc > 1. ? 1. : muc ;
	mul = 1. - muc ;

	d2tot = 0. ;
    for (m = 0 ; m < nbseg ; m++) d2tot += pUf[m]*pUf[m] ;
	muc *= d2tot/nbseg ;
	

    dimY = 2 + 5*(nbseg-1) ;	/* dimension Y */
    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;


	buffer = (double *) calloc (nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx*nbx ;

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne M1t */
    mc = dimY ;	mt = nbx*dimY ;	
	m = mc ; ml = mt ;
	Mat[m++] = Mat[ml] = d14    ; ml +=nbx;
	Mat[m++] = Mat[ml] = 4.*d13 ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d12 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 4.*d1  ;
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 2eme colonne M1 et 2eme ligne M1t */
	Mat[m++] = Mat[ml] = d15     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 5.*d14  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 10.*d13 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 10.*d12 ; 

	mc += nbx ; mt++ ;  /* Ready pour Q/M2  et Qt/M2t */

	/* Elements de R1 */
    r = 0 ;
	Mat[r]                        = mul*16.*d177    + muc*7.2*d15 ;
	Mat[r+1] = Mat[r+nbx]         = mul*2.5*d18     + muc*10.*d16 ;
	Mat[r+nbx+1]                  = mul*25.*d19/9.  + muc*100.*d177 ;

	r += nbxx2+2;		/*Ready pour R2 */
    
	for (k = 1; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  mul*dk          ;
		
		Mat[r+1]       = Mat[r+nbx]     = mul*dk2         ;
		Mat[r+nbxp1]   =                  mul*4.*dk3/3.  + muc*dk ;

		Mat[r+2]       = Mat[r+nbxx2]   = mul*dk3         ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = mul*1.5*dk4    + muc*0.5*dk2;
		Mat[r+nbxx2+2] =                  mul*1.8*dk5    + muc*dk3/3 ;

		Mat[r+3]       = Mat[r+nbxx3]   = mul*dk4         ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = mul*1.6*dk5    + muc*2.*dk3 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = mul*2.*dk6     + muc*1.5*dk4 ;
		Mat[r+nbxx3+3] =                  mul*16.*dk77   + muc*7.2*dk5 ;

		Mat[r+4]       = Mat[r+nbxx4]   = mul*dk5         ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = mul*5.*dk6/3.  + muc*2.5*dk4 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = mul*15.*dk77   + muc*2.*dk5 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = mul*2.5*dk8    + muc*10.*dk6 ;
		Mat[r+nbxx4+4] =                  mul*25.*dk9/9. + muc*100.*dk77 ;
 
		r += nbxp1*5;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{

	    S[0] = -mul*(2.*d14*B1[n]+3.2*d15*C1[n]+4.*d16*D1[n]) 
			-muc*(2.*d13*C1[n]+1.5*d14*D1[n]) ;
		S[1] = -mul*(2.*d15*B1[n]+10.*d16*C1[n]/3.+30.*d177*D1[n]) 
			-muc*(2.5*d14*C1[n]+2.*d15*D1[n]) ;

		/* Second membre de dimension 4*nbseg */
		S += dimY ;
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ;

			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
		}
		
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec minimisation de longueur et/ou courbure entre N+1	*/
/*      points par N pols de degr 6 externes et 5 internes					*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
/*                         MODULE REPUTE MARCHER                            */
int trNminLCp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, double muc)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

    muc     : partie courbure de la minimisation  0 <= muc <= 1.
	           (1-muc)*crit_longueur + muc*crit_courbure 

SORTIES : (Tous les coefficients sont calculs)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 3)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double d1=pUf[0], d12=d1*d1, d13=d12*d1, d14=d13*d1, d15=d14*d1, 
                             d16=d15*d1, d17=d16*d1, d177=d17/7.,
							 d18=d17*d1, d19=d18*d1, d110=d19*d1,
							 d111=d110*d1;

double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9, dk10, dk11 ;

int n = dimsp ;

double *A1 = Coefs[0], *B1=A1+n, *C1=B1+n, *D1=C1+n, *E1=D1+n, *F1=E1+n,
		*G1=F1+n ;

double *A2 = Coefs[1] ;

double *AN = Coefs[nbseg-1] ; 

double *AN1 = Coefs[nbseg], *BN1=AN1+n, *CN1=BN1+n, *DN1=CN1+n ; 

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int dimS = 4*nbseg,			/* Dimension S : Nb de contraintes */
    dimY ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,nbxx5,
	nbxp1, nbxp2, nbxp3, nbxp4, nbxp5 ;		/* Ordre du systme */

int resok, k, m, mc, mt, ml, r ;

double *S, *Y, *Mat, *Sec, *buffer ;
double mul, d2tot ;

    if (nbseg < 3) return 1 ;

	muc = muc < 0. ? 0. : muc ;
	muc = muc > 1. ? 1. : muc ;
	mul = 1. - muc ;

	d2tot = 0. ;
    for (m = 0 ; m < nbseg ; m++) d2tot += pUf[m]*pUf[m] ;
	muc *= d2tot/nbseg ;
	

    dimY = 3 + 5*(nbseg-2) + 6  ;	/* dimension Y */
    nbx = dimS+dimY ; 			  /* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxx5 = 5*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxp5 = nbx+5 ;


	buffer = (double *) calloc (nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Mat = buffer ;
	Sec = Mat + nbx*nbx ;

	/* Elements de M1 et -M1t */

    /* 1ere colonne M1 et 1ere ligne M1t */
    mc = dimY ;	mt = nbx*dimY ;	
	m = mc ; ml = mt ;
	Mat[m++] = Mat[ml] = d14    ; ml +=nbx;
	Mat[m++] = Mat[ml] = 4.*d13 ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d12 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 4.*d1  ;
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 2eme colonne M1 et 2eme ligne M1t */
	Mat[m++] = Mat[ml] = d15     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 5.*d14  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 10.*d13 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 10.*d12 ; 
	mc += nbx ; mt++ ;
	m = mc ; ml = mt ;

    /* 3eme colonne M1 et 3eme ligne M1t */
	Mat[m++] = Mat[ml] = d16     ; ml +=nbx;
	Mat[m++] = Mat[ml] = 6.*d15  ; ml +=nbx;
	Mat[m++] = Mat[ml] = 15.*d14 ; ml +=nbx;
	Mat[m  ] = Mat[ml] = 20.*d13 ;

	mc += nbx ; mt++ ;  /* Ready pour Q/M2  et Qt/M2t */

	/* Elements de R1 */
    r = 0 ;
	Mat[r]                        = mul*16.*d177    + muc*7.2*d15 ;
	Mat[r+1] = Mat[r+nbx]         = mul*2.5*d18     + muc*10.*d16 ;
	Mat[r+nbx+1]                  = mul*25.*d19/9.  + muc*100.*d177 ;
	Mat[r+2] = Mat[r+nbxx2]       = mul*8.*d19/3.   + muc*90.*d177;
	Mat[r+nbx+2] = Mat[r+nbxx2+1] = mul*3.*d110     + muc*18.75*d18;
	Mat[r+nbxx2+2]                = mul*36.*d111/11.+ muc*25.*d19 ;

	r += nbxx3+3;		/*Ready pour R2 */
    
	for (k = 1; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 

		if (k == nbseg-1)
		{
			mc += nbx ; mt++ ;
			m = mc ; ml = mt ;

			/* 6eme colonne Mk et 6eme ligne Mkt */
			Mat[m++] = Mat[ml] = dk6     ; ml +=nbx;
			Mat[m++] = Mat[ml] = 6.*dk5  ; ml +=nbx;
			Mat[m++] = Mat[ml] = 15.*dk4 ; ml +=nbx;
			Mat[m  ] = Mat[ml] = 20.*dk3 ;
		}

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  mul*dk          ;
		
		Mat[r+1]       = Mat[r+nbx]     = mul*dk2         ;
		Mat[r+nbxp1]   =                  mul*4.*dk3/3.  + muc*dk ;

		Mat[r+2]       = Mat[r+nbxx2]   = mul*dk3         ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = mul*1.5*dk4    + muc*0.5*dk2;
		Mat[r+nbxx2+2] =                  mul*1.8*dk5    + muc*dk3/3 ;

		Mat[r+3]       = Mat[r+nbxx3]   = mul*dk4         ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = mul*1.6*dk5    + muc*2.*dk3 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = mul*2.*dk6     + muc*1.5*dk4 ;
		Mat[r+nbxx3+3] =                  mul*16.*dk77   + muc*7.2*dk5 ;

		Mat[r+4]       = Mat[r+nbxx4]   = mul*dk5         ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = mul*5.*dk6/3.  + muc*2.5*dk4 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = mul*15.*dk77   + muc*2.*dk5 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = mul*2.5*dk8    + muc*10.*dk6 ;
		Mat[r+nbxx4+4] =                  mul*25.*dk9/9. + muc*100.*dk77 ;
 
		if (k == nbseg-1)
		{
			dk10 = dk9*dk ;
			dk11 = dk10*dk ;

			Mat[r+5]       = Mat[r+nbxx5]   = mul*dk6         ;
			Mat[r+nbxp5]   = Mat[r+nbxx5+1] = mul*12.*dk77   + muc*3.*dk5 ;
			Mat[r+nbxx2+5] = Mat[r+nbxx5+2] = mul*2.25*dk8   + muc*2.5*dk6 ;
			Mat[r+nbxx3+5] = Mat[r+nbxx5+3] = mul*8.*dk9/3.  + muc*90.*dk77 ;
			Mat[r+nbxx4+5] = Mat[r+nbxx5+4] = mul*3.*dk10    + muc*18.75*dk8 ;
			Mat[r+nbxx5+5] =                  mul*36.*dk11/11.+ muc*25.*dk9 ;
		}

		r += nbxp1*5;		/*Ready pour Rk+1 */
	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{

	    S[0] = -mul*(2.*d14*B1[n]+3.2*d15*C1[n]+4.*d16*D1[n]) 
			-muc*(2.*d13*C1[n]+1.5*d14*D1[n]) ;
		S[1] = -mul*(2.*d15*B1[n]+10.*d16*C1[n]/3.+30.*d177*D1[n]) 
			-muc*(2.5*d14*C1[n]+2.*d15*D1[n]) ;
		S[2] = -mul*(2.*d16*B1[n]+24.*d177*C1[n]+4.5*d18*D1[n]) 
			-muc*(3.*d15*C1[n]+2.5*d16*D1[n]) ;

		/* Second membre de dimension 4*nbseg */
		S += dimY ;
		*S++ = A2[n]-(A1[n]+B1[n]*d1+C1[n]*d12+D1[n]*d13) ;
		*S++ = - (B1[n]+2.*C1[n]*d1+3.*D1[n]*d12) ;
		*S++ = - (C1[n]+3.*D1[n]*d1) ;
		*S++ = -D1[n] ;
		for (k = 1 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = AN1[n] - AN[n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		E1[n] = *Y++ ;
		F1[n] = *Y++ ;
		G1[n] = *Y++ ;
		for (k = 1 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ;

			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
			if (k == nbseg-1)
			{
				Gk=Fk+dimsp ;
				Gk[n] = *Y++ ;
			}
		}
		
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}


/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec lissage aux points et minimisation de longueur		*/
/*		et/ou  courbure entre N+1 points par N pols de degr 6				*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminDLCp6_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, 
				                        double *dM, double clis, double muc)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

    dM     : Tableau des prcisions de passage aux points

    clis    : Coefficient de lissage (0 : passage aux points, 1 filtrage max)
	          clis est ramen en interne sur 0 - infini

    muc     : coefficient minimisation de courbure  0 <= muc <= 1.


    Critere :  (1/clis)*((Ai-Mi)/dM)^2 + clis*[
	                 (1-muc)*crit_longueur + muc*d^2*crit_courbure  ]

             d^2  est un terme correctif pour rendre crit_longueur et
			 crit_courbure homognes.
               

SORTIES : (Tous les coefficients sont calculs y compris An+1)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 2)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double *A1 = Coefs[0], *B1=A1+dimsp, *C1=B1+dimsp, *D1=C1+dimsp ;
double *AN1 = Coefs[nbseg], *BN1=AN1+dimsp, *CN1=BN1+dimsp, *DN1=CN1+dimsp ; 


double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9, dk10, dk11 ;

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int dimS = 4*nbseg+3,		/* Dimension S : Nb de contraintes */
    dimY = 6*nbseg  ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,nbxx5,
	nbxp1, nbxp2, nbxp3, nbxp4, nbxp5 ;		/* Ordre du systme */

int n, resok, k, m, mc, mt, ml, r, e ;

double *S, *Y, *Mat, *Sec, *buffer, *Eps2 ;
double mul, d2tot ;

    if (nbseg < 2) return 1 ;

    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxx5 = 5*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxp5 = nbx+5 ;

	buffer = (double *) calloc (nbseg+1 + nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Eps2 = buffer ;
	Mat = Eps2 + nbseg+1 ;
	Sec = Mat + nbx*nbx ;


	/* clis = 1/lambda, avec lambda coef critre (A-M)^2 
	   clis ramen de (0, 0.5, 1.)  (0, 1., infini) */
	clis = clis < 0.0001 ? 0.0001 : clis ;
	clis = clis > 0.99 ? 0.99 : clis ;
	clis = tan (clis*M_PI_2);

	for (k = 0 ; k <= nbseg; k++) Eps2[k] = clis*dM[k]*dM[k] ;

	/* Correction mul et muc pour range 0-1 et somme = 1 */
	muc = muc < 0. ? 0. : muc ;
	muc = muc > 1. ? 1. : muc ;
	mul = 1. - muc ;

	/* Correction mu pour produit lambda*mu = 1 */
	muc *= clis ;
	mul *= clis ;

	/* Correction muc pour homognit avec mul */
	d2tot = 0. ;
    for (m = 0 ; m < nbseg ; m++) d2tot += pUf[m]*pUf[m] ;
	muc *= d2tot/nbseg ;

    mc = dimY-1 ;	        /* -1 par rapport au debut de Q */
	mt = nbx*(dimY-1) ;	    /* -1 par rapport au debut de Qt */
	m = mc ; ml = mt ;
    r = 0 ;				    /* debut de R */
	e = (dimY+3)*nbxp1 ;	/* debut de E */

	for (k = 0; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		dk10 = dk9*dk ;
		dk11 = dk10*dk ;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 6eme colonne Mk et 6eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk6     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk5  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 15.*dk4 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 20.*dk3 ;

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  mul*dk          ;
		
		Mat[r+1]       = Mat[r+nbx]     = mul*dk2         ;
		Mat[r+nbxp1]   =                  mul*4.*dk3/3.  + muc*dk ;

		Mat[r+2]       = Mat[r+nbxx2]   = mul*dk3         ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = mul*1.5*dk4    + muc*0.5*dk2;
		Mat[r+nbxx2+2] =                  mul*1.8*dk5    + muc*dk3/3 ;

		Mat[r+3]       = Mat[r+nbxx3]   = mul*dk4         ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = mul*1.6*dk5    + muc*2.*dk3 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = mul*2.*dk6     + muc*1.5*dk4 ;
		Mat[r+nbxx3+3] =                  mul*16.*dk77   + muc*7.2*dk5 ;

		Mat[r+4]       = Mat[r+nbxx4]   = mul*dk5         ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = mul*5.*dk6/3.  + muc*2.5*dk4 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = mul*15.*dk77   + muc*2.*dk5 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = mul*2.5*dk8    + muc*10.*dk6 ;
		Mat[r+nbxx4+4] =                  mul*25.*dk9/9. + muc*100.*dk77 ;
 
		Mat[r+5]       = Mat[r+nbxx5]   = mul*dk6         ;
		Mat[r+nbxp5]   = Mat[r+nbxx5+1] = mul*12.*dk77   + muc*3.*dk5 ;
		Mat[r+nbxx2+5] = Mat[r+nbxx5+2] = mul*2.25*dk8   + muc*2.5*dk6 ;
		Mat[r+nbxx3+5] = Mat[r+nbxx5+3] = mul*8.*dk9/3.  + muc*90.*dk77 ;
		Mat[r+nbxx4+5] = Mat[r+nbxx5+4] = mul*3.*dk10    + muc*18.75*dk8 ;
		Mat[r+nbxx5+5] =                  mul*36.*dk11/11.+ muc*25.*dk9 ;

		r += nbxp1*6;		/*Ready pour Rk+1 */

		/* Elements de E */
		Mat[e] = - (Eps2[k]+Eps2[k+1]) ;
		if (k) /* Pas pour k = 0 */
		{
			Mat[e-nbxx4] = Eps2[k] ;
			Mat[e-4] = Eps2[k] ;
		}

		e += 4*nbxp1 ;

	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{
		/* Second membre */
		S += dimY ;
		*S++ = - B1[n] ;
		*S++ = - C1[n] ;
		*S++ = - D1[n] ;
		for (k = 0 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = Coefs[k+1][n] - Coefs[k][n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		S = Y + dimY-1 ;
		for (k = 0 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; Gk=Fk+dimsp ;

		    if (k) Ak[n] += Eps2[k]*(S[0]-S[4]) ;
			else Ak[n] -= Eps2[k]*S[4] ;
			S += 4 ;
			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
			Gk[n] = *Y++ ;
		}
		AN1[n] += Eps2[k]*S[0] ;  /* Pour les tracs */
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec lissage aux points et minimisation de longueur		*/
/*		et/ou  courbure entre N+1 points par N pols de degr 5				*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminDLCp5_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, 
				                        double *dM, double clis, double muc)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 6 coefficients
		(avec dimsp composantes) des polynomes de degre 5, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

    dM     : Tableau des prcisions de passage aux points

    clis    : Coefficient de lissage (0 : passage aux points, 1 filtrage max)
	          clis est ramen en interne sur 0 - infini

    muc     : coefficient minimisation de courbure  0 <= muc <= 1.


    Critere :  (1/clis)*((Ai-Mi)/dM)^2 + clis*[
	                 (1-muc)*crit_longueur + muc*d^2*crit_courbure  ]

             d^2  est un terme correctif pour rendre crit_longueur et
			 crit_courbure homognes.
               

SORTIES : (Tous les coefficients sont calculs y compris An+1)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 4)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double *A1 = Coefs[0], *B1=A1+dimsp, *C1=B1+dimsp, *D1=C1+dimsp ;
double *AN1 = Coefs[nbseg], *BN1=AN1+dimsp, *CN1=BN1+dimsp, *DN1=CN1+dimsp ; 


double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9 ;

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk ;
   
int dimS = 4*nbseg+3,		/* Dimension S : Nb de contraintes */
    dimY = 5*nbseg  ,					/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,
	nbxp1, nbxp2, nbxp3, nbxp4 ;		/* Ordre du systme */

int n, resok, k, m, mc, mt, ml, r, e ;

double *S, *Y, *Mat, *Sec, *buffer, *Eps2 ;
double mul, d2tot ;

    if (nbseg < 4) return 1 ;

    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;

	buffer = (double *) calloc (nbseg+1 + nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Eps2 = buffer ;
	Mat = Eps2 + nbseg+1 ;
	Sec = Mat + nbx*nbx ;


	/* clis = 1/lambda, avec lambda coef critre (A-M)^2 
	   clis ramen de (0, 0.5, 1.)  (0, 1., infini) */
	clis = clis < 0.0001 ? 0.0001 : clis ;
	clis = clis > 0.99 ? 0.99 : clis ;
	clis = tan (clis*M_PI_2);

	for (k = 0 ; k <= nbseg; k++) Eps2[k] = clis*dM[k]*dM[k] ;

	/* Correction mul et muc pour range 0-1 et somme = 1 */
	muc = muc < 0. ? 0. : muc ;
	muc = muc > 1. ? 1. : muc ;
	mul = 1. - muc ;

	/* Correction mu pour produit lambda*mu = 1 */
	muc *= clis ;
	mul *= clis ;

	/* Correction muc pour homognit avec mul */
	d2tot = 0. ;
    for (m = 0 ; m < nbseg ; m++) d2tot += pUf[m]*pUf[m] ;
	muc *= d2tot/nbseg ;

    mc = dimY-1 ;	        /* -1 par rapport au debut de Q */
	mt = nbx*(dimY-1) ;	    /* -1 par rapport au debut de Qt */
	m = mc ; ml = mt ;
    r = 0 ;				    /* debut de R */
	e = (dimY+3)*nbxp1 ;	/* debut de E */

	for (k = 0; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1. ; 
		Mat[mc+nbxp2] = -1. ;
		Mat[mc+nbxx2+3] = -1. ;
		Mat[mt+nbx] = -1. ;
		Mat[mt+nbxx2+1] = -1. ;
		Mat[mt+nbxx3+2] = -1. ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  mul*dk          ;
		
		Mat[r+1]       = Mat[r+nbx]     = mul*dk2         ;
		Mat[r+nbxp1]   =                  mul*4.*dk3/3.  + muc*dk ;

		Mat[r+2]       = Mat[r+nbxx2]   = mul*dk3         ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = mul*1.5*dk4    + muc*0.5*dk2;
		Mat[r+nbxx2+2] =                  mul*1.8*dk5    + muc*dk3/3 ;

		Mat[r+3]       = Mat[r+nbxx3]   = mul*dk4         ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = mul*1.6*dk5    + muc*2.*dk3 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = mul*2.*dk6     + muc*1.5*dk4 ;
		Mat[r+nbxx3+3] =                  mul*16.*dk77   + muc*7.2*dk5 ;

		Mat[r+4]       = Mat[r+nbxx4]   = mul*dk5         ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = mul*5.*dk6/3.  + muc*2.5*dk4 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = mul*15.*dk77   + muc*2.*dk5 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = mul*2.5*dk8    + muc*10.*dk6 ;
		Mat[r+nbxx4+4] =                  mul*25.*dk9/9. + muc*100.*dk77 ;

		r += nbxp1*5;		/*Ready pour Rk+1 */

		/* Elements de E */
		Mat[e] = - (Eps2[k]+Eps2[k+1]) ;
		if (k) /* Pas pour k = 0 */
		{
			Mat[e-nbxx4] = Eps2[k] ;
			Mat[e-4] = Eps2[k] ;
		}

		e += 4*nbxp1 ;

	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{
		/* Second membre */
		S += dimY ;
		*S++ = - B1[n] ;
		*S++ = - C1[n] ;
		*S++ = - D1[n] ;
		for (k = 0 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = Coefs[k+1][n] - Coefs[k][n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) 
	{ 
		free(buffer) ; 
		return 3 ; 
	}

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		S = Y + dimY-1 ;
		for (k = 0 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ;

		    if (k) Ak[n] += Eps2[k]*(S[0]-S[4]) ;
			else Ak[n] -= Eps2[k]*S[4] ;
			S += 4 ;
			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
		}
		AN1[n] += Eps2[k]*S[0] ;  /* Pour les tracs */
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}

/*--------------------------------------------------------------------------*/
/*      RACCORD c3 avec lissage aux points et minimisation de longueur		*/
/*		et/ou  courbure entre N+1 points par N pols de degr 6 et 5 int.	*/
/*       Les coefficients b1,c1,d1 et bN+1,cN+1,dN+1 sont donns            */
/*      Renvoi 0 si ok.														*/
/*--------------------------------------------------------------------------*/
int trNminDLCp656_c3 (double **Coefs, double *pUf, int nbseg, int dimsp, 
				                        double *dM, double clis, double muc)
/*--------------------------------------------------------------------------*/
/*
ENTREES : Coefs, pUf, nbseg, dimsp
	Coefs	: Pointeur sur nbseg+1 blocs de doubles.
	    Dans chaque bloc il y au moins la place pour les 7 coefficients
		(avec dimsp composantes) des polynomes de degre 6, sauf pour
		le nbseg+1 qui peut ne contenir la place que pour 4 coefficients.

    pUf		: Tableau des valeurs finales des parametres des nbseg segments

    nbseg	: Nombre de segments du raccord

    dimsp	: Dimension de l'espace de la trajectoire

    dM     : Tableau des prcisions de passage aux points

    clis    : Coefficient de lissage (0 : passage aux points, 1 filtrage max)
	          clis est ramen en interne sur 0 - infini

    muc     : coefficient minimisation de courbure  0 <= muc <= 1.


    Critere :  (1/clis)*((Ai-Mi)/dM)^2 + clis*[
	                 (1-muc)*crit_longueur + muc*d^2*crit_courbure  ]

             d^2  est un terme correctif pour rendre crit_longueur et
			 crit_courbure homognes.
               

SORTIES : (Tous les coefficients sont calculs y compris An+1)
	0 : Calcul Ok
	1 : erreur donnes (nbseg < 3)
	2 : echec allocation
	3 : echec resolution
*/
/*--------------------------------------------------------------------------*/
{
double *A1 = Coefs[0], *B1=A1+dimsp, *C1=B1+dimsp, *D1=C1+dimsp ;
double *AN1 = Coefs[nbseg], *BN1=AN1+dimsp, *CN1=BN1+dimsp, *DN1=CN1+dimsp ; 


double dk, dk2, dk3, dk4, dk5, dk6, dk7, dk77, dk8, dk9, dk10, dk11 ;

double *Ak, *Bk, *Ck, *Dk, *Ek, *Fk, *Gk ;
   
int dimS = 4*nbseg+3,		/* Dimension S : Nb de contraintes */
    dimY = 5*nbseg+2,		/* dimension Y */
	nbx, nbxx2, nbxx3,nbxx4,nbxx5,
	nbxp1, nbxp2, nbxp3, nbxp4, nbxp5 ;		/* Ordre du systme */

int n, resok, k, m, mc, mt, ml, r, e ;

double *S, *Y, *Mat, *Sec, *buffer, *Eps2 ;
double mul, d2tot ;

    if (nbseg < 3) return 1 ;

    nbx = dimS+dimY ; 			/* ordre systeme */
	nbxx2 = 2*nbx ;
	nbxx3 = 3*nbx ;
	nbxx4 = 4*nbx ;
	nbxx5 = 5*nbx ;
	nbxp1 = nbx+1 ;
	nbxp2 = nbx+2 ;
	nbxp3 = nbx+3 ;
	nbxp4 = nbx+4 ;
	nbxp5 = nbx+5 ;

	buffer = (double *) calloc (nbseg+1 + nbx*(nbx + dimsp) ,sizeof(double)) ;

    if (buffer == NULL) return 2 ;

	Eps2 = buffer ;
	Mat = Eps2 + nbseg+1 ;
	Sec = Mat + nbx*nbx ;


	/* clis = 1/lambda, avec lambda coef critre (A-M)^2 
	   clis ramen de (0, 0.5, 1.)  (0, 1., infini) */
	clis = clis < 0.0001 ? 0.0001 : clis ;
	clis = clis > 0.99 ? 0.99 : clis ;
	clis = tan (clis*M_PI_2);

	for (k = 0 ; k <= nbseg; k++) Eps2[k] = clis*dM[k]*dM[k] ;

	/* Correction mul et muc pour range 0-1 et somme = 1 */
	muc = muc < 0. ? 0. : muc ;
	muc = muc > 1. ? 1. : muc ;
	mul = 1. - muc ;

	/* Correction mu pour produit lambda*mu = 1 */
	muc *= clis ;
	mul *= clis ;

	/* Correction muc pour homognit avec mul */
	d2tot = 0. ;
    for (m = 0 ; m < nbseg ; m++) d2tot += pUf[m]*pUf[m] ;
	muc *= d2tot/nbseg ;

    mc = dimY-1 ;	        /* -1 par rapport au debut de Q */
	mt = nbx*(dimY-1) ;	    /* -1 par rapport au debut de Qt */
	m = mc ; ml = mt ;
    r = 0 ;				    /* debut de R */
	e = (dimY+3)*nbxp1 ;	/* debut de E */

	for (k = 0; k < nbseg ; k++)
	{
		dk = pUf[k] ;
		dk2 = dk*dk ;
		dk3 = dk2*dk ;
		dk4 = dk3*dk ;
		dk5 = dk4*dk ; 
        dk6 = dk5*dk ;
		dk7 = dk6*dk ;
		dk77 = dk7/7 ;
		dk8 = dk7*dk;
		dk9 = dk8*dk;
		
		/* Elements de Q et Qt */
		Mat[mc+1] = -1 ; 
		Mat[mc+nbxp2] = -1 ;
		Mat[mc+nbxx2+3] = -1 ;
		Mat[mt+nbx] = -1 ;
		Mat[mt+nbxx2+1] = -1 ;
		Mat[mt+nbxx3+2] = -1 ;

		mc += 4 ; mt += nbxx4 ;
		m = mc ; ml = mt ;

		/* 1ere colonne Mk et 1ere ligne Mkt */
		Mat[m++] = Mat[ml] = dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1. ; ml +=nbx;
		Mat[m++] = Mat[ml] = 0. ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0. ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 2eme colonne Mk et 2eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk2   ; ml +=nbx;
		Mat[m++] = Mat[ml] = 2.*dk ; ml +=nbx;
		Mat[m++] = Mat[ml] = 1.    ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 0.    ;
		mc += nbx ; mt++ ; 
		m = mc ; ml = mt ;

		/* 3eme colonne Mk et 3eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk3    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk2 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 3.*dk  ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 1.     ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;
 
		/* 4eme colonne Mk et 4eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk4    ; ml +=nbx;
		Mat[m++] = Mat[ml] = 4.*dk3 ; ml +=nbx;
		Mat[m++] = Mat[ml] = 6.*dk2 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 4.*dk  ;
		mc += nbx ; mt++ ;
		m = mc ; ml = mt ;

		/* 5eme colonne Mk et 5eme ligne Mkt */
		Mat[m++] = Mat[ml] = dk5     ; ml +=nbx;
		Mat[m++] = Mat[ml] = 5.*dk4  ; ml +=nbx;
		Mat[m++] = Mat[ml] = 10.*dk3 ; ml +=nbx;
		Mat[m  ] = Mat[ml] = 10.*dk2 ; 

        if (k==0 || k==nbseg-1)
		{
			mc += nbx ; mt++ ;
			m = mc ; ml = mt ;

			/* 6eme colonne Mk et 6eme ligne Mkt */
			Mat[m++] = Mat[ml] = dk6     ; ml +=nbx;
			Mat[m++] = Mat[ml] = 6.*dk5  ; ml +=nbx;
			Mat[m++] = Mat[ml] = 15.*dk4 ; ml +=nbx;
			Mat[m  ] = Mat[ml] = 20.*dk3 ;
		}

		mc += nbx ; mt++ ;  /* Ready pour Q/Mk+1  et Qt/Mk+1t */

		/* Elements de Rk */
		Mat[r]         =                  mul*dk          ;
		
		Mat[r+1]       = Mat[r+nbx]     = mul*dk2         ;
		Mat[r+nbxp1]   =                  mul*4.*dk3/3.  + muc*dk ;

		Mat[r+2]       = Mat[r+nbxx2]   = mul*dk3         ;
		Mat[r+nbxp2]   = Mat[r+nbxx2+1] = mul*1.5*dk4    + muc*0.5*dk2;
		Mat[r+nbxx2+2] =                  mul*1.8*dk5    + muc*dk3/3 ;

		Mat[r+3]       = Mat[r+nbxx3]   = mul*dk4         ;
		Mat[r+nbxp3]   = Mat[r+nbxx3+1] = mul*1.6*dk5    + muc*2.*dk3 ;
		Mat[r+nbxx2+3] = Mat[r+nbxx3+2] = mul*2.*dk6     + muc*1.5*dk4 ;
		Mat[r+nbxx3+3] =                  mul*16.*dk77   + muc*7.2*dk5 ;

		Mat[r+4]       = Mat[r+nbxx4]   = mul*dk5         ;
		Mat[r+nbxp4]   = Mat[r+nbxx4+1] = mul*5.*dk6/3.  + muc*2.5*dk4 ;
		Mat[r+nbxx2+4] = Mat[r+nbxx4+2] = mul*15.*dk77   + muc*2.*dk5 ;
		Mat[r+nbxx3+4] = Mat[r+nbxx4+3] = mul*2.5*dk8    + muc*10.*dk6 ;
		Mat[r+nbxx4+4] =                  mul*25.*dk9/9. + muc*100.*dk77 ;

		if (k==0 || k==nbseg-1)
		{
			dk10 = dk9*dk ;
			dk11 = dk10*dk ;

			Mat[r+5]       = Mat[r+nbxx5]   = mul*dk6         ;
			Mat[r+nbxp5]   = Mat[r+nbxx5+1] = mul*12.*dk77   + muc*3.*dk5 ;
			Mat[r+nbxx2+5] = Mat[r+nbxx5+2] = mul*2.25*dk8   + muc*2.5*dk6 ;
			Mat[r+nbxx3+5] = Mat[r+nbxx5+3] = mul*8.*dk9/3.  + muc*90.*dk77 ;
			Mat[r+nbxx4+5] = Mat[r+nbxx5+4] = mul*3.*dk10    + muc*18.75*dk8 ;
			Mat[r+nbxx5+5] =                  mul*36.*dk11/11.+ muc*25.*dk9 ;

			r += nbxp1*6;		/*Ready pour Rk+1 */
		}
		else r += nbxp1*5;		/*Ready pour Rk+1 */

		/* Elements de E */
		Mat[e] = - (Eps2[k]+Eps2[k+1]) ;
		if (k) /* Pas pour k = 0 */
		{
			Mat[e-nbxx4] = Eps2[k] ;
			Mat[e-4] = Eps2[k] ;
		}

		e += 4*nbxp1 ;

	}

	/* Remplissage des 2eme membre */
	n = dimsp ;
	S = Sec ;
	while(n--)
	{
		/* Second membre */
		S += dimY ;
		*S++ = - B1[n] ;
		*S++ = - C1[n] ;
		*S++ = - D1[n] ;
		for (k = 0 ; k < nbseg-1 ; k++, S += 4)
		{
			*S = Coefs[k+1][n] - Coefs[k][n];
		}
		*S++ = Coefs[k+1][n] - Coefs[k][n] ;
		*S++ = BN1[n] ;
		*S++ = CN1[n] ;
		*S++ = DN1[n] ;	
	}

	resok = solv_lu(Sec, Mat, nbx, dimsp, 0) ;
	if (!resok) { free(buffer) ; return 3 ; }

//	resok = choll(Sec, Mat, nbx, dimsp) ;
//	if (resok < 0) { free(buffer) ; return 3 ; }

	n = dimsp ;
	Y = Sec ;
	while(n--)
	{
		S = Y + dimY-1 ;
		for (k = 0 ; k < nbseg ; k++)
		{
			Ak = Coefs[k] ; 
			Bk=Ak+dimsp; Ck=Bk+dimsp ; 
			Dk=Ck+dimsp ; Ek=Dk+dimsp ;
			Fk=Ek+dimsp ; 

		    if (k) Ak[n] += Eps2[k]*(S[0]-S[4]) ;
			else Ak[n] -= Eps2[k]*S[4] ;
			S += 4 ;
			Bk[n] = *Y++ ;
			Ck[n] = *Y++ ;
			Dk[n] = *Y++ ;
			Ek[n] = *Y++ ;
			Fk[n] = *Y++ ;
			if (k==0 || k==nbseg-1)
			{
				Gk=Fk+dimsp ;
				Gk[n] = *Y++ ;
			}
		}
		AN1[n] += Eps2[k]*S[0] ;  /* Pour les tracs */
		Y += dimS ;
	}

    free(buffer) ; 
	return 0 ;
}

/* Prototypes */
int lufactor (double *a, int lda, int n, int *ipvt) ;
int lusolv (double *a, int lda, int n, int *ipvt, double *b, int trp) ;
int idx_maxabselm (double *dx, int n) ;
void cum1_vect (double *y, double a, double *x, int n) ;
void fact1_vect (double *x, double a, int n) ;
double muls_vect (double* u, double* v, int n) ;


/*-------------------------------------------------------------------------*/
int solv_lu (double *XY, double *A, int n, int ns, int trp)
{
/*
    !!!!!!!!!!    ATTENTION la matrice A est detruite       !!!!!!!!!!
				       --------------
    Resolution de Y = A  * X :   X = inv(A)  * Y    si trp = 0
	       de Y = At * X :   X = inv(At) * Y    si trp = 1

    A  : matrice n x n
    XY : ns vecteurs de dimension n

    En entree XY contient les ns vecteurs Y et en sortie les ns vecteurs X

    Renvoi : 1 si OK
	     0 si resolution impossible
*/
int flag, *ipvt = (int *) malloc (n * sizeof(int) ) ;

    if (ipvt == NULL)
    { puts ("Echec allocation memoire dans solv_lu") ; exit(1) ; }

    if ( lufactor (A, n, n, ipvt) == 0)
    {
	flag = 1 ;
	while (ns--) lusolv (A, n, n, ipvt, &XY[n*ns], trp) ;
    }
    else flag = 0 ;

    free(ipvt) ;
    return flag ;
}
/*-------------------------------------------------------------------------*/
int lufactor (double *a, int lda, int n, int *ipvt)
{
/*
     lufactor effectue la factorisation L U d'une matrice n x n
     extraite d'une matrice rangees par vecteurs de lda composantes.
     Les facteurs sont renvoyes dans A et les indices des pivots
     des lignes dans ipvt.
     Renvoi 0 si OK ou l'indice fortran du 1er element nul de la diag
*/
double t ;
int info, j, k, l, nm1, lk, kk, lj, kj, klda;
int lda1 = 1 + lda ;

/*  Elimination de Gauss avec pivotage partiel */

    info = 0 ;
    nm1 = n - 1 ;
    kk = 0 ;
    if (nm1 >= 1)
    {
	for (k = 0 , klda = 0 , kk = 0 ; k < nm1 ;
					 k++ , klda += lda, kk += lda1)
	{
	/*  Recherche l = indice du pivot */
	    l = idx_maxabselm (&a[kk], n-k) + k ;
	    ipvt[k] = l ;
	/* Si 0 la colonne est deja triangularisee */
	    lk = l + klda ;
	    if ( a[lk] != 0.)
	    {
		if (l != k) { t = a[lk] ; a[lk] = a[kk] ; a[kk] = t ; }
	    /* Calcul des multiplicateurs */
		t = -1./a[kk] ;
		fact1_vect (&a[kk+1], t, nm1-k) ;
	    /* Elimination des colonnes */
		lj = lk ; kj = kk ;
		for (j = k+1 ; j < n ; j++)
		{
		    lj += lda ; kj += lda ; t= a[lj] ;
		    if (l != k) { a[lj] = a[kj] ; a[kj] = t ; }
		    cum1_vect (&a[kj+1], t, &a[kk+1], nm1-k) ;
		}
	    }
	    else info = k+1 ;
	}
    }
    ipvt[nm1] = nm1 ;
    kk = nm1*lda1 ;
    if ( a[kk] == 0.) info = n ;
    return info ;
}
/*-------------------------------------------------------------------------*/
int lusolv (double *a, int lda, int n, int *ipvt, double *b, int trp)
{
/*
      Renvoi 1 si Ok et 0 si tentative division par 0 (calcul non fait)
*/
double t, diag ;
int k, kp1, klda, l, nm1 ;

    nm1 = n - 1 ;
    if (trp == 0 )
    {

		if (nm1 > 0)
        {
            for (k = 0 , klda = 0 ; k < nm1 ; k++, klda += lda )
            {
                kp1 = k + 1 ;
                l = ipvt[k] ; t = b[l] ;
				if (l != k) { b[l] = b[k] ; b[k] = t ; }
				cum1_vect (&b[kp1], t, &a[klda+kp1], n-k-1) ;
            }
        }

        for (k = nm1 , klda = nm1 * lda ; k >= 0 ; k--, klda -= lda)
	    {
			if ( (diag = a[k+klda]) == 0.) return 0 ;
            b[k] /= diag ;
			t = - b[k] ;
			cum1_vect (b, t , &a[klda], k) ;
        }
    }
    else
    {
        for (k = 0, klda = 0 ; k < n ; k++, klda += lda)
        {
			t = muls_vect (&a[klda], b, k) ;
            if ( (diag = a[k+klda]) == 0.) return 0 ;
            b[k] = ( b[k] - t ) / diag ;
        }
		if (nm1 > 0)
		{
			for (k = n-2, klda = k * lda ; k >= 0 ; k-- , klda -= lda)
			{
				kp1 = k + 1 ;
				b[k] += muls_vect (&a[klda+kp1], &b[kp1], nm1 - k) ;
				l = ipvt[k] ;
				if (l != k) {t = b[l]; b[l] = b[k] ; b[k] = t ; }
			}
		}
    }
    return 1 ;
}

/*-------------------------------------------------------------------------*/
int idx_maxabselm (double *dx, int n)
{
/*     trouve l'index du + grand element en valeur absolue */
int i, id ;
double dmax, a ;
double fabs(double) ;

    if (n <= 1) return (n-1) ;
    dmax = fabs(dx[0]) ; id = 0 ;
    for (i = 1 ; i < n ; i++)
    { a = fabs(dx[i]) ; if (a > dmax) {id = i ; dmax = a ; } }
    return id ;
}

/*-------------------------------------------------------------------------*/
void cum1_vect (double *y, double a, double *x, int n)
{
/* Cumulation de vecteurs */
    if (n > 0 && a != 0.) while (n--) y[n] += a * x[n] ;
}

/*-------------------------------------------------------------------------*/
void fact1_vect (double *x, double a, int n)
{ /* multiplie un vecteur par une constante */

    if (n > 0) while (n--) x[n] = a * x[n] ;
}

/*-------------------------------------------------------------------------*/
double muls_vect (double* u, double* v, int n)
{ /*    Produit scalaire de 2 VECT_N :  s = muls_vect(u,v,n) */
double s ;
    s = 0. ;
    if (n > 0) while (n--) s += u[n] * v[n] ;
    return(s);
}
